%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This module contains all information for the compiler about
% external functions by facts for the predicate externalSpec.
% Each fact must have the form:
% 
% externalSpec(Name,Evals,Module).
%
% Name:   name of the external function (as it occurs in FlatCurry)
% Module: module to be loaded (relative to this directory!) in order to
%         execute this function (usually, contains the external implementation)
% Evals:  specification of the evaluation degree of the arguments
%         for external functions, where:
%         hnf:  evaluate this argument to head normal form
%         nf:   evaluate this argument to normal form
%         gnf:  evaluate this argument to ground normal form
%         none: do not evaluate this argument
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Standard prelude:
externalSpec((+),[hnf,hnf],prim_standard).
externalSpec((-),[hnf,hnf],prim_standard).
externalSpec((*),[hnf,hnf],prim_standard).
externalSpec((div),[hnf,hnf],prim_standard).
externalSpec((mod),[hnf,hnf],prim_standard).
externalSpec((<),[hnf,hnf],prim_standard).
externalSpec((>),[hnf,hnf],prim_standard).
externalSpec((<=),[hnf,hnf],prim_standard).
externalSpec((>=),[hnf,hnf],prim_standard).
externalSpec(ord,[hnf],prim_standard).
externalSpec(chr,[hnf],prim_standard).
externalSpec((>>=),[hnf,none,none],prim_standard).
	% although (>>) is a defined function,
	% we provide a slightly faster implementation:
externalSpec((>>),[hnf,none,none],prim_standard).
externalSpec(putChar,[hnf,none],prim_standard).
externalSpec(getChar,[none],prim_standard).
externalSpec(return,[none,none],prim_standard).
externalSpec(readFile,[gnf,none],prim_standard).
externalSpec(readFileContents,[none,none],prim_standard).
externalSpec(writeFile,[gnf,none,none],prim_standard).
externalSpec(appendFile,[gnf,none,none],prim_standard).
externalSpec(writeFileContents,[none,hnf,none],prim_standard).
externalSpec(catchFail,[none,none,none],prim_standard).
externalSpec(findall,[hnf],prim_standard).
externalSpec(findfirst,[hnf],prim_standard).
externalSpec(show,[gnf],prim_standard).
externalSpec(seq,[hnf,none],prim_standard).
externalSpec(error,[gnf],prim_standard).
externalSpec((==),[none,none],prim_standard).
externalSpec((=:=),[none,none],prim_standard).
externalSpec((&),[none,none],prim_standard).
externalSpec(apply,[hnf,none],prim_standard).
externalSpec(cond,[hnf,none],prim_standard).
externalSpec(commit,[none],prim_standard).
externalSpec(try,[none],prim_standard).

% Module AllSolutions:
externalSpec('AllSolutions.getFirstSolution',[hnf,none],prim_standard).

% Module CLPR:
externalSpec('CLPR.+.',[hnf,hnf],prim_clpr).
externalSpec('CLPR.-.',[hnf,hnf],prim_clpr).
externalSpec('CLPR.*.',[hnf,hnf],prim_clpr).
externalSpec('CLPR./.',[hnf,hnf],prim_clpr).
externalSpec('CLPR.<.',[hnf,hnf],prim_clpr).
externalSpec('CLPR.>.',[hnf,hnf],prim_clpr).
externalSpec('CLPR.<=.',[hnf,hnf],prim_clpr).
externalSpec('CLPR.>=.',[hnf,hnf],prim_clpr).
externalSpec('CLPR.i2f',[hnf],prim_clpr).
externalSpec('CLPR.minimumFor',[hnf,hnf],prim_clpr).
externalSpec('CLPR.maximumFor',[hnf,hnf],prim_clpr).

% Module CLPFD:
externalSpec('CLPFD.+#',[hnf,hnf],prim_clpfd).
externalSpec('CLPFD.-#',[hnf,hnf],prim_clpfd).
externalSpec('CLPFD.*#',[hnf,hnf],prim_clpfd).
externalSpec('CLPFD.=#',[hnf,hnf],prim_clpfd).
externalSpec('CLPFD./=#',[hnf,hnf],prim_clpfd).
externalSpec('CLPFD.<#',[hnf,hnf],prim_clpfd).
externalSpec('CLPFD.>#',[hnf,hnf],prim_clpfd).
externalSpec('CLPFD.<=#',[hnf,hnf],prim_clpfd).
externalSpec('CLPFD.>=#',[hnf,hnf],prim_clpfd).
externalSpec('CLPFD.domain',[nf,hnf,hnf],prim_clpfd).
externalSpec('CLPFD.all_different',[nf],prim_clpfd).
externalSpec('CLPFD.labeling',[nf],prim_clpfd).

% Module Directory:
externalSpec('Directory.doesFileExist',[gnf,none],prim_directory).
externalSpec('Directory.doesDirectoryExist',[gnf,none],prim_directory).
externalSpec('Directory.fileSize',[gnf,none],prim_directory).
externalSpec('Directory.getModificationTime',[gnf,none],prim_directory).
externalSpec('Directory.getDirectoryContents',[gnf,none],prim_directory).

% Module Dynamic:
externalSpec('Dynamic.assertFact',[gnf,none],prim_dynamic).
externalSpec('Dynamic.retractFact',[gnf,none],prim_dynamic).
externalSpec('Dynamic.getDynamicKnowledge',[none],prim_dynamic).
externalSpec('Dynamic.isKnownAtTime',[hnf,nf],prim_dynamic).
externalSpec('Dynamic.startTransaction',[none],prim_dynamic).
externalSpec('Dynamic.commitTransaction',[hnf,none],prim_dynamic).
externalSpec('Dynamic.abortTransaction',[none],prim_dynamic).

% Module Flat:
externalSpec('Flat.readFlatCurryFile',[gnf,none],prim_flat).

% Module Float:
externalSpec('Float.+.',[hnf,hnf],prim_float).
externalSpec('Float.-.',[hnf,hnf],prim_float).
externalSpec('Float.*.',[hnf,hnf],prim_float).
externalSpec('Float./.',[hnf,hnf],prim_float).
externalSpec('Float.<.',[hnf,hnf],prim_float).
externalSpec('Float.>.',[hnf,hnf],prim_float).
externalSpec('Float.<=.',[hnf,hnf],prim_float).
externalSpec('Float.>=.',[hnf,hnf],prim_float).
externalSpec('Float.i2f',[hnf],prim_float).
externalSpec('Float.truncate',[hnf],prim_float).
externalSpec('Float.round',[hnf],prim_float).
externalSpec('Float.sqrt',[hnf],prim_float).
externalSpec('Float.log',[hnf],prim_float).
externalSpec('Float.exp',[hnf],prim_float).
externalSpec('Float.sin',[hnf],prim_float).
externalSpec('Float.cos',[hnf],prim_float).
externalSpec('Float.tan',[hnf],prim_float).

% Module IO:
externalSpec('IO.stdin',[],prim_io).
externalSpec('IO.stdout',[],prim_io).
externalSpec('IO.stderr',[],prim_io).
externalSpec('IO.openFile',[gnf,gnf,none],prim_io).
externalSpec('IO.hClose',[nf,none],prim_io).
externalSpec('IO.hFlush',[nf,none],prim_io).
externalSpec('IO.hIsEOF',[nf,none],prim_io).
externalSpec('IO.hGetChar',[nf,none],prim_io).
externalSpec('IO.hPutChar',[nf,hnf,none],prim_io).
externalSpec('IO.hIsReadable',[nf,none],prim_io).
externalSpec('IO.hIsWritable',[nf,none],prim_io).

% Module IOExts:
externalSpec('IOExts.execCmd',[gnf,none],prim_ioexts).
externalSpec('IOExts.connectToSocket',[gnf,gnf,none],prim_ioexts).
externalSpec('IOExts.setAssoc',[gnf,gnf,none],prim_ioexts).
externalSpec('IOExts.getAssoc',[gnf,none],prim_ioexts).

% Module PlProfileData:
externalSpec('PlProfileData.profileReset',[none],prim_plprofiledata).
externalSpec('PlProfileData.profilePredicates',[hnf,none],prim_plprofiledata).
externalSpec('PlProfileData.profileClauses',[hnf,gnf,none],prim_plprofiledata).
externalSpec('PlProfileData.profileView',[none],prim_plprofiledata).
externalSpec('PlProfileData.getHnfDefinitions',[none],prim_plprofiledata).

% Module Ports:
externalSpec('Ports.openPort',[nf,nf],prim_ports).
externalSpec('Ports.basicServerLoop',[none],prim_ports).
externalSpec('Ports.send',[nf,hnf],prim_ports).
externalSpec('Ports.openStdIn',[none],prim_ports).
externalSpec('Ports.after',[hnf],prim_ports).
externalSpec('Ports.openProcessPort',[gnf,none],prim_ports).
externalSpec('Ports.openSocketConnectPort',[gnf,gnf,none],prim_ports).
externalSpec('Ports.ping',[gnf,hnf,none],prim_ports).
externalSpec('Ports.timeoutOnStream',[gnf,none],prim_ports).
externalSpec('Ports.openPortOnSocket',[nf,nf,none],prim_ports).
externalSpec('Ports.connectPortAtSocket',[gnf,gnf,gnf,none],prim_ports).
externalSpec('Ports.choiceSPEP',[hnf,none],prim_ports).

% Module ReadShowTerm:
externalSpec('ReadShowTerm.showQTerm',[gnf],prim_readshowterm).
externalSpec('ReadShowTerm.showTerm',[gnf],prim_readshowterm).
externalSpec('ReadShowTerm.readsQTerm',[gnf],prim_readshowterm).
externalSpec('ReadShowTerm.readsTerm',[gnf],prim_readshowterm).

% Module System:
externalSpec('System.getCPUTime',[none],prim_system).
externalSpec('System.getElapsedTime',[none],prim_system).
externalSpec('System.getDate',[none],prim_system).
externalSpec('System.getArgs',[none],prim_system).
externalSpec('System.getEnviron',[gnf,none],prim_system).
externalSpec('System.getHostname',[none],prim_system).
externalSpec('System.getPID',[none],prim_system).
externalSpec('System.getProgName',[none],prim_system).
externalSpec('System.system',[gnf,none],prim_system).
externalSpec('System.sleep',[gnf,none],prim_system).
externalSpec('System.evalTime',[none],prim_system).
externalSpec('System.evalSpace',[none],prim_system).

% Module TermDB:
externalSpec('TermDB.openDB',[gnf,none],prim_termdb).
externalSpec('TermDB.closeDB',[nf,none],prim_termdb).
externalSpec('TermDB.storeDB',[gnf,nf,none],prim_termdb).
externalSpec('TermDB.eraseDB',[nf,nf,none],prim_termdb).
externalSpec('TermDB.findallDB',[nf,nf,none],prim_termdb).

% Module Time:
externalSpec('Time.getClockTime',[none],prim_time).

% Module Unsafe:
externalSpec('Unsafe.unsafePerformIO',[hnf],prim_unsafe).
externalSpec('Unsafe.isVar',[hnf],prim_unsafe).
externalSpec('Unsafe.spawnConstraint',[none,none],prim_unsafe).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
