%PAKCS1.6V2

:-prolog_flag(single_var_warnings,A,off).
:-prolog_flag(redefine_warnings,A,off).

modulename(prelude).

opFixity(seq,0,right).
opFixity(==,4,non).
opFixity(+,6,left).
opFixity(-,6,left).
opFixity(*,7,left).
opFixity(div,7,left).
opFixity(mod,7,left).
opFixity(<,4,non).
opFixity(>,4,non).
opFixity(<=,4,non).
opFixity(>=,4,non).
opFixity(=:=,4,non).
opFixity(&,0,right).
opFixity(>>=,1,left).
opFixity('.',9,right).
opFixity($,0,right).
opFixity('$!',0,right).
opFixity(&&,3,right).
opFixity('||',2,right).
opFixity(/=,4,non).
opFixity(++,5,right).
opFixity('!!',9,left).
opFixity(elem,4,non).
opFixity(notElem,4,non).
opFixity(&>,0,right).
opFixity(>>,1,left).
opFixity(?,0,right).

:- block 'prelude..'(?,?,?,?,-,?).
'prelude..'(A,B,C,D,E,F):-hnf(apply(prelude_lambda0_0(A,B),C),D,E,F).

:- block id(?,?,-,?).
id(A,B,C,D):-hnf(A,B,C,D).

:- block const(?,?,?,-,?).
const(A,B,C,D,E):-hnf(A,C,D,E).

:- block curry(?,?,?,?,-,?).
curry(A,B,C,D,E,F):-hnf(apply(A,'(,)'(B,C)),D,E,F).

:- block uncurry(?,?,?,-,?).
uncurry(A,B,C,D,E):-hnf(B,F,D,G),uncurry_2(F,A,C,G,E).

:- block uncurry_2(?,?,?,-,?).
uncurry_2('(,)'(A,B),C,D,E,F):-hnf(apply(apply(C,A),B),D,E,F).

:- block flip(?,?,?,?,-,?).
flip(A,B,C,D,E,F):-hnf(apply(apply(A,C),B),D,E,F).

:- block until(?,?,?,?,-,?).
until(A,B,C,D,E,F):-makeShare(A,G),makeShare(C,H),makeShare(B,I),if_then_else(apply(G,H),H,until(G,I,apply(I,H)),D,E,F).

:- block $(?,?,?,-,?).
$(A,B,C,D,E):-hnf(apply(A,B),C,D,E).

:- block '$!'(?,?,?,-,?).
'$!'(A,B,C,D,E):-makeShare(B,F),hnf(seq(F,apply(A,F)),C,D,E).

:- block failed(?,-,?).
failed(A,B,C):-hnf(cond(1=:=2,D),A,B,C).

:- block &&(?,?,?,-,?).
&&(A,B,C,D,E):-hnf(A,F,D,G),'&&_1'(F,B,C,G,E).

:- block '&&_1'(?,?,?,-,?).
'&&_1'('True',A,B,C,D):-hnf(A,B,C,D).
'&&_1'('False',A,'False',B,B).

:- block '||'(?,?,?,-,?).
'||'(A,B,C,D,E):-hnf(A,F,D,G),'||_1'(F,B,C,G,E).

:- block '||_1'(?,?,?,-,?).
'||_1'('True',A,'True',B,B).
'||_1'('False',A,B,C,D):-hnf(A,B,C,D).

:- block not(?,?,-,?).
not(A,B,C,D):-hnf(A,E,C,F),not_1(E,B,F,D).

:- block not_1(?,?,-,?).
not_1('True','False',A,A).
not_1('False','True',A,A).

:- block otherwise(?,-,?).
otherwise('True',A,A).

:- block if_then_else(?,?,?,?,-,?).
if_then_else(A,B,C,D,E,F):-hnf(A,G,E,H),if_then_else_1(G,B,C,D,H,F).

:- block if_then_else_1(-,?,?,?,?,?),if_then_else_1(?,?,?,?,-,?).
if_then_else_1('True',A,B,C,D,E):-hnf(A,C,D,E).
if_then_else_1('False',A,B,C,D,E):-hnf(B,C,D,E).

:- block /=(?,?,?,-,?).
/=(A,B,C,D,E):-not(A==B,C,D,E).

:- block fst(?,?,-,?).
fst(A,B,C,D):-hnf(A,E,C,F),fst_1(E,B,F,D).

:- block fst_1(?,?,-,?).
fst_1('(,)'(A,B),C,D,E):-hnf(A,C,D,E).

:- block snd(?,?,-,?).
snd(A,B,C,D):-hnf(A,E,C,F),snd_1(E,B,F,D).

:- block snd_1(?,?,-,?).
snd_1('(,)'(A,B),C,D,E):-hnf(B,C,D,E).

:- block head(?,?,-,?).
head(A,B,C,D):-hnf(A,E,C,F),head_1(E,B,F,D).

:- block head_1(?,?,-,?).
head_1([A|B],C,D,E):-hnf(A,C,D,E).

:- block tail(?,?,-,?).
tail(A,B,C,D):-hnf(A,E,C,F),tail_1(E,B,F,D).

:- block tail_1(?,?,-,?).
tail_1([A|B],C,D,E):-hnf(B,C,D,E).

:- block null(?,?,-,?).
null(A,B,C,D):-hnf(A,E,C,F),null_1(E,B,F,D).

:- block null_1(?,?,-,?).
null_1([],'True',A,A).
null_1([A|B],'False',C,C).

:- block ++(?,?,?,-,?).
++(A,B,C,D,E):-hnf(A,F,D,G),'++_1'(F,B,C,G,E).

:- block '++_1'(?,?,?,-,?).
'++_1'([],A,B,C,D):-hnf(A,B,C,D).
'++_1'([A|B],C,[A|++(B,C)],D,D).

:- block length(?,?,-,?).
length(A,B,C,D):-hnf(A,E,C,F),length_1(E,B,F,D).

:- block length_1(?,?,-,?).
length_1([],0,A,A).
length_1([A|B],C,D,E):-hnf(1+length(B),C,D,E).

:- block '!!'(?,?,?,-,?).
'!!'(A,B,C,D,E):-hnf(A,F,D,G),'!!_1'(F,B,C,G,E).

:- block '!!_1'(?,?,?,-,?).
'!!_1'([A|B],C,D,E,F):-makeShare(C,G),if_then_else(G==0,A,if_then_else(G>0,'!!'(B,G-1),failed),D,E,F).

:- block map(?,?,?,-,?).
map(A,B,C,D,E):-hnf(B,F,D,G),map_2(F,A,C,G,E).

:- block map_2(?,?,?,-,?).
map_2([],A,[],B,B).
map_2([A|B],C,[apply(D,A)|map(D,B)],E,F):-makeShare(C,D),E=F.

:- block foldl(?,?,?,?,-,?).
foldl(A,B,C,D,E,F):-hnf(C,G,E,H),foldl_3(G,A,B,D,H,F).

:- block foldl_3(?,?,?,?,-,?).
foldl_3([],A,B,C,D,E):-hnf(B,C,D,E).
foldl_3([A|B],C,D,E,F,G):-makeShare(C,H),foldl(H,apply(apply(H,D),A),B,E,F,G).

:- block foldl1(?,?,?,-,?).
foldl1(A,B,C,D,E):-hnf(B,F,D,G),foldl1_2(F,A,C,G,E).

:- block foldl1_2(?,?,?,-,?).
foldl1_2([A|B],C,D,E,F):-foldl(C,A,B,D,E,F).

:- block foldr(?,?,?,?,-,?).
foldr(A,B,C,D,E,F):-hnf(C,G,E,H),foldr_3(G,A,B,D,H,F).

:- block foldr_3(?,?,?,?,-,?).
foldr_3([],A,B,C,D,E):-hnf(B,C,D,E).
foldr_3([A|B],C,D,E,F,G):-makeShare(C,H),hnf(apply(apply(H,A),foldr(H,D,B)),E,F,G).

:- block foldr1(?,?,?,-,?).
foldr1(A,B,C,D,E):-hnf(B,F,D,G),foldr1_2(F,A,C,G,E).

:- block foldr1_2(?,?,?,-,?).
foldr1_2([A|B],C,D,E,F):-hnf(B,G,E,H),'foldr1_2_._2'(G,A,C,D,H,F).

:- block 'foldr1_2_._2'(?,?,?,?,-,?).
'foldr1_2_._2'([],A,B,C,D,E):-hnf(A,C,D,E).
'foldr1_2_._2'([A|B],C,D,E,F,G):-makeShare(D,H),hnf(apply(apply(H,C),foldr1(H,[A|B])),E,F,G).

:- block filter(?,?,?,-,?).
filter(A,B,C,D,E):-hnf(B,F,D,G),filter_2(F,A,C,G,E).

:- block filter_2(?,?,?,-,?).
filter_2([],A,[],B,B).
filter_2([A|B],C,D,E,F):-makeShare(C,G),makeShare(A,H),makeShare(B,I),if_then_else(apply(G,H),[H|filter(G,I)],filter(G,I),D,E,F).

:- block zip(?,?,?,-,?).
zip(A,B,C,D,E):-hnf(A,F,D,G),zip_1(F,B,C,G,E).

:- block zip_1(?,?,?,-,?).
zip_1([],A,[],B,B).
zip_1([A|B],C,D,E,F):-hnf(C,G,E,H),'zip_1_._3'(G,A,B,D,H,F).

:- block 'zip_1_._3'(?,?,?,?,-,?).
'zip_1_._3'([],A,B,[],C,C).
'zip_1_._3'([A|B],C,D,['(,)'(C,A)|zip(D,B)],E,E).

:- block zip3(?,?,?,?,-,?).
zip3(A,B,C,D,E,F):-hnf(A,G,E,H),zip3_1(G,B,C,D,H,F).

:- block zip3_1(?,?,?,?,-,?).
zip3_1([],A,B,[],C,C).
zip3_1([A|B],C,D,E,F,G):-hnf(C,H,F,I),'zip3_1_._3'(H,A,B,D,E,I,G).

:- block 'zip3_1_._3'(?,?,?,?,?,-,?).
'zip3_1_._3'([],A,B,C,[],D,D).
'zip3_1_._3'([A|B],C,D,E,F,G,H):-hnf(E,I,G,J),'zip3_1_._3_._5'(I,A,B,C,D,F,J,H).

:- block 'zip3_1_._3_._5'(?,?,?,?,?,?,-,?).
'zip3_1_._3_._5'([],A,B,C,D,[],E,E).
'zip3_1_._3_._5'([A|B],C,D,E,F,['(,,)'(E,C,A)|zip3(F,D,B)],G,G).

:- block zipWith(?,?,?,?,-,?).
zipWith(A,B,C,D,E,F):-hnf(B,G,E,H),zipWith_2(G,A,C,D,H,F).

:- block zipWith_2(?,?,?,?,-,?).
zipWith_2([],A,B,[],C,C).
zipWith_2([A|B],C,D,E,F,G):-hnf(D,H,F,I),'zipWith_2_._4'(H,A,B,C,E,I,G).

:- block 'zipWith_2_._4'(?,?,?,?,?,-,?).
'zipWith_2_._4'([],A,B,C,[],D,D).
'zipWith_2_._4'([A|B],C,D,E,[apply(apply(F,C),A)|zipWith(F,D,B)],G,H):-makeShare(E,F),G=H.

:- block zipWith3(?,?,?,?,?,-,?).
zipWith3(A,B,C,D,E,F,G):-hnf(B,H,F,I),zipWith3_2(H,A,C,D,E,I,G).

:- block zipWith3_2(?,?,?,?,?,-,?).
zipWith3_2([],A,B,C,[],D,D).
zipWith3_2([A|B],C,D,E,F,G,H):-hnf(D,I,G,J),'zipWith3_2_._4'(I,A,B,C,E,F,J,H).

:- block 'zipWith3_2_._4'(?,?,?,?,?,?,-,?).
'zipWith3_2_._4'([],A,B,C,D,[],E,E).
'zipWith3_2_._4'([A|B],C,D,E,F,G,H,I):-hnf(F,J,H,K),'zipWith3_2_._4_._6'(J,A,B,C,D,E,G,K,I).

:- block 'zipWith3_2_._4_._6'(?,?,?,?,?,?,?,-,?).
'zipWith3_2_._4_._6'([],A,B,C,D,E,[],F,F).
'zipWith3_2_._4_._6'([A|B],C,D,E,F,G,[apply(apply(apply(H,E),C),A)|zipWith3(H,F,D,B)],I,J):-makeShare(G,H),I=J.

:- block unzip(?,?,-,?).
unzip(A,B,C,D):-hnf(A,E,C,F),unzip_1(E,B,F,D).

:- block unzip_1(?,?,-,?).
unzip_1([],'(,)'([],[]),A,A).
unzip_1([A|B],C,D,E):-hnf(A,F,D,G),'unzip_1_._1'(F,B,C,G,E).

:- block 'unzip_1_._1'(?,?,?,-,?).
'unzip_1_._1'('(,)'(A,B),C,D,E,F):-prelude_lambda1_0(A,B,unzip(C),D,E,F).

:- block unzip3(?,?,-,?).
unzip3(A,B,C,D):-hnf(A,E,C,F),unzip3_1(E,B,F,D).

:- block unzip3_1(?,?,-,?).
unzip3_1([],'(,,)'([],[],[]),A,A).
unzip3_1([A|B],C,D,E):-hnf(A,F,D,G),'unzip3_1_._1'(F,B,C,G,E).

:- block 'unzip3_1_._1'(?,?,?,-,?).
'unzip3_1_._1'('(,,)'(A,B,C),D,E,F,G):-prelude_lambda2_0(A,B,C,unzip3(D),E,F,G).

:- block concat(?,?,-,?).
concat(A,B,C,D):-foldr(++,[],A,B,C,D).

:- block concatMap(?,?,?,-,?).
concatMap(A,B,C,D,E):-hnf(apply('prelude..'(concat,map(A)),B),C,D,E).

:- block iterate(?,?,?,-,?).
iterate(A,B,[C|iterate(D,apply(D,C))],E,F):-makeShare(B,C),makeShare(A,D),E=F.

:- block repeat(?,?,-,?).
repeat(A,[B|repeat(B)],C,D):-makeShare(A,B),C=D.

:- block replicate(?,?,?,-,?).
replicate(A,B,C,D,E):-take(A,repeat(B),C,D,E).

:- block take(?,?,?,-,?).
take(A,B,C,D,E):-makeShare(A,F),if_then_else(<=(F,0),[],takep_0(F,B),C,D,E).

:- block drop(?,?,?,-,?).
drop(A,B,C,D,E):-makeShare(A,F),makeShare(B,G),if_then_else(<=(F,0),G,dropp_0(F,G),C,D,E).

:- block splitAt(?,?,?,-,?).
splitAt(A,B,C,D,E):-makeShare(A,F),makeShare(B,G),if_then_else(<=(F,0),'(,)'([],G),splitAtp_0(F,G),C,D,E).

:- block takeWhile(?,?,?,-,?).
takeWhile(A,B,C,D,E):-hnf(B,F,D,G),takeWhile_2(F,A,C,G,E).

:- block takeWhile_2(?,?,?,-,?).
takeWhile_2([],A,[],B,B).
takeWhile_2([A|B],C,D,E,F):-makeShare(C,G),makeShare(A,H),if_then_else(apply(G,H),[H|takeWhile(G,B)],[],D,E,F).

:- block dropWhile(?,?,?,-,?).
dropWhile(A,B,C,D,E):-hnf(B,F,D,G),dropWhile_2(F,A,C,G,E).

:- block dropWhile_2(?,?,?,-,?).
dropWhile_2([],A,[],B,B).
dropWhile_2([A|B],C,D,E,F):-makeShare(C,G),makeShare(A,H),makeShare(B,I),if_then_else(apply(G,H),dropWhile(G,I),[H|I],D,E,F).

:- block span(?,?,?,-,?).
span(A,B,C,D,E):-hnf(B,F,D,G),span_2(F,A,C,G,E).

:- block span_2(?,?,?,-,?).
span_2([],A,'(,)'([],[]),B,B).
span_2([A|B],C,D,E,F):-makeShare(C,G),makeShare(A,H),makeShare(B,I),if_then_else(apply(G,H),prelude_lambda4_0(H,span(G,I)),'(,)'([],[H|I]),D,E,F).

:- block break(?,?,?,-,?).
break(A,B,C,D,E):-hnf(apply(span('prelude..'(not,A)),B),C,D,E).

:- block lines(?,?,-,?).
lines(A,B,C,D):-hnf(A,E,C,F),lines_1(E,B,F,D).

:- block lines_1(?,?,-,?).
lines_1([],[],A,A).
lines_1([A|B],C,D,E):-prelude_lambda6_0(splitline_0([A|B]),C,D,E).

:- block unlines(?,?,-,?).
unlines(A,B,C,D):-hnf(apply(concatMap(prelude_lambda7_0),A),B,C,D).

:- block words(?,?,-,?).
words(A,B,C,D):-prelude_lambda8_0(dropWhile(isSpace_0,A),B,C,D).

:- block unwords(?,?,-,?).
unwords(A,B,C,D):-makeShare(A,E),if_then_else(E==[],[],foldr1(prelude_lambda10_0,E),B,C,D).

:- block reverse(?,?,-,?).
reverse(A,B,C,D):-hnf(apply(foldl(flip('.'),[]),A),B,C,D).

:- block and(?,?,-,?).
and(A,B,C,D):-hnf(apply(foldr(&&,'True'),A),B,C,D).

:- block or(?,?,-,?).
or(A,B,C,D):-hnf(apply(foldr('||','False'),A),B,C,D).

:- block any(?,?,?,-,?).
any(A,B,C,D,E):-hnf(apply('prelude..'(or,map(A)),B),C,D,E).

:- block all(?,?,?,-,?).
all(A,B,C,D,E):-hnf(apply('prelude..'(and,map(A)),B),C,D,E).

:- block elem(?,?,?,-,?).
elem(A,B,C,D,E):-hnf(apply(any(prelude_lambda11_0(A)),B),C,D,E).

:- block notElem(?,?,?,-,?).
notElem(A,B,C,D,E):-hnf(apply(all(prelude_lambda12_0(A)),B),C,D,E).

:- block lookup(?,?,?,-,?).
lookup(A,B,C,D,E):-hnf(B,F,D,G),lookup_2(F,A,C,G,E).

:- block lookup_2(?,?,?,-,?).
lookup_2([],A,'Nothing',B,B).
lookup_2([A|B],C,D,E,F):-hnf(A,G,E,H),'lookup_2_._1'(G,B,C,D,H,F).

:- block 'lookup_2_._1'(?,?,?,?,-,?).
'lookup_2_._1'('(,)'(A,B),C,D,E,F,G):-makeShare(D,H),if_then_else(H==A,'Just'(B),lookup(H,C),E,F,G).

:- block enumFrom(?,?,-,?).
enumFrom(A,[B|enumFrom(B+1)],C,D):-makeShare(A,B),C=D.

:- block enumFromThen(?,?,?,-,?).
enumFromThen(A,B,C,D,E):-makeShare(A,F),iterate(prelude_lambda13_0(B,F),F,C,D,E).

:- block enumFromTo(?,?,?,-,?).
enumFromTo(A,B,C,D,E):-makeShare(A,F),makeShare(B,G),if_then_else(F>G,[],[F|enumFromTo(F+1,G)],C,D,E).

:- block enumFromThenTo(?,?,?,?,-,?).
enumFromThenTo(A,B,C,D,E,F):-makeShare(B,G),makeShare(A,H),takeWhile(p_0(G,H,C),enumFromThen(H,G),D,E,F).

:- block negate(?,?,-,?).
negate(A,B,C,D):-hnf(0-A,B,C,D).

:- block &>(?,?,?,-,?).
&>(A,B,C,D,E):-hnf(cond(A,B),C,D,E).

:- block maybe(?,?,?,?,-,?).
maybe(A,B,C,D,E,F):-hnf(C,G,E,H),maybe_3(G,A,B,D,H,F).

:- block maybe_3(?,?,?,?,-,?).
maybe_3('Nothing',A,B,C,D,E):-hnf(A,C,D,E).
maybe_3('Just'(A),B,C,D,E,F):-hnf(apply(C,A),D,E,F).

:- block either(?,?,?,?,-,?).
either(A,B,C,D,E,F):-hnf(C,G,E,H),either_3(G,A,B,D,H,F).

:- block either_3(?,?,?,?,-,?).
either_3('Left'(A),B,C,D,E,F):-hnf(apply(B,A),D,E,F).
either_3('Right'(A),B,C,D,E,F):-hnf(apply(C,A),D,E,F).

:- block done(?,?,-,?).
done(A,B,C,D):-hnf(apply(return('()'),A),B,C,D).

:- block putStr(?,?,?,-,?).
putStr(A,B,C,D,E):-hnf(A,F,D,G),putStr_1(F,B,C,G,E).

:- block putStr_1(-,?,?,?,?),putStr_1(?,?,?,-,?).
putStr_1([],A,B,C,D):-hnf(apply(done,A),B,C,D).
putStr_1([A|B],C,D,E,F):-hnf(apply(putChar(A)>>putStr(B),C),D,E,F).

:- block putStrLn(?,?,?,-,?).
putStrLn(A,B,C,D,E):-hnf(apply(putStr(A)>>putChar('\'\n'),B),C,D,E).

:- block getLine(?,?,-,?).
getLine(A,B,C,D):-hnf(apply(>>=(getChar,lambdafun_0),A),B,C,D).

:- block print(?,?,?,-,?).
print(A,B,C,D,E):-hnf(apply(putStrLn(show(A)),B),C,D,E).

:- block doSolve(?,?,?,-,?).
doSolve(A,B,C,D,E):-hnf(cond(A,apply(done,B)),C,D,E).

:- block sequenceIO(?,?,?,-,?).
sequenceIO(A,B,C,D,E):-hnf(A,F,D,G),sequenceIO_1(F,B,C,G,E).

:- block sequenceIO_1(-,?,?,?,?),sequenceIO_1(?,?,?,-,?).
sequenceIO_1([],A,B,C,D):-hnf(apply(return([]),A),B,C,D).
sequenceIO_1([A|B],C,D,E,F):-hnf(apply(>>=(A,lambdafun_1(B)),C),D,E,F).

:- block sequenceIO_(?,?,?,-,?).
sequenceIO_(A,B,C,D,E):-hnf(apply(apply(foldr(>>,done),A),B),C,D,E).

:- block mapIO(?,?,?,?,-,?).
mapIO(A,B,C,D,E,F):-hnf(apply(apply('prelude..'(sequenceIO,map(A)),B),C),D,E,F).

:- block mapIO_(?,?,?,?,-,?).
mapIO_(A,B,C,D,E,F):-hnf(apply(apply('prelude..'(sequenceIO_,map(A)),B),C),D,E,F).

:- block ?(?,?,?,-,?).
?(A,B,C,D,E):-hnf(A,C,D,E).
?(A,B,C,D,E):-hnf(B,C,D,E).

:- block inject(?,?,?,?,-,?).
inject(A,B,C,D,E,F):-hnf(apply(prelude_lambda15_0(B,A),C),D,E,F).

:- block solveAll(?,?,-,?).
solveAll(A,B,C,D):-evalall_0(try(A),B,C,D).

:- block solveAll2(?,?,-,?).
solveAll2(A,B,C,D):-evalResult_0(try(A),B,C,D).

:- block once(?,?,?,-,?).
once(A,B,C,D,E):-hnf(apply(head(solveAll(A)),B),C,D,E).

:- block one(?,?,-,?).
one(A,B,C,D):-solveAll(prelude_lambda16_0(A),B,C,D).

:- block condSearch(?,?,?,-,?).
condSearch(A,B,C,D,E):-one(prelude_lambda17_0(A,B),C,D,E).

:- block best(?,?,?,-,?).
best(A,B,C,D,E):-bestHelp_0(B,[],try(A),[],C,D,E).

:- block browse(?,?,?,-,?).
browse(A,B,C,D,E):-hnf(apply(putStr(show(unpack(A))),B),C,D,E).

:- block browseList(?,?,?,-,?).
browseList(A,B,C,D,E):-hnf(A,F,D,G),browseList_1(F,B,C,G,E).

:- block browseList_1(-,?,?,?,?),browseList_1(?,?,?,-,?).
browseList_1([],A,B,C,D):-hnf(apply(done,A),B,C,D).
browseList_1([A|B],C,D,E,F):-hnf(apply(browse(A)>>putChar('\'\n')>>browseList(B),C),D,E,F).

:- block unpack(?,?,-,?).
unpack(A,B,C,D):-makeShare(E,F),hnf(cond(apply(A,F),F),B,C,D).

:- block 'PEVAL'(?,?,-,?).
'PEVAL'(A,B,C,D):-hnf(A,B,C,D).

:- block prelude_lambda0_0(?,?,?,?,-,?).
prelude_lambda0_0(A,B,C,D,E,F):-hnf(apply(A,apply(B,C)),D,E,F).

:- block prelude_lambda1_0(?,?,?,?,-,?).
prelude_lambda1_0(A,B,C,D,E,F):-hnf(C,G,E,H),prelude_lambda1_0_3(G,A,B,D,H,F).

:- block prelude_lambda1_0_3(?,?,?,?,-,?).
prelude_lambda1_0_3('(,)'(A,B),C,D,'(,)'([C|A],[D|B]),E,E).

:- block prelude_lambda2_0(?,?,?,?,?,-,?).
prelude_lambda2_0(A,B,C,D,E,F,G):-hnf(D,H,F,I),prelude_lambda2_0_4(H,A,B,C,E,I,G).

:- block prelude_lambda2_0_4(?,?,?,?,?,-,?).
prelude_lambda2_0_4('(,,)'(A,B,C),D,E,F,'(,,)'([D|A],[E|B],[F|C]),G,G).

:- block takep_0(?,?,?,-,?).
takep_0(A,B,C,D,E):-hnf(B,F,D,G),takep_0_2(F,A,C,G,E).

:- block takep_0_2(?,?,?,-,?).
takep_0_2([],A,[],B,B).
takep_0_2([A|B],C,[A|take(C-1,B)],D,D).

:- block dropp_0(?,?,?,-,?).
dropp_0(A,B,C,D,E):-hnf(B,F,D,G),dropp_0_2(F,A,C,G,E).

:- block dropp_0_2(?,?,?,-,?).
dropp_0_2([],A,[],B,B).
dropp_0_2([A|B],C,D,E,F):-drop(C-1,B,D,E,F).

:- block splitAtp_0(?,?,?,-,?).
splitAtp_0(A,B,C,D,E):-hnf(B,F,D,G),splitAtp_0_2(F,A,C,G,E).

:- block splitAtp_0_2(?,?,?,-,?).
splitAtp_0_2([],A,'(,)'([],[]),B,B).
splitAtp_0_2([A|B],C,D,E,F):-prelude_lambda3_0(A,splitAt(C-1,B),D,E,F).

:- block prelude_lambda3_0(?,?,?,-,?).
prelude_lambda3_0(A,B,C,D,E):-hnf(B,F,D,G),prelude_lambda3_0_2(F,A,C,G,E).

:- block prelude_lambda3_0_2(?,?,?,-,?).
prelude_lambda3_0_2('(,)'(A,B),C,'(,)'([C|A],B),D,D).

:- block prelude_lambda4_0(?,?,?,-,?).
prelude_lambda4_0(A,B,C,D,E):-hnf(B,F,D,G),prelude_lambda4_0_2(F,A,C,G,E).

:- block prelude_lambda4_0_2(?,?,?,-,?).
prelude_lambda4_0_2('(,)'(A,B),C,'(,)'([C|A],B),D,D).

:- block splitline_0(?,?,-,?).
splitline_0(A,B,C,D):-hnf(A,E,C,F),splitline_0_1(E,B,F,D).

:- block splitline_0_1(?,?,-,?).
splitline_0_1([],'(,)'([],[]),A,A).
splitline_0_1([A|B],C,D,E):-makeShare(A,F),makeShare(B,G),if_then_else(F=='\'\n','(,)'([],G),prelude_lambda5_0(F,splitline_0(G)),C,D,E).

:- block prelude_lambda5_0(?,?,?,-,?).
prelude_lambda5_0(A,B,C,D,E):-hnf(B,F,D,G),prelude_lambda5_0_2(F,A,C,G,E).

:- block prelude_lambda5_0_2(?,?,?,-,?).
prelude_lambda5_0_2('(,)'(A,B),C,'(,)'([C|A],B),D,D).

:- block prelude_lambda6_0(?,?,-,?).
prelude_lambda6_0(A,B,C,D):-hnf(A,E,C,F),prelude_lambda6_0_1(E,B,F,D).

:- block prelude_lambda6_0_1(?,?,-,?).
prelude_lambda6_0_1('(,)'(A,B),[A|lines(B)],C,C).

:- block prelude_lambda7_0(?,?,-,?).
prelude_lambda7_0(A,B,C,D):- ++(A,['\'\n'],B,C,D).

:- block isSpace_0(?,?,-,?).
isSpace_0(A,B,C,D):-makeShare(A,E),'||'(E=='\' ','||'(E=='\'\t','||'(E=='\'\n',E=='\'\r')),B,C,D).

:- block prelude_lambda9_0(?,?,-,?).
prelude_lambda9_0(A,B,C,D):-hnf(A,E,C,F),prelude_lambda9_0_1(E,B,F,D).

:- block prelude_lambda9_0_1(?,?,-,?).
prelude_lambda9_0_1('(,)'(A,B),[A|words(B)],C,C).

:- block prelude_lambda8_0(?,?,-,?).
prelude_lambda8_0(A,B,C,D):-makeShare(A,E),if_then_else(E==[],[],prelude_lambda9_0(apply(break(isSpace_0),E)),B,C,D).

:- block prelude_lambda10_0(?,?,?,-,?).
prelude_lambda10_0(A,B,C,D,E):- ++(A,['\' '|B],C,D,E).

:- block prelude_lambda11_0(?,?,?,-,?).
prelude_lambda11_0(A,B,C,D,E):-hnf(A==B,C,D,E).

:- block prelude_lambda12_0(?,?,?,-,?).
prelude_lambda12_0(A,B,C,D,E):- /=(A,B,C,D,E).

:- block prelude_lambda13_0(?,?,?,?,-,?).
prelude_lambda13_0(A,B,C,D,E,F):-hnf(A-B+C,D,E,F).

:- block p_0(?,?,?,?,?,-,?).
p_0(A,B,C,D,E,F,G):-makeShare(D,H),makeShare(C,I),if_then_else(A>=B,<=(H,I),H>=I,E,F,G).

:- block prelude_lambda14_0(?,?,?,?,-,?).
prelude_lambda14_0(A,B,C,D,E,F):-hnf(apply(A,C),D,E,F).

:- block lambdafun_0(?,?,?,-,?).
lambdafun_0(A,B,C,D,E):-makeShare(A,F),hnf(apply(if_then_else(F=='\'\n',return([]),>>=(getLine,lambdafun_2(F))),B),C,D,E).

:- block lambdafun_2(?,?,?,?,-,?).
lambdafun_2(A,B,C,D,E,F):-hnf(apply(return([A|B]),C),D,E,F).

:- block lambdafun_3(?,?,?,?,-,?).
lambdafun_3(A,B,C,D,E,F):-hnf(apply(return([A|B]),C),D,E,F).

:- block lambdafun_1(?,?,?,?,-,?).
lambdafun_1(A,B,C,D,E,F):-hnf(apply(>>=(sequenceIO(A),lambdafun_3(B)),C),D,E,F).

:- block prelude_lambda15_0(?,?,?,?,-,?).
prelude_lambda15_0(A,B,C,D,E,F):-makeShare(C,G),hnf(&(apply(A,G),apply(B,G)),D,E,F).

:- block evalall_0(?,?,-,?).
evalall_0(A,B,C,D):-hnf(A,E,C,F),evalall_0_1(E,B,F,D).

:- block evalall_0_1(?,?,-,?).
evalall_0_1([],[],A,A).
evalall_0_1([A|B],C,D,E):-hnf(B,F,D,G),'evalall_0_1_._2'(F,A,C,G,E).

:- block 'evalall_0_1_._2'(?,?,?,-,?).
'evalall_0_1_._2'([],A,[A],B,B).
'evalall_0_1_._2'([A|B],C,D,E,F):-evalall3_0(try(C),[A|B],D,E,F).

:- block evalall2_0(?,?,-,?).
evalall2_0(A,B,C,D):-hnf(A,E,C,F),evalall2_0_1(E,B,F,D).

:- block evalall2_0_1(?,?,-,?).
evalall2_0_1([],[],A,A).
evalall2_0_1([A|B],C,D,E):-evalall3_0(try(A),B,C,D,E).

:- block evalall3_0(?,?,?,-,?).
evalall3_0(A,B,C,D,E):-hnf(A,F,D,G),evalall3_0_1(F,B,C,G,E).

:- block evalall3_0_1(?,?,?,-,?).
evalall3_0_1([],A,B,C,D):-evalall2_0(A,B,C,D).
evalall3_0_1([A|B],C,D,E,F):-hnf(B,G,E,H),'evalall3_0_1_._2'(G,A,C,D,H,F).

:- block 'evalall3_0_1_._2'(?,?,?,?,-,?).
'evalall3_0_1_._2'([],A,B,[A|evalall2_0(B)],C,C).
'evalall3_0_1_._2'([A|B],C,D,E,F,G):-evalall3_0(try(C),[A|++(B,D)],E,F,G).

:- block evalResult_0(?,?,-,?).
evalResult_0(A,B,C,D):-hnf(A,E,C,F),evalResult_0_1(E,B,F,D).

:- block evalResult_0_1(?,?,-,?).
evalResult_0_1([],[],A,A).
evalResult_0_1([A|B],C,D,E):-hnf(B,F,D,G),'evalResult_0_1_._2'(F,A,C,G,E).

:- block 'evalResult_0_1_._2'(?,?,?,-,?).
'evalResult_0_1_._2'([],A,[A],B,B).
'evalResult_0_1_._2'([A|B],C,D,E,F):-hnf(apply(concatMap(solveAll2),[C,A|B]),D,E,F).

:- block prelude_lambda16_0(?,?,?,-,?).
prelude_lambda16_0(A,B,C,D,E):-hnf(B=:=tryone(A),C,D,E).

:- block prelude_lambda17_0(?,?,?,?,-,?).
prelude_lambda17_0(A,B,C,D,E,F):-makeShare(C,G),hnf(&(apply(A,G),apply(B,G)=:='True'),D,E,F).

:- block bestHelp_0(?,?,?,?,?,-,?).
bestHelp_0(A,B,C,D,E,F,G):-hnf(B,H,F,I),bestHelp_0_2(H,A,C,D,E,I,G).

:- block bestHelp_0_2(?,?,?,?,?,-,?).
bestHelp_0_2([],A,B,C,D,E,F):-hnf(B,G,E,H),'bestHelp_0_2_[]_2'(G,A,C,D,H,F).

:- block 'bestHelp_0_2_[]_2'(?,?,?,?,-,?).
'bestHelp_0_2_[]_2'([],A,B,C,D,E):-hnf(B,C,D,E).
'bestHelp_0_2_[]_2'([A|B],C,D,E,F,G):-makeShare(C,H),makeShare(D,I),evalY_0(H,try(constrain_0(H,A,I)),B,I,E,F,G).
bestHelp_0_2([A|B],C,D,E,F,G,H):-evalX_0(C,try(A),B,D,E,F,G,H).

:- block evalY_0(?,?,?,?,?,-,?).
evalY_0(A,B,C,D,E,F,G):-hnf(B,H,F,I),evalY_0_2(H,A,C,D,E,I,G).

:- block evalY_0_2(?,?,?,?,?,-,?).
evalY_0_2([],A,B,C,D,E,F):-bestHelp_0(A,[],B,C,D,E,F).
evalY_0_2([A|B],C,D,E,F,G,H):-hnf(B,I,G,J),'evalY_0_2_._2'(I,A,C,D,E,F,J,H).

:- block 'evalY_0_2_._2'(?,?,?,?,?,?,-,?).
'evalY_0_2_._2'([],A,B,C,D,E,F,G):-bestHelp_0(B,[],C,[A],E,F,G).
'evalY_0_2_._2'([A|B],C,D,E,F,G,H,I):-bestHelp_0(D,[C,A|B],E,F,G,H,I).

:- block evalX_0(?,?,?,?,?,?,-,?).
evalX_0(A,B,C,D,E,F,G,H):-hnf(B,I,G,J),evalX_0_2(I,A,C,D,E,F,J,H).

:- block evalX_0_2(?,?,?,?,?,?,-,?).
evalX_0_2([],A,B,C,D,E,F,G):-bestHelp_0(A,B,C,D,E,F,G).
evalX_0_2([A|B],C,D,E,F,G,H,I):-hnf(B,J,H,K),'evalX_0_2_._2'(J,A,C,D,E,F,G,K,I).

:- block 'evalX_0_2_._2'(?,?,?,?,?,?,?,-,?).
'evalX_0_2_._2'([],A,B,C,D,E,F,G,H):-bestHelp_0(B,[],++(C,D),[A],F,G,H).
'evalX_0_2_._2'([A|B],C,D,E,F,G,H,I,J):-bestHelp_0(D,++([C,A|B],E),F,G,H,I,J).

:- block constrain_0(?,?,?,?,?,-,?).
constrain_0(A,B,C,D,E,F,G):-hnf(C,H,F,I),constrain_0_3(H,A,B,D,E,I,G).

:- block constrain_0_3(?,?,?,?,?,-,?).
constrain_0_3([],A,B,C,D,E,F):-hnf(apply(B,C),D,E,F).
constrain_0_3([A|B],C,D,E,F,G,H):-hnf(B,I,G,J),'constrain_0_3_._2'(I,A,C,D,E,F,J,H).

:- block 'constrain_0_3_._2'(?,?,?,?,?,?,-,?).
'constrain_0_3_._2'([],A,B,C,D,E,F,G):-hnf(apply(inject(C,prelude_lambda18_0(A,B)),D),E,F,G).

:- block prelude_lambda18_0(?,?,?,?,-,?).
prelude_lambda18_0(A,B,C,D,E,F):-makeShare(G,H),hnf(&(apply(A,H),apply(apply(B,C),H)=:='True'),D,E,F).

:-block hnf(?,?,-,?).
hnf(A,B,C,D):-var(A),!,B=A,C=D.
hnf(share(A),B,C,D):-!,get_mutable(E,A),(E='$eval'(F)->B=F,C=D;hnf(E,G,C,H),(nonvar(G),(functor(G,'$io',1);functor(G,success,0))->B=G;propagateShare(G,B)),update_mutable('$eval'(B),A),H=D).
hnf('prelude..'(A,B,C),D,E,F):-!,'prelude..'(A,B,C,D,E,F).
hnf(id(A),B,C,D):-!,id(A,B,C,D).
hnf(const(A,B),C,D,E):-!,const(A,B,C,D,E).
hnf(curry(A,B,C),D,E,F):-!,curry(A,B,C,D,E,F).
hnf(uncurry(A,B),C,D,E):-!,uncurry(A,B,C,D,E).
hnf(flip(A,B,C),D,E,F):-!,flip(A,B,C,D,E,F).
hnf(until(A,B,C),D,E,F):-!,until(A,B,C,D,E,F).
hnf($(A,B),C,D,E):-!,$(A,B,C,D,E).
hnf('$!'(A,B),C,D,E):-!,'$!'(A,B,C,D,E).
hnf(failed,A,B,C):-!,failed(A,B,C).
hnf(&&(A,B),C,D,E):-!,&&(A,B,C,D,E).
hnf('||'(A,B),C,D,E):-!,'||'(A,B,C,D,E).
hnf(not(A),B,C,D):-!,not(A,B,C,D).
hnf(otherwise,A,B,C):-!,otherwise(A,B,C).
hnf(if_then_else(A,B,C),D,E,F):-!,if_then_else(A,B,C,D,E,F).
hnf(/=(A,B),C,D,E):-!,/=(A,B,C,D,E).
hnf(fst(A),B,C,D):-!,fst(A,B,C,D).
hnf(snd(A),B,C,D):-!,snd(A,B,C,D).
hnf(head(A),B,C,D):-!,head(A,B,C,D).
hnf(tail(A),B,C,D):-!,tail(A,B,C,D).
hnf(null(A),B,C,D):-!,null(A,B,C,D).
hnf(++(A,B),C,D,E):-!,++(A,B,C,D,E).
hnf(length(A),B,C,D):-!,length(A,B,C,D).
hnf('!!'(A,B),C,D,E):-!,'!!'(A,B,C,D,E).
hnf(map(A,B),C,D,E):-!,map(A,B,C,D,E).
hnf(foldl(A,B,C),D,E,F):-!,foldl(A,B,C,D,E,F).
hnf(foldl1(A,B),C,D,E):-!,foldl1(A,B,C,D,E).
hnf(foldr(A,B,C),D,E,F):-!,foldr(A,B,C,D,E,F).
hnf(foldr1(A,B),C,D,E):-!,foldr1(A,B,C,D,E).
hnf(filter(A,B),C,D,E):-!,filter(A,B,C,D,E).
hnf(zip(A,B),C,D,E):-!,zip(A,B,C,D,E).
hnf(zip3(A,B,C),D,E,F):-!,zip3(A,B,C,D,E,F).
hnf(zipWith(A,B,C),D,E,F):-!,zipWith(A,B,C,D,E,F).
hnf(zipWith3(A,B,C,D),E,F,G):-!,zipWith3(A,B,C,D,E,F,G).
hnf(unzip(A),B,C,D):-!,unzip(A,B,C,D).
hnf(unzip3(A),B,C,D):-!,unzip3(A,B,C,D).
hnf(concat(A),B,C,D):-!,concat(A,B,C,D).
hnf(concatMap(A,B),C,D,E):-!,concatMap(A,B,C,D,E).
hnf(iterate(A,B),C,D,E):-!,iterate(A,B,C,D,E).
hnf(repeat(A),B,C,D):-!,repeat(A,B,C,D).
hnf(replicate(A,B),C,D,E):-!,replicate(A,B,C,D,E).
hnf(take(A,B),C,D,E):-!,take(A,B,C,D,E).
hnf(drop(A,B),C,D,E):-!,drop(A,B,C,D,E).
hnf(splitAt(A,B),C,D,E):-!,splitAt(A,B,C,D,E).
hnf(takeWhile(A,B),C,D,E):-!,takeWhile(A,B,C,D,E).
hnf(dropWhile(A,B),C,D,E):-!,dropWhile(A,B,C,D,E).
hnf(span(A,B),C,D,E):-!,span(A,B,C,D,E).
hnf(break(A,B),C,D,E):-!,break(A,B,C,D,E).
hnf(lines(A),B,C,D):-!,lines(A,B,C,D).
hnf(unlines(A),B,C,D):-!,unlines(A,B,C,D).
hnf(words(A),B,C,D):-!,words(A,B,C,D).
hnf(unwords(A),B,C,D):-!,unwords(A,B,C,D).
hnf(reverse(A),B,C,D):-!,reverse(A,B,C,D).
hnf(and(A),B,C,D):-!,and(A,B,C,D).
hnf(or(A),B,C,D):-!,or(A,B,C,D).
hnf(any(A,B),C,D,E):-!,any(A,B,C,D,E).
hnf(all(A,B),C,D,E):-!,all(A,B,C,D,E).
hnf(elem(A,B),C,D,E):-!,elem(A,B,C,D,E).
hnf(notElem(A,B),C,D,E):-!,notElem(A,B,C,D,E).
hnf(lookup(A,B),C,D,E):-!,lookup(A,B,C,D,E).
hnf(enumFrom(A),B,C,D):-!,enumFrom(A,B,C,D).
hnf(enumFromThen(A,B),C,D,E):-!,enumFromThen(A,B,C,D,E).
hnf(enumFromTo(A,B),C,D,E):-!,enumFromTo(A,B,C,D,E).
hnf(enumFromThenTo(A,B,C),D,E,F):-!,enumFromThenTo(A,B,C,D,E,F).
hnf(negate(A),B,C,D):-!,negate(A,B,C,D).
hnf(&>(A,B),C,D,E):-!,&>(A,B,C,D,E).
hnf(maybe(A,B,C),D,E,F):-!,maybe(A,B,C,D,E,F).
hnf(either(A,B,C),D,E,F):-!,either(A,B,C,D,E,F).
hnf(done(A),B,C,D):-!,done(A,B,C,D).
hnf(putStr(A,B),C,D,E):-!,putStr(A,B,C,D,E).
hnf(putStrLn(A,B),C,D,E):-!,putStrLn(A,B,C,D,E).
hnf(getLine(A),B,C,D):-!,getLine(A,B,C,D).
hnf(print(A,B),C,D,E):-!,print(A,B,C,D,E).
hnf(doSolve(A,B),C,D,E):-!,doSolve(A,B,C,D,E).
hnf(sequenceIO(A,B),C,D,E):-!,sequenceIO(A,B,C,D,E).
hnf(sequenceIO_(A,B),C,D,E):-!,sequenceIO_(A,B,C,D,E).
hnf(mapIO(A,B,C),D,E,F):-!,mapIO(A,B,C,D,E,F).
hnf(mapIO_(A,B,C),D,E,F):-!,mapIO_(A,B,C,D,E,F).
hnf(?(A,B),C,D,E):-!,?(A,B,C,D,E).
hnf(inject(A,B,C),D,E,F):-!,inject(A,B,C,D,E,F).
hnf(solveAll(A),B,C,D):-!,solveAll(A,B,C,D).
hnf(solveAll2(A),B,C,D):-!,solveAll2(A,B,C,D).
hnf(once(A,B),C,D,E):-!,once(A,B,C,D,E).
hnf(one(A),B,C,D):-!,one(A,B,C,D).
hnf(condSearch(A,B),C,D,E):-!,condSearch(A,B,C,D,E).
hnf(best(A,B),C,D,E):-!,best(A,B,C,D,E).
hnf(browse(A,B),C,D,E):-!,browse(A,B,C,D,E).
hnf(browseList(A,B),C,D,E):-!,browseList(A,B,C,D,E).
hnf(unpack(A),B,C,D):-!,unpack(A,B,C,D).
hnf('PEVAL'(A),B,C,D):-!,'PEVAL'(A,B,C,D).
hnf(prelude_lambda0_0(A,B,C),D,E,F):-!,prelude_lambda0_0(A,B,C,D,E,F).
hnf(prelude_lambda1_0(A,B,C),D,E,F):-!,prelude_lambda1_0(A,B,C,D,E,F).
hnf(prelude_lambda2_0(A,B,C,D),E,F,G):-!,prelude_lambda2_0(A,B,C,D,E,F,G).
hnf(takep_0(A,B),C,D,E):-!,takep_0(A,B,C,D,E).
hnf(dropp_0(A,B),C,D,E):-!,dropp_0(A,B,C,D,E).
hnf(splitAtp_0(A,B),C,D,E):-!,splitAtp_0(A,B,C,D,E).
hnf(prelude_lambda3_0(A,B),C,D,E):-!,prelude_lambda3_0(A,B,C,D,E).
hnf(prelude_lambda4_0(A,B),C,D,E):-!,prelude_lambda4_0(A,B,C,D,E).
hnf(splitline_0(A),B,C,D):-!,splitline_0(A,B,C,D).
hnf(prelude_lambda5_0(A,B),C,D,E):-!,prelude_lambda5_0(A,B,C,D,E).
hnf(prelude_lambda6_0(A),B,C,D):-!,prelude_lambda6_0(A,B,C,D).
hnf(prelude_lambda7_0(A),B,C,D):-!,prelude_lambda7_0(A,B,C,D).
hnf(isSpace_0(A),B,C,D):-!,isSpace_0(A,B,C,D).
hnf(prelude_lambda9_0(A),B,C,D):-!,prelude_lambda9_0(A,B,C,D).
hnf(prelude_lambda8_0(A),B,C,D):-!,prelude_lambda8_0(A,B,C,D).
hnf(prelude_lambda10_0(A,B),C,D,E):-!,prelude_lambda10_0(A,B,C,D,E).
hnf(prelude_lambda11_0(A,B),C,D,E):-!,prelude_lambda11_0(A,B,C,D,E).
hnf(prelude_lambda12_0(A,B),C,D,E):-!,prelude_lambda12_0(A,B,C,D,E).
hnf(prelude_lambda13_0(A,B,C),D,E,F):-!,prelude_lambda13_0(A,B,C,D,E,F).
hnf(p_0(A,B,C,D),E,F,G):-!,p_0(A,B,C,D,E,F,G).
hnf(prelude_lambda14_0(A,B,C),D,E,F):-!,prelude_lambda14_0(A,B,C,D,E,F).
hnf(lambdafun_0(A,B),C,D,E):-!,lambdafun_0(A,B,C,D,E).
hnf(lambdafun_2(A,B,C),D,E,F):-!,lambdafun_2(A,B,C,D,E,F).
hnf(lambdafun_3(A,B,C),D,E,F):-!,lambdafun_3(A,B,C,D,E,F).
hnf(lambdafun_1(A,B,C),D,E,F):-!,lambdafun_1(A,B,C,D,E,F).
hnf(prelude_lambda15_0(A,B,C),D,E,F):-!,prelude_lambda15_0(A,B,C,D,E,F).
hnf(evalall_0(A),B,C,D):-!,evalall_0(A,B,C,D).
hnf(evalall2_0(A),B,C,D):-!,evalall2_0(A,B,C,D).
hnf(evalall3_0(A,B),C,D,E):-!,evalall3_0(A,B,C,D,E).
hnf(evalResult_0(A),B,C,D):-!,evalResult_0(A,B,C,D).
hnf(prelude_lambda16_0(A,B),C,D,E):-!,prelude_lambda16_0(A,B,C,D,E).
hnf(prelude_lambda17_0(A,B,C),D,E,F):-!,prelude_lambda17_0(A,B,C,D,E,F).
hnf(bestHelp_0(A,B,C,D),E,F,G):-!,bestHelp_0(A,B,C,D,E,F,G).
hnf(evalY_0(A,B,C,D),E,F,G):-!,evalY_0(A,B,C,D,E,F,G).
hnf(evalX_0(A,B,C,D,E),F,G,H):-!,evalX_0(A,B,C,D,E,F,G,H).
hnf(constrain_0(A,B,C,D),E,F,G):-!,constrain_0(A,B,C,D,E,F,G).
hnf(prelude_lambda18_0(A,B,C),D,E,F):-!,prelude_lambda18_0(A,B,C,D,E,F).
hnf(seq(A,B),C,D,E):-!,hnf(A,F,D,G),prim_seq(F,B,C,G,E).
hnf(error(A),B,C,D):-!,nf(A,E,C,F),boolEq(E,E,G,F,H),prim_error(E,B,H,D).
hnf(A==B,C,D,E):-!,boolEq(A,B,C,D,E).
hnf(ord(A),B,C,D):-!,hnf(A,E,C,F),prim_ord(E,B,F,D).
hnf(chr(A),B,C,D):-!,hnf(A,E,C,F),prim_chr(E,B,F,D).
hnf(A+B,C,D,E):-!,hnf(A,F,D,G),hnf(B,H,G,I),'prim_+'(F,H,C,I,E).
hnf(A-B,C,D,E):-!,hnf(A,F,D,G),hnf(B,H,G,I),'prim_-'(F,H,C,I,E).
hnf(A*B,C,D,E):-!,hnf(A,F,D,G),hnf(B,H,G,I),'prim_*'(F,H,C,I,E).
hnf(div(A,B),C,D,E):-!,hnf(A,F,D,G),hnf(B,H,G,I),prim_div(F,H,C,I,E).
hnf(A mod B,C,D,E):-!,hnf(A,F,D,G),hnf(B,H,G,I),prim_mod(F,H,C,I,E).
hnf(A<B,C,D,E):-!,hnf(A,F,D,G),hnf(B,H,G,I),'prim_<'(F,H,C,I,E).
hnf(A>B,C,D,E):-!,hnf(A,F,D,G),hnf(B,H,G,I),'prim_>'(F,H,C,I,E).
hnf(<=(A,B),C,D,E):-!,hnf(A,F,D,G),hnf(B,H,G,I),'prim_<='(F,H,C,I,E).
hnf(A>=B,C,D,E):-!,hnf(A,F,D,G),hnf(B,H,G,I),'prim_>='(F,H,C,I,E).
hnf(A=:=B,success,C,D):-!,constrEq(A,B,C,D).
hnf(&(A,B),C,D,E):-!,'prim_&'(A,B,C,D,E).
hnf(>>=(A,B,C),D,E,F):-!,hnf(A,G,E,H),'prim_>>='(G,B,C,D,H,F).
hnf(return(A,B),C,D,E):-!,prim_return(A,B,C,D,E).
hnf(putChar(A,B),C,D,E):-!,hnf(A,F,D,G),prim_putChar(F,B,C,G,E).
hnf(getChar(A),B,C,D):-!,prim_getChar(A,B,C,D).
hnf(readFile(A,B),C,D,E):-!,nf(A,F,D,G),boolEq(F,F,H,G,I),prim_readFile(F,B,C,I,E).
hnf(writeFile(A,B,C),D,E,F):-!,nf(A,G,E,H),boolEq(G,G,I,H,J),prim_writeFile(G,B,C,D,J,F).
hnf(appendFile(A,B,C),D,E,F):-!,nf(A,G,E,H),boolEq(G,G,I,H,J),prim_appendFile(G,B,C,D,J,F).
hnf(catchFail(A,B,C),D,E,F):-!,prim_catchFail(A,B,C,D,E,F).
hnf(show(A),B,C,D):-!,nf(A,E,C,F),boolEq(E,E,G,F,H),prim_show(E,B,H,D).
hnf(try(A),B,C,D):-!,prim_try(A,B,C,D).
hnf(apply(A,B),C,D,E):-!,hnf(A,F,D,G),apply(F,B,C,G,E).
hnf(cond(A,B),C,D,E):-!,hnf(A,F,D,G),prim_cond(F,B,C,G,E).
hnf(commit(A),B,C,D):-!,prim_commit(A,B,C,D).
hnf(>>(A,B,C),D,E,F):-!,hnf(A,G,E,H),'prim_>>'(G,B,C,D,H,F).
hnf(readFileContents(A,B),C,D,E):-!,prim_readFileContents(A,B,C,D,E).
hnf(writeFileContents(A,B,C),D,E,F):-!,hnf(B,G,E,H),prim_writeFileContents(A,G,C,D,H,F).
hnf(findall(A),B,C,D):-!,hnf(A,E,C,F),prim_findall(E,B,F,D).
hnf(findfirst(A),B,C,D):-!,hnf(A,E,C,F),prim_findfirst(E,B,F,D).
hnf(A,A,B,B).

:-block apply(-,?,?,?,?),apply(?,?,?,-,?).
apply('prelude..',A,'prelude..'(A),B,B).
apply('prelude..'(A),B,'prelude..'(A,B),C,C).
apply('prelude..'(A,B),C,D,E,F):-hnf('prelude..'(A,B,C),D,E,F).
apply(id,A,B,C,D):-hnf(id(A),B,C,D).
apply(const,A,const(A),B,B).
apply(const(A),B,C,D,E):-hnf(const(A,B),C,D,E).
apply(curry,A,curry(A),B,B).
apply(curry(A),B,curry(A,B),C,C).
apply(curry(A,B),C,D,E,F):-hnf(curry(A,B,C),D,E,F).
apply(uncurry,A,uncurry(A),B,B).
apply(uncurry(A),B,C,D,E):-hnf(uncurry(A,B),C,D,E).
apply(flip,A,flip(A),B,B).
apply(flip(A),B,flip(A,B),C,C).
apply(flip(A,B),C,D,E,F):-hnf(flip(A,B,C),D,E,F).
apply(until,A,until(A),B,B).
apply(until(A),B,until(A,B),C,C).
apply(until(A,B),C,D,E,F):-hnf(until(A,B,C),D,E,F).
apply($,A,$(A),B,B).
apply($(A),B,C,D,E):-hnf($(A,B),C,D,E).
apply('$!',A,'$!'(A),B,B).
apply('$!'(A),B,C,D,E):-hnf('$!'(A,B),C,D,E).
apply(&&,A,&&(A),B,B).
apply(&&(A),B,C,D,E):-hnf(&&(A,B),C,D,E).
apply('||',A,'||'(A),B,B).
apply('||'(A),B,C,D,E):-hnf('||'(A,B),C,D,E).
apply(not,A,B,C,D):-hnf(not(A),B,C,D).
apply(if_then_else,A,if_then_else(A),B,B).
apply(if_then_else(A),B,if_then_else(A,B),C,C).
apply(if_then_else(A,B),C,D,E,F):-hnf(if_then_else(A,B,C),D,E,F).
apply(/=,A,/=(A),B,B).
apply(/=(A),B,C,D,E):-hnf(/=(A,B),C,D,E).
apply(fst,A,B,C,D):-hnf(fst(A),B,C,D).
apply(snd,A,B,C,D):-hnf(snd(A),B,C,D).
apply(head,A,B,C,D):-hnf(head(A),B,C,D).
apply(tail,A,B,C,D):-hnf(tail(A),B,C,D).
apply(null,A,B,C,D):-hnf(null(A),B,C,D).
apply(++,A,++(A),B,B).
apply(++(A),B,C,D,E):-hnf(++(A,B),C,D,E).
apply(length,A,B,C,D):-hnf(length(A),B,C,D).
apply('!!',A,'!!'(A),B,B).
apply('!!'(A),B,C,D,E):-hnf('!!'(A,B),C,D,E).
apply(map,A,map(A),B,B).
apply(map(A),B,C,D,E):-hnf(map(A,B),C,D,E).
apply(foldl,A,foldl(A),B,B).
apply(foldl(A),B,foldl(A,B),C,C).
apply(foldl(A,B),C,D,E,F):-hnf(foldl(A,B,C),D,E,F).
apply(foldl1,A,foldl1(A),B,B).
apply(foldl1(A),B,C,D,E):-hnf(foldl1(A,B),C,D,E).
apply(foldr,A,foldr(A),B,B).
apply(foldr(A),B,foldr(A,B),C,C).
apply(foldr(A,B),C,D,E,F):-hnf(foldr(A,B,C),D,E,F).
apply(foldr1,A,foldr1(A),B,B).
apply(foldr1(A),B,C,D,E):-hnf(foldr1(A,B),C,D,E).
apply(filter,A,filter(A),B,B).
apply(filter(A),B,C,D,E):-hnf(filter(A,B),C,D,E).
apply(zip,A,zip(A),B,B).
apply(zip(A),B,C,D,E):-hnf(zip(A,B),C,D,E).
apply(zip3,A,zip3(A),B,B).
apply(zip3(A),B,zip3(A,B),C,C).
apply(zip3(A,B),C,D,E,F):-hnf(zip3(A,B,C),D,E,F).
apply(zipWith,A,zipWith(A),B,B).
apply(zipWith(A),B,zipWith(A,B),C,C).
apply(zipWith(A,B),C,D,E,F):-hnf(zipWith(A,B,C),D,E,F).
apply(zipWith3,A,zipWith3(A),B,B).
apply(zipWith3(A),B,zipWith3(A,B),C,C).
apply(zipWith3(A,B),C,zipWith3(A,B,C),D,D).
apply(zipWith3(A,B,C),D,E,F,G):-hnf(zipWith3(A,B,C,D),E,F,G).
apply(unzip,A,B,C,D):-hnf(unzip(A),B,C,D).
apply(unzip3,A,B,C,D):-hnf(unzip3(A),B,C,D).
apply(concat,A,B,C,D):-hnf(concat(A),B,C,D).
apply(concatMap,A,concatMap(A),B,B).
apply(concatMap(A),B,C,D,E):-hnf(concatMap(A,B),C,D,E).
apply(iterate,A,iterate(A),B,B).
apply(iterate(A),B,C,D,E):-hnf(iterate(A,B),C,D,E).
apply(repeat,A,B,C,D):-hnf(repeat(A),B,C,D).
apply(replicate,A,replicate(A),B,B).
apply(replicate(A),B,C,D,E):-hnf(replicate(A,B),C,D,E).
apply(take,A,take(A),B,B).
apply(take(A),B,C,D,E):-hnf(take(A,B),C,D,E).
apply(drop,A,drop(A),B,B).
apply(drop(A),B,C,D,E):-hnf(drop(A,B),C,D,E).
apply(splitAt,A,splitAt(A),B,B).
apply(splitAt(A),B,C,D,E):-hnf(splitAt(A,B),C,D,E).
apply(takeWhile,A,takeWhile(A),B,B).
apply(takeWhile(A),B,C,D,E):-hnf(takeWhile(A,B),C,D,E).
apply(dropWhile,A,dropWhile(A),B,B).
apply(dropWhile(A),B,C,D,E):-hnf(dropWhile(A,B),C,D,E).
apply(span,A,span(A),B,B).
apply(span(A),B,C,D,E):-hnf(span(A,B),C,D,E).
apply(break,A,break(A),B,B).
apply(break(A),B,C,D,E):-hnf(break(A,B),C,D,E).
apply(lines,A,B,C,D):-hnf(lines(A),B,C,D).
apply(unlines,A,B,C,D):-hnf(unlines(A),B,C,D).
apply(words,A,B,C,D):-hnf(words(A),B,C,D).
apply(unwords,A,B,C,D):-hnf(unwords(A),B,C,D).
apply(reverse,A,B,C,D):-hnf(reverse(A),B,C,D).
apply(and,A,B,C,D):-hnf(and(A),B,C,D).
apply(or,A,B,C,D):-hnf(or(A),B,C,D).
apply(any,A,any(A),B,B).
apply(any(A),B,C,D,E):-hnf(any(A,B),C,D,E).
apply(all,A,all(A),B,B).
apply(all(A),B,C,D,E):-hnf(all(A,B),C,D,E).
apply(elem,A,elem(A),B,B).
apply(elem(A),B,C,D,E):-hnf(elem(A,B),C,D,E).
apply(notElem,A,notElem(A),B,B).
apply(notElem(A),B,C,D,E):-hnf(notElem(A,B),C,D,E).
apply(lookup,A,lookup(A),B,B).
apply(lookup(A),B,C,D,E):-hnf(lookup(A,B),C,D,E).
apply(enumFrom,A,B,C,D):-hnf(enumFrom(A),B,C,D).
apply(enumFromThen,A,enumFromThen(A),B,B).
apply(enumFromThen(A),B,C,D,E):-hnf(enumFromThen(A,B),C,D,E).
apply(enumFromTo,A,enumFromTo(A),B,B).
apply(enumFromTo(A),B,C,D,E):-hnf(enumFromTo(A,B),C,D,E).
apply(enumFromThenTo,A,enumFromThenTo(A),B,B).
apply(enumFromThenTo(A),B,enumFromThenTo(A,B),C,C).
apply(enumFromThenTo(A,B),C,D,E,F):-hnf(enumFromThenTo(A,B,C),D,E,F).
apply(negate,A,B,C,D):-hnf(negate(A),B,C,D).
apply(&>,A,&>(A),B,B).
apply(&>(A),B,C,D,E):-hnf(&>(A,B),C,D,E).
apply(maybe,A,maybe(A),B,B).
apply(maybe(A),B,maybe(A,B),C,C).
apply(maybe(A,B),C,D,E,F):-hnf(maybe(A,B,C),D,E,F).
apply(either,A,either(A),B,B).
apply(either(A),B,either(A,B),C,C).
apply(either(A,B),C,D,E,F):-hnf(either(A,B,C),D,E,F).
apply(done,A,B,C,D):-hnf(done(A),B,C,D).
apply(putStr,A,putStr(A),B,B).
apply(putStr(A),B,C,D,E):-hnf(putStr(A,B),C,D,E).
apply(putStrLn,A,putStrLn(A),B,B).
apply(putStrLn(A),B,C,D,E):-hnf(putStrLn(A,B),C,D,E).
apply(getLine,A,B,C,D):-hnf(getLine(A),B,C,D).
apply(print,A,print(A),B,B).
apply(print(A),B,C,D,E):-hnf(print(A,B),C,D,E).
apply(doSolve,A,doSolve(A),B,B).
apply(doSolve(A),B,C,D,E):-hnf(doSolve(A,B),C,D,E).
apply(sequenceIO,A,sequenceIO(A),B,B).
apply(sequenceIO(A),B,C,D,E):-hnf(sequenceIO(A,B),C,D,E).
apply(sequenceIO_,A,sequenceIO_(A),B,B).
apply(sequenceIO_(A),B,C,D,E):-hnf(sequenceIO_(A,B),C,D,E).
apply(mapIO,A,mapIO(A),B,B).
apply(mapIO(A),B,mapIO(A,B),C,C).
apply(mapIO(A,B),C,D,E,F):-hnf(mapIO(A,B,C),D,E,F).
apply(mapIO_,A,mapIO_(A),B,B).
apply(mapIO_(A),B,mapIO_(A,B),C,C).
apply(mapIO_(A,B),C,D,E,F):-hnf(mapIO_(A,B,C),D,E,F).
apply(?,A,?(A),B,B).
apply(?(A),B,C,D,E):-hnf(?(A,B),C,D,E).
apply(inject,A,inject(A),B,B).
apply(inject(A),B,inject(A,B),C,C).
apply(inject(A,B),C,D,E,F):-hnf(inject(A,B,C),D,E,F).
apply(solveAll,A,B,C,D):-hnf(solveAll(A),B,C,D).
apply(solveAll2,A,B,C,D):-hnf(solveAll2(A),B,C,D).
apply(once,A,once(A),B,B).
apply(once(A),B,C,D,E):-hnf(once(A,B),C,D,E).
apply(one,A,B,C,D):-hnf(one(A),B,C,D).
apply(condSearch,A,condSearch(A),B,B).
apply(condSearch(A),B,C,D,E):-hnf(condSearch(A,B),C,D,E).
apply(best,A,best(A),B,B).
apply(best(A),B,C,D,E):-hnf(best(A,B),C,D,E).
apply(browse,A,browse(A),B,B).
apply(browse(A),B,C,D,E):-hnf(browse(A,B),C,D,E).
apply(browseList,A,browseList(A),B,B).
apply(browseList(A),B,C,D,E):-hnf(browseList(A,B),C,D,E).
apply(unpack,A,B,C,D):-hnf(unpack(A),B,C,D).
apply('PEVAL',A,B,C,D):-hnf('PEVAL'(A),B,C,D).
apply(prelude_lambda0_0,A,prelude_lambda0_0(A),B,B).
apply(prelude_lambda0_0(A),B,prelude_lambda0_0(A,B),C,C).
apply(prelude_lambda0_0(A,B),C,D,E,F):-hnf(prelude_lambda0_0(A,B,C),D,E,F).
apply(prelude_lambda1_0,A,prelude_lambda1_0(A),B,B).
apply(prelude_lambda1_0(A),B,prelude_lambda1_0(A,B),C,C).
apply(prelude_lambda1_0(A,B),C,D,E,F):-hnf(prelude_lambda1_0(A,B,C),D,E,F).
apply(prelude_lambda2_0,A,prelude_lambda2_0(A),B,B).
apply(prelude_lambda2_0(A),B,prelude_lambda2_0(A,B),C,C).
apply(prelude_lambda2_0(A,B),C,prelude_lambda2_0(A,B,C),D,D).
apply(prelude_lambda2_0(A,B,C),D,E,F,G):-hnf(prelude_lambda2_0(A,B,C,D),E,F,G).
apply(takep_0,A,takep_0(A),B,B).
apply(takep_0(A),B,C,D,E):-hnf(takep_0(A,B),C,D,E).
apply(dropp_0,A,dropp_0(A),B,B).
apply(dropp_0(A),B,C,D,E):-hnf(dropp_0(A,B),C,D,E).
apply(splitAtp_0,A,splitAtp_0(A),B,B).
apply(splitAtp_0(A),B,C,D,E):-hnf(splitAtp_0(A,B),C,D,E).
apply(prelude_lambda3_0,A,prelude_lambda3_0(A),B,B).
apply(prelude_lambda3_0(A),B,C,D,E):-hnf(prelude_lambda3_0(A,B),C,D,E).
apply(prelude_lambda4_0,A,prelude_lambda4_0(A),B,B).
apply(prelude_lambda4_0(A),B,C,D,E):-hnf(prelude_lambda4_0(A,B),C,D,E).
apply(splitline_0,A,B,C,D):-hnf(splitline_0(A),B,C,D).
apply(prelude_lambda5_0,A,prelude_lambda5_0(A),B,B).
apply(prelude_lambda5_0(A),B,C,D,E):-hnf(prelude_lambda5_0(A,B),C,D,E).
apply(prelude_lambda6_0,A,B,C,D):-hnf(prelude_lambda6_0(A),B,C,D).
apply(prelude_lambda7_0,A,B,C,D):-hnf(prelude_lambda7_0(A),B,C,D).
apply(isSpace_0,A,B,C,D):-hnf(isSpace_0(A),B,C,D).
apply(prelude_lambda9_0,A,B,C,D):-hnf(prelude_lambda9_0(A),B,C,D).
apply(prelude_lambda8_0,A,B,C,D):-hnf(prelude_lambda8_0(A),B,C,D).
apply(prelude_lambda10_0,A,prelude_lambda10_0(A),B,B).
apply(prelude_lambda10_0(A),B,C,D,E):-hnf(prelude_lambda10_0(A,B),C,D,E).
apply(prelude_lambda11_0,A,prelude_lambda11_0(A),B,B).
apply(prelude_lambda11_0(A),B,C,D,E):-hnf(prelude_lambda11_0(A,B),C,D,E).
apply(prelude_lambda12_0,A,prelude_lambda12_0(A),B,B).
apply(prelude_lambda12_0(A),B,C,D,E):-hnf(prelude_lambda12_0(A,B),C,D,E).
apply(prelude_lambda13_0,A,prelude_lambda13_0(A),B,B).
apply(prelude_lambda13_0(A),B,prelude_lambda13_0(A,B),C,C).
apply(prelude_lambda13_0(A,B),C,D,E,F):-hnf(prelude_lambda13_0(A,B,C),D,E,F).
apply(p_0,A,p_0(A),B,B).
apply(p_0(A),B,p_0(A,B),C,C).
apply(p_0(A,B),C,p_0(A,B,C),D,D).
apply(p_0(A,B,C),D,E,F,G):-hnf(p_0(A,B,C,D),E,F,G).
apply(prelude_lambda14_0,A,prelude_lambda14_0(A),B,B).
apply(prelude_lambda14_0(A),B,prelude_lambda14_0(A,B),C,C).
apply(prelude_lambda14_0(A,B),C,D,E,F):-hnf(prelude_lambda14_0(A,B,C),D,E,F).
apply(lambdafun_0,A,lambdafun_0(A),B,B).
apply(lambdafun_0(A),B,C,D,E):-hnf(lambdafun_0(A,B),C,D,E).
apply(lambdafun_2,A,lambdafun_2(A),B,B).
apply(lambdafun_2(A),B,lambdafun_2(A,B),C,C).
apply(lambdafun_2(A,B),C,D,E,F):-hnf(lambdafun_2(A,B,C),D,E,F).
apply(lambdafun_3,A,lambdafun_3(A),B,B).
apply(lambdafun_3(A),B,lambdafun_3(A,B),C,C).
apply(lambdafun_3(A,B),C,D,E,F):-hnf(lambdafun_3(A,B,C),D,E,F).
apply(lambdafun_1,A,lambdafun_1(A),B,B).
apply(lambdafun_1(A),B,lambdafun_1(A,B),C,C).
apply(lambdafun_1(A,B),C,D,E,F):-hnf(lambdafun_1(A,B,C),D,E,F).
apply(prelude_lambda15_0,A,prelude_lambda15_0(A),B,B).
apply(prelude_lambda15_0(A),B,prelude_lambda15_0(A,B),C,C).
apply(prelude_lambda15_0(A,B),C,D,E,F):-hnf(prelude_lambda15_0(A,B,C),D,E,F).
apply(evalall_0,A,B,C,D):-hnf(evalall_0(A),B,C,D).
apply(evalall2_0,A,B,C,D):-hnf(evalall2_0(A),B,C,D).
apply(evalall3_0,A,evalall3_0(A),B,B).
apply(evalall3_0(A),B,C,D,E):-hnf(evalall3_0(A,B),C,D,E).
apply(evalResult_0,A,B,C,D):-hnf(evalResult_0(A),B,C,D).
apply(prelude_lambda16_0,A,prelude_lambda16_0(A),B,B).
apply(prelude_lambda16_0(A),B,C,D,E):-hnf(prelude_lambda16_0(A,B),C,D,E).
apply(prelude_lambda17_0,A,prelude_lambda17_0(A),B,B).
apply(prelude_lambda17_0(A),B,prelude_lambda17_0(A,B),C,C).
apply(prelude_lambda17_0(A,B),C,D,E,F):-hnf(prelude_lambda17_0(A,B,C),D,E,F).
apply(bestHelp_0,A,bestHelp_0(A),B,B).
apply(bestHelp_0(A),B,bestHelp_0(A,B),C,C).
apply(bestHelp_0(A,B),C,bestHelp_0(A,B,C),D,D).
apply(bestHelp_0(A,B,C),D,E,F,G):-hnf(bestHelp_0(A,B,C,D),E,F,G).
apply(evalY_0,A,evalY_0(A),B,B).
apply(evalY_0(A),B,evalY_0(A,B),C,C).
apply(evalY_0(A,B),C,evalY_0(A,B,C),D,D).
apply(evalY_0(A,B,C),D,E,F,G):-hnf(evalY_0(A,B,C,D),E,F,G).
apply(evalX_0,A,evalX_0(A),B,B).
apply(evalX_0(A),B,evalX_0(A,B),C,C).
apply(evalX_0(A,B),C,evalX_0(A,B,C),D,D).
apply(evalX_0(A,B,C),D,evalX_0(A,B,C,D),E,E).
apply(evalX_0(A,B,C,D),E,F,G,H):-hnf(evalX_0(A,B,C,D,E),F,G,H).
apply(constrain_0,A,constrain_0(A),B,B).
apply(constrain_0(A),B,constrain_0(A,B),C,C).
apply(constrain_0(A,B),C,constrain_0(A,B,C),D,D).
apply(constrain_0(A,B,C),D,E,F,G):-hnf(constrain_0(A,B,C,D),E,F,G).
apply(prelude_lambda18_0,A,prelude_lambda18_0(A),B,B).
apply(prelude_lambda18_0(A),B,prelude_lambda18_0(A,B),C,C).
apply(prelude_lambda18_0(A,B),C,D,E,F):-hnf(prelude_lambda18_0(A,B,C),D,E,F).
apply(seq,A,seq(A),B,B).
apply(seq(A),B,C,D,E):-hnf(seq(A,B),C,D,E).
apply(error,A,B,C,D):-hnf(error(A),B,C,D).
apply(==,A,==(A),B,B).
apply(==(A),B,C,D,E):-hnf(A==B,C,D,E).
apply(ord,A,B,C,D):-hnf(ord(A),B,C,D).
apply(chr,A,B,C,D):-hnf(chr(A),B,C,D).
apply(+,A,+(A),B,B).
apply(+(A),B,C,D,E):-hnf(A+B,C,D,E).
apply(-,A,-(A),B,B).
apply(-(A),B,C,D,E):-hnf(A-B,C,D,E).
apply(*,A,*(A),B,B).
apply(*(A),B,C,D,E):-hnf(A*B,C,D,E).
apply(div,A,div(A),B,B).
apply(div(A),B,C,D,E):-hnf(div(A,B),C,D,E).
apply(mod,A,mod(A),B,B).
apply(mod(A),B,C,D,E):-hnf(A mod B,C,D,E).
apply(<,A,<(A),B,B).
apply(<(A),B,C,D,E):-hnf(A<B,C,D,E).
apply(>,A,>(A),B,B).
apply(>(A),B,C,D,E):-hnf(A>B,C,D,E).
apply(<=,A,<=(A),B,B).
apply(<=(A),B,C,D,E):-hnf(<=(A,B),C,D,E).
apply(>=,A,>=(A),B,B).
apply(>=(A),B,C,D,E):-hnf(A>=B,C,D,E).
apply(=:=,A,=:=(A),B,B).
apply(=:=(A),B,C,D,E):-hnf(A=:=B,C,D,E).
apply(&,A,&(A),B,B).
apply(&(A),B,C,D,E):-hnf(&(A,B),C,D,E).
apply(>>=,A,>>=(A),B,B).
apply(>>=(A),B,>>=(A,B),C,C).
apply(>>=(A,B),C,D,E,F):-hnf(>>=(A,B,C),D,E,F).
apply(return,A,return(A),B,B).
apply(return(A),B,C,D,E):-hnf(return(A,B),C,D,E).
apply(putChar,A,putChar(A),B,B).
apply(putChar(A),B,C,D,E):-hnf(putChar(A,B),C,D,E).
apply(getChar,A,B,C,D):-hnf(getChar(A),B,C,D).
apply(readFile,A,readFile(A),B,B).
apply(readFile(A),B,C,D,E):-hnf(readFile(A,B),C,D,E).
apply(writeFile,A,writeFile(A),B,B).
apply(writeFile(A),B,writeFile(A,B),C,C).
apply(writeFile(A,B),C,D,E,F):-hnf(writeFile(A,B,C),D,E,F).
apply(appendFile,A,appendFile(A),B,B).
apply(appendFile(A),B,appendFile(A,B),C,C).
apply(appendFile(A,B),C,D,E,F):-hnf(appendFile(A,B,C),D,E,F).
apply(catchFail,A,catchFail(A),B,B).
apply(catchFail(A),B,catchFail(A,B),C,C).
apply(catchFail(A,B),C,D,E,F):-hnf(catchFail(A,B,C),D,E,F).
apply(show,A,B,C,D):-hnf(show(A),B,C,D).
apply(try,A,B,C,D):-hnf(try(A),B,C,D).
apply(apply,A,apply(A),B,B).
apply(apply(A),B,C,D,E):-hnf(apply(A,B),C,D,E).
apply(cond,A,cond(A),B,B).
apply(cond(A),B,C,D,E):-hnf(cond(A,B),C,D,E).
apply(commit,A,B,C,D):-hnf(commit(A),B,C,D).
apply(>>,A,>>(A),B,B).
apply(>>(A),B,A>>B,C,C).
apply(A>>B,C,D,E,F):-hnf(>>(A,B,C),D,E,F).
apply(readFileContents,A,readFileContents(A),B,B).
apply(readFileContents(A),B,C,D,E):-hnf(readFileContents(A,B),C,D,E).
apply(writeFileContents,A,writeFileContents(A),B,B).
apply(writeFileContents(A),B,writeFileContents(A,B),C,C).
apply(writeFileContents(A,B),C,D,E,F):-hnf(writeFileContents(A,B,C),D,E,F).
apply(findall,A,B,C,D):-hnf(findall(A),B,C,D).
apply(findfirst,A,B,C,D):-hnf(findfirst(A),B,C,D).
apply('.',A,'.'(A),B,B).
apply('.'(A),B,C,D,E):-hnf([A|B],C,D,E).
apply('(,)',A,'(,)'(A),B,B).
apply('(,)'(A),B,C,D,E):-hnf('(,)'(A,B),C,D,E).
apply('(,,)',A,'(,,)'(A),B,B).
apply('(,,)'(A),B,'(,,)'(A,B),C,C).
apply('(,,)'(A,B),C,D,E,F):-hnf('(,,)'(A,B,C),D,E,F).
apply('(,,,)',A,'(,,,)'(A),B,B).
apply('(,,,)'(A),B,'(,,,)'(A,B),C,C).
apply('(,,,)'(A,B),C,'(,,,)'(A,B,C),D,D).
apply('(,,,)'(A,B,C),D,E,F,G):-hnf('(,,,)'(A,B,C,D),E,F,G).
apply('(,,,,)',A,'(,,,,)'(A),B,B).
apply('(,,,,)'(A),B,'(,,,,)'(A,B),C,C).
apply('(,,,,)'(A,B),C,'(,,,,)'(A,B,C),D,D).
apply('(,,,,)'(A,B,C),D,'(,,,,)'(A,B,C,D),E,E).
apply('(,,,,)'(A,B,C,D),E,F,G,H):-hnf('(,,,,)'(A,B,C,D,E),F,G,H).
apply('(,,,,,)',A,'(,,,,,)'(A),B,B).
apply('(,,,,,)'(A),B,'(,,,,,)'(A,B),C,C).
apply('(,,,,,)'(A,B),C,'(,,,,,)'(A,B,C),D,D).
apply('(,,,,,)'(A,B,C),D,'(,,,,,)'(A,B,C,D),E,E).
apply('(,,,,,)'(A,B,C,D),E,'(,,,,,)'(A,B,C,D,E),F,F).
apply('(,,,,,)'(A,B,C,D,E),F,G,H,I):-hnf('(,,,,,)'(A,B,C,D,E,F),G,H,I).
apply('(,,,,,,)',A,'(,,,,,,)'(A),B,B).
apply('(,,,,,,)'(A),B,'(,,,,,,)'(A,B),C,C).
apply('(,,,,,,)'(A,B),C,'(,,,,,,)'(A,B,C),D,D).
apply('(,,,,,,)'(A,B,C),D,'(,,,,,,)'(A,B,C,D),E,E).
apply('(,,,,,,)'(A,B,C,D),E,'(,,,,,,)'(A,B,C,D,E),F,F).
apply('(,,,,,,)'(A,B,C,D,E),F,'(,,,,,,)'(A,B,C,D,E,F),G,G).
apply('(,,,,,,)'(A,B,C,D,E,F),G,H,I,J):-hnf('(,,,,,,)'(A,B,C,D,E,F,G),H,I,J).
apply('(,,,,,,,)',A,'(,,,,,,,)'(A),B,B).
apply('(,,,,,,,)'(A),B,'(,,,,,,,)'(A,B),C,C).
apply('(,,,,,,,)'(A,B),C,'(,,,,,,,)'(A,B,C),D,D).
apply('(,,,,,,,)'(A,B,C),D,'(,,,,,,,)'(A,B,C,D),E,E).
apply('(,,,,,,,)'(A,B,C,D),E,'(,,,,,,,)'(A,B,C,D,E),F,F).
apply('(,,,,,,,)'(A,B,C,D,E),F,'(,,,,,,,)'(A,B,C,D,E,F),G,G).
apply('(,,,,,,,)'(A,B,C,D,E,F),G,'(,,,,,,,)'(A,B,C,D,E,F,G),H,H).
apply('(,,,,,,,)'(A,B,C,D,E,F,G),H,I,J,K):-hnf('(,,,,,,,)'(A,B,C,D,E,F,G,H),I,J,K).
apply('(,,,,,,,,)',A,'(,,,,,,,,)'(A),B,B).
apply('(,,,,,,,,)'(A),B,'(,,,,,,,,)'(A,B),C,C).
apply('(,,,,,,,,)'(A,B),C,'(,,,,,,,,)'(A,B,C),D,D).
apply('(,,,,,,,,)'(A,B,C),D,'(,,,,,,,,)'(A,B,C,D),E,E).
apply('(,,,,,,,,)'(A,B,C,D),E,'(,,,,,,,,)'(A,B,C,D,E),F,F).
apply('(,,,,,,,,)'(A,B,C,D,E),F,'(,,,,,,,,)'(A,B,C,D,E,F),G,G).
apply('(,,,,,,,,)'(A,B,C,D,E,F),G,'(,,,,,,,,)'(A,B,C,D,E,F,G),H,H).
apply('(,,,,,,,,)'(A,B,C,D,E,F,G),H,'(,,,,,,,,)'(A,B,C,D,E,F,G,H),I,I).
apply('(,,,,,,,,)'(A,B,C,D,E,F,G,H),I,J,K,L):-hnf('(,,,,,,,,)'(A,B,C,D,E,F,G,H,I),J,K,L).
apply('(,,,,,,,,,)',A,'(,,,,,,,,,)'(A),B,B).
apply('(,,,,,,,,,)'(A),B,'(,,,,,,,,,)'(A,B),C,C).
apply('(,,,,,,,,,)'(A,B),C,'(,,,,,,,,,)'(A,B,C),D,D).
apply('(,,,,,,,,,)'(A,B,C),D,'(,,,,,,,,,)'(A,B,C,D),E,E).
apply('(,,,,,,,,,)'(A,B,C,D),E,'(,,,,,,,,,)'(A,B,C,D,E),F,F).
apply('(,,,,,,,,,)'(A,B,C,D,E),F,'(,,,,,,,,,)'(A,B,C,D,E,F),G,G).
apply('(,,,,,,,,,)'(A,B,C,D,E,F),G,'(,,,,,,,,,)'(A,B,C,D,E,F,G),H,H).
apply('(,,,,,,,,,)'(A,B,C,D,E,F,G),H,'(,,,,,,,,,)'(A,B,C,D,E,F,G,H),I,I).
apply('(,,,,,,,,,)'(A,B,C,D,E,F,G,H),I,'(,,,,,,,,,)'(A,B,C,D,E,F,G,H,I),J,J).
apply('(,,,,,,,,,)'(A,B,C,D,E,F,G,H,I),J,K,L,M):-hnf('(,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J),K,L,M).
apply('(,,,,,,,,,,)',A,'(,,,,,,,,,,)'(A),B,B).
apply('(,,,,,,,,,,)'(A),B,'(,,,,,,,,,,)'(A,B),C,C).
apply('(,,,,,,,,,,)'(A,B),C,'(,,,,,,,,,,)'(A,B,C),D,D).
apply('(,,,,,,,,,,)'(A,B,C),D,'(,,,,,,,,,,)'(A,B,C,D),E,E).
apply('(,,,,,,,,,,)'(A,B,C,D),E,'(,,,,,,,,,,)'(A,B,C,D,E),F,F).
apply('(,,,,,,,,,,)'(A,B,C,D,E),F,'(,,,,,,,,,,)'(A,B,C,D,E,F),G,G).
apply('(,,,,,,,,,,)'(A,B,C,D,E,F),G,'(,,,,,,,,,,)'(A,B,C,D,E,F,G),H,H).
apply('(,,,,,,,,,,)'(A,B,C,D,E,F,G),H,'(,,,,,,,,,,)'(A,B,C,D,E,F,G,H),I,I).
apply('(,,,,,,,,,,)'(A,B,C,D,E,F,G,H),I,'(,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I),J,J).
apply('(,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I),J,'(,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J),K,K).
apply('(,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J),K,L,M,N):-hnf('(,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K),L,M,N).
apply('(,,,,,,,,,,,)',A,'(,,,,,,,,,,,)'(A),B,B).
apply('(,,,,,,,,,,,)'(A),B,'(,,,,,,,,,,,)'(A,B),C,C).
apply('(,,,,,,,,,,,)'(A,B),C,'(,,,,,,,,,,,)'(A,B,C),D,D).
apply('(,,,,,,,,,,,)'(A,B,C),D,'(,,,,,,,,,,,)'(A,B,C,D),E,E).
apply('(,,,,,,,,,,,)'(A,B,C,D),E,'(,,,,,,,,,,,)'(A,B,C,D,E),F,F).
apply('(,,,,,,,,,,,)'(A,B,C,D,E),F,'(,,,,,,,,,,,)'(A,B,C,D,E,F),G,G).
apply('(,,,,,,,,,,,)'(A,B,C,D,E,F),G,'(,,,,,,,,,,,)'(A,B,C,D,E,F,G),H,H).
apply('(,,,,,,,,,,,)'(A,B,C,D,E,F,G),H,'(,,,,,,,,,,,)'(A,B,C,D,E,F,G,H),I,I).
apply('(,,,,,,,,,,,)'(A,B,C,D,E,F,G,H),I,'(,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I),J,J).
apply('(,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I),J,'(,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J),K,K).
apply('(,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J),K,'(,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K),L,L).
apply('(,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K),L,M,N,O):-hnf('(,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K,L),M,N,O).
apply('(,,,,,,,,,,,,)',A,'(,,,,,,,,,,,,)'(A),B,B).
apply('(,,,,,,,,,,,,)'(A),B,'(,,,,,,,,,,,,)'(A,B),C,C).
apply('(,,,,,,,,,,,,)'(A,B),C,'(,,,,,,,,,,,,)'(A,B,C),D,D).
apply('(,,,,,,,,,,,,)'(A,B,C),D,'(,,,,,,,,,,,,)'(A,B,C,D),E,E).
apply('(,,,,,,,,,,,,)'(A,B,C,D),E,'(,,,,,,,,,,,,)'(A,B,C,D,E),F,F).
apply('(,,,,,,,,,,,,)'(A,B,C,D,E),F,'(,,,,,,,,,,,,)'(A,B,C,D,E,F),G,G).
apply('(,,,,,,,,,,,,)'(A,B,C,D,E,F),G,'(,,,,,,,,,,,,)'(A,B,C,D,E,F,G),H,H).
apply('(,,,,,,,,,,,,)'(A,B,C,D,E,F,G),H,'(,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H),I,I).
apply('(,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H),I,'(,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I),J,J).
apply('(,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I),J,'(,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J),K,K).
apply('(,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J),K,'(,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K),L,L).
apply('(,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K),L,'(,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K,L),M,M).
apply('(,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K,L),M,N,O,P):-hnf('(,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K,L,M),N,O,P).
apply('(,,,,,,,,,,,,,)',A,'(,,,,,,,,,,,,,)'(A),B,B).
apply('(,,,,,,,,,,,,,)'(A),B,'(,,,,,,,,,,,,,)'(A,B),C,C).
apply('(,,,,,,,,,,,,,)'(A,B),C,'(,,,,,,,,,,,,,)'(A,B,C),D,D).
apply('(,,,,,,,,,,,,,)'(A,B,C),D,'(,,,,,,,,,,,,,)'(A,B,C,D),E,E).
apply('(,,,,,,,,,,,,,)'(A,B,C,D),E,'(,,,,,,,,,,,,,)'(A,B,C,D,E),F,F).
apply('(,,,,,,,,,,,,,)'(A,B,C,D,E),F,'(,,,,,,,,,,,,,)'(A,B,C,D,E,F),G,G).
apply('(,,,,,,,,,,,,,)'(A,B,C,D,E,F),G,'(,,,,,,,,,,,,,)'(A,B,C,D,E,F,G),H,H).
apply('(,,,,,,,,,,,,,)'(A,B,C,D,E,F,G),H,'(,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H),I,I).
apply('(,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H),I,'(,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I),J,J).
apply('(,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I),J,'(,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J),K,K).
apply('(,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J),K,'(,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K),L,L).
apply('(,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K),L,'(,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K,L),M,M).
apply('(,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K,L),M,'(,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K,L,M),N,N).
apply('(,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K,L,M),N,O,P,Q):-hnf('(,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K,L,M,N),O,P,Q).
apply('(,,,,,,,,,,,,,,)',A,'(,,,,,,,,,,,,,,)'(A),B,B).
apply('(,,,,,,,,,,,,,,)'(A),B,'(,,,,,,,,,,,,,,)'(A,B),C,C).
apply('(,,,,,,,,,,,,,,)'(A,B),C,'(,,,,,,,,,,,,,,)'(A,B,C),D,D).
apply('(,,,,,,,,,,,,,,)'(A,B,C),D,'(,,,,,,,,,,,,,,)'(A,B,C,D),E,E).
apply('(,,,,,,,,,,,,,,)'(A,B,C,D),E,'(,,,,,,,,,,,,,,)'(A,B,C,D,E),F,F).
apply('(,,,,,,,,,,,,,,)'(A,B,C,D,E),F,'(,,,,,,,,,,,,,,)'(A,B,C,D,E,F),G,G).
apply('(,,,,,,,,,,,,,,)'(A,B,C,D,E,F),G,'(,,,,,,,,,,,,,,)'(A,B,C,D,E,F,G),H,H).
apply('(,,,,,,,,,,,,,,)'(A,B,C,D,E,F,G),H,'(,,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H),I,I).
apply('(,,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H),I,'(,,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I),J,J).
apply('(,,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I),J,'(,,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J),K,K).
apply('(,,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J),K,'(,,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K),L,L).
apply('(,,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K),L,'(,,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K,L),M,M).
apply('(,,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K,L),M,'(,,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K,L,M),N,N).
apply('(,,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K,L,M),N,'(,,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K,L,M,N),O,O).
apply('(,,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K,L,M,N),O,P,Q,R):-hnf('(,,,,,,,,,,,,,,)'(A,B,C,D,E,F,G,H,I,J,K,L,M,N,O),P,Q,R).
apply('Just',A,B,C,D):-hnf('Just'(A),B,C,D).
apply('Left',A,B,C,D):-hnf('Left'(A),B,C,D).
apply('Right',A,B,C,D):-hnf('Right'(A),B,C,D).
apply('$stream',A,B,C,D):-hnf('$stream'(A),B,C,D).

:-block constrEq(?,?,-,?).
constrEq(A,B,C,D):-hnf(A,E,C,F),hnf(B,G,F,H),constrEqHnf(E,G,H,D).

:-block constrEqHnf(?,?,-,?).
constrEqHnf(A,B,C,D):-var(A),!,bindTryNf(A,B,C,D).
constrEqHnf(A,B,C,D):-var(B),!,bindTryNf(B,A,C,D).
constrEqHnf(A,B,C,D):-integer(A),!,A=B,C=D.
constrEqHnf(A,B,C,D):-float(A),!,{A=B},C=D.
constrEqHnf(A,B,C,D):-functor(A,E,F),functor(B,G,H),E==G,F==H,genConstrEqHnfBody(1,F,A,B,I),hnf(I,J,C,D).

genConstrEqHnfBody(A,B,C,D,success):-A>B,!.
genConstrEqHnfBody(A,B,C,D,E=:=F):-A=B,!,arg(A,C,E),arg(A,D,F).
genConstrEqHnfBody(A,B,C,D,&(E=:=F,G)):-arg(A,C,E),arg(A,D,F),H is A+1,genConstrEqHnfBody(H,B,C,D,G).

bindTryNf(A,B,C,D):-nf(B,E,C,F),(nonvar(F)->bindDirect(A,E,F,D);bind(A,B,C,D)).
bindDirect(A,B,C,D):-var(B),!,A=B,C=D.
bindDirect(A,B,C,D):-occursNot(A,B),A=B,C=D.

bind(A,B,C,D):-var(B),!,A=B,C=D.
bind(A,B,C,D):-integer(B),!,A=B,C=D.
bind(A,B,C,D):-float(B),!,{A=B},C=D.
bind(A,B,C,D):-functor(B,E,F),occursNotArgs(1,F,A,B),functor(A,E,F),bindArgs(1,F,A,B,C,D).

occursNotArgs(A,B,C,D):-A>B,!.
occursNotArgs(A,B,C,D):-arg(A,D,E),occursNot(C,E),F is A+1,occursNotArgs(F,B,C,D).

bindArgs(A,B,C,D,E,F):-A>B,!,E=F.
bindArgs(A,B,C,D,E,F):-arg(A,C,G),arg(A,D,H),hnf(H,I,E,J),bind(G,I,J,K),L is A+1,bindArgs(L,B,C,D,K,F).

occursNot(A,B):-var(B),!,A\==B.
occursNot(A,'()'):-!.
occursNot(A,[]):-!.
occursNot(A,[B|C]):-!,occursNot(A,B),occursNot(A,C).
occursNot(A,'True'):-!.
occursNot(A,'False'):-!.
occursNot(A,'(,)'(B,C)):-!,occursNot(A,B),occursNot(A,C).
occursNot(A,'(,,)'(B,C,D)):-!,occursNot(A,B),occursNot(A,C),occursNot(A,D).
occursNot(A,'(,,,)'(B,C,D,E)):-!,occursNot(A,B),occursNot(A,C),occursNot(A,D),occursNot(A,E).
occursNot(A,'(,,,,)'(B,C,D,E,F)):-!,occursNot(A,B),occursNot(A,C),occursNot(A,D),occursNot(A,E),occursNot(A,F).
occursNot(A,'(,,,,,)'(B,C,D,E,F,G)):-!,occursNot(A,B),occursNot(A,C),occursNot(A,D),occursNot(A,E),occursNot(A,F),occursNot(A,G).
occursNot(A,'(,,,,,,)'(B,C,D,E,F,G,H)):-!,occursNot(A,B),occursNot(A,C),occursNot(A,D),occursNot(A,E),occursNot(A,F),occursNot(A,G),occursNot(A,H).
occursNot(A,'(,,,,,,,)'(B,C,D,E,F,G,H,I)):-!,occursNot(A,B),occursNot(A,C),occursNot(A,D),occursNot(A,E),occursNot(A,F),occursNot(A,G),occursNot(A,H),occursNot(A,I).
occursNot(A,'(,,,,,,,,)'(B,C,D,E,F,G,H,I,J)):-!,occursNot(A,B),occursNot(A,C),occursNot(A,D),occursNot(A,E),occursNot(A,F),occursNot(A,G),occursNot(A,H),occursNot(A,I),occursNot(A,J).
occursNot(A,'(,,,,,,,,,)'(B,C,D,E,F,G,H,I,J,K)):-!,occursNot(A,B),occursNot(A,C),occursNot(A,D),occursNot(A,E),occursNot(A,F),occursNot(A,G),occursNot(A,H),occursNot(A,I),occursNot(A,J),occursNot(A,K).
occursNot(A,'(,,,,,,,,,,)'(B,C,D,E,F,G,H,I,J,K,L)):-!,occursNot(A,B),occursNot(A,C),occursNot(A,D),occursNot(A,E),occursNot(A,F),occursNot(A,G),occursNot(A,H),occursNot(A,I),occursNot(A,J),occursNot(A,K),occursNot(A,L).
occursNot(A,'(,,,,,,,,,,,)'(B,C,D,E,F,G,H,I,J,K,L,M)):-!,occursNot(A,B),occursNot(A,C),occursNot(A,D),occursNot(A,E),occursNot(A,F),occursNot(A,G),occursNot(A,H),occursNot(A,I),occursNot(A,J),occursNot(A,K),occursNot(A,L),occursNot(A,M).
occursNot(A,'(,,,,,,,,,,,,)'(B,C,D,E,F,G,H,I,J,K,L,M,N)):-!,occursNot(A,B),occursNot(A,C),occursNot(A,D),occursNot(A,E),occursNot(A,F),occursNot(A,G),occursNot(A,H),occursNot(A,I),occursNot(A,J),occursNot(A,K),occursNot(A,L),occursNot(A,M),occursNot(A,N).
occursNot(A,'(,,,,,,,,,,,,,)'(B,C,D,E,F,G,H,I,J,K,L,M,N,O)):-!,occursNot(A,B),occursNot(A,C),occursNot(A,D),occursNot(A,E),occursNot(A,F),occursNot(A,G),occursNot(A,H),occursNot(A,I),occursNot(A,J),occursNot(A,K),occursNot(A,L),occursNot(A,M),occursNot(A,N),occursNot(A,O).
occursNot(A,'(,,,,,,,,,,,,,,)'(B,C,D,E,F,G,H,I,J,K,L,M,N,O,P)):-!,occursNot(A,B),occursNot(A,C),occursNot(A,D),occursNot(A,E),occursNot(A,F),occursNot(A,G),occursNot(A,H),occursNot(A,I),occursNot(A,J),occursNot(A,K),occursNot(A,L),occursNot(A,M),occursNot(A,N),occursNot(A,O),occursNot(A,P).
occursNot(A,'LT'):-!.
occursNot(A,'EQ'):-!.
occursNot(A,'GT'):-!.
occursNot(A,'Nothing'):-!.
occursNot(A,'Just'(B)):-!,occursNot(A,B).
occursNot(A,'Left'(B)):-!,occursNot(A,B).
occursNot(A,'Right'(B)):-!,occursNot(A,B).
occursNot(A,'$stream'(B)):-!,occursNot(A,B).
occursNot(A,B).

:-block boolEq(?,?,?,-,?).
boolEq(A,B,C,D,E):-hnf(A,F,D,G),hnf(B,H,G,I),boolEqHnf(F,H,C,I,E).

:-block boolEqHnf(-,?,?,?,?),boolEqHnf(?,-,?,?,?),boolEqHnf(?,?,?,-,?).
boolEqHnf(A,B,C,D,E):-number(A),!,(A==B->C='True';C='False'),D=E.
boolEqHnf(A,B,C,D,E):-functor(A,F,G),functor(B,H,I),(F==H,G==I,genBoolEqHnfBody(1,G,A,B,J)->hnf(J,C,D,E);C='False',D=E).

genBoolEqHnfBody(A,B,C,D,'True'):-A>B,!.
genBoolEqHnfBody(A,B,C,D,&&(E==F,G)):-arg(A,C,E),arg(A,D,F),H is A+1,genBoolEqHnfBody(H,B,C,D,G).

:-block nf(?,?,-,?).
nf(A,B,C,D):-hnf(A,E,C,F),nfHnf(E,B,F,D).

:-block nfHnf(?,?,-,?).
nfHnf(A,B,C,D):-var(A),!,B=A,C=D.
nfHnf(A,A,B,C):-number(A),!,B=C.
nfHnf(A,B,C,D):-functor(A,E,F),functor(B,E,F),nfHnfArgs(1,F,A,B,C,D).

nfHnfArgs(A,B,C,D,E,F):-A>B,!,E=F.
nfHnfArgs(A,B,C,D,E,F):-arg(A,C,G),arg(A,D,H),nf(G,H,E,I),J is A+1,nfHnfArgs(J,B,C,D,I,F).

propagateShare(A,A):-var(A),!.
propagateShare('Ports.internalPort'(A,B,C,D),'Ports.internalPort'(A,B,C,D)):-!.
propagateShare('$stream'(A),'$stream'(A)):-!.
propagateShare(A,A):-number(A),!.
propagateShare(A,B):-functor(A,C,D),functor(B,C,D),propagateShareArgs(1,D,A,B).

propagateShareArgs(A,B,C,D):-A>B,!.
propagateShareArgs(A,B,C,D):-arg(A,C,E),arg(A,D,F),propagateShareArg(E,F),G is A+1,propagateShareArgs(G,B,C,D).

propagateShareArg(A,A):-nonvar(A),functor(A,'Ports.internalPort',4),!.
propagateShareArg(A,B):-makeShare(A,B).

makeShare(A,A):-var(A),!.
makeShare(A,A):-A=share(B),!.
makeShare(A,share(B)):-create_mutable(A,B).

translate(:,'.').
translate('.','prelude..').
translate('()','()').
translate([],[]).
translate(:,:).
translate('True','True').
translate('False','False').
translate('(,)','(,)').
translate('(,,)','(,,)').
translate('(,,,)','(,,,)').
translate('(,,,,)','(,,,,)').
translate('(,,,,,)','(,,,,,)').
translate('(,,,,,,)','(,,,,,,)').
translate('(,,,,,,,)','(,,,,,,,)').
translate('(,,,,,,,,)','(,,,,,,,,)').
translate('(,,,,,,,,,)','(,,,,,,,,,)').
translate('(,,,,,,,,,,)','(,,,,,,,,,,)').
translate('(,,,,,,,,,,,)','(,,,,,,,,,,,)').
translate('(,,,,,,,,,,,,)','(,,,,,,,,,,,,)').
translate('(,,,,,,,,,,,,,)','(,,,,,,,,,,,,,)').
translate('(,,,,,,,,,,,,,,)','(,,,,,,,,,,,,,,)').
translate('LT','LT').
translate('EQ','EQ').
translate('GT','GT').
translate('Nothing','Nothing').
translate('Just','Just').
translate('Left','Left').
translate('Right','Right').
translate(seq,seq).
translate(error,error).
translate(==,==).
translate(ord,ord).
translate(chr,chr).
translate(+,+).
translate(-,-).
translate(*,*).
translate(div,div).
translate(mod,mod).
translate(<,<).
translate(>,>).
translate(<=,<=).
translate(>=,>=).
translate(=:=,=:=).
translate(success,success).
translate(&,&).
translate(>>=,>>=).
translate(return,return).
translate(putChar,putChar).
translate(getChar,getChar).
translate(readFile,readFile).
translate(writeFile,writeFile).
translate(appendFile,appendFile).
translate(catchFail,catchFail).
translate(show,show).
translate(try,try).
translate(apply,apply).
translate(cond,cond).
translate(commit,commit).
translate('.','.').
translate(id,id).
translate(const,const).
translate(curry,curry).
translate(uncurry,uncurry).
translate(flip,flip).
translate(until,until).
translate($,$).
translate('$!','$!').
translate(failed,failed).
translate(&&,&&).
translate('||','||').
translate(not,not).
translate(otherwise,otherwise).
translate(if_then_else,if_then_else).
translate(/=,/=).
translate(fst,fst).
translate(snd,snd).
translate(head,head).
translate(tail,tail).
translate(null,null).
translate(++,++).
translate(length,length).
translate('!!','!!').
translate(map,map).
translate(foldl,foldl).
translate(foldl1,foldl1).
translate(foldr,foldr).
translate(foldr1,foldr1).
translate(filter,filter).
translate(zip,zip).
translate(zip3,zip3).
translate(zipWith,zipWith).
translate(zipWith3,zipWith3).
translate(unzip,unzip).
translate(unzip3,unzip3).
translate(concat,concat).
translate(concatMap,concatMap).
translate(iterate,iterate).
translate(repeat,repeat).
translate(replicate,replicate).
translate(take,take).
translate(drop,drop).
translate(splitAt,splitAt).
translate(takeWhile,takeWhile).
translate(dropWhile,dropWhile).
translate(span,span).
translate(break,break).
translate(lines,lines).
translate(unlines,unlines).
translate(words,words).
translate(unwords,unwords).
translate(reverse,reverse).
translate(and,and).
translate(or,or).
translate(any,any).
translate(all,all).
translate(elem,elem).
translate(notElem,notElem).
translate(lookup,lookup).
translate(enumFrom,enumFrom).
translate(enumFromThen,enumFromThen).
translate(enumFromTo,enumFromTo).
translate(enumFromThenTo,enumFromThenTo).
translate(negate,negate).
translate(&>,&>).
translate(maybe,maybe).
translate(either,either).
translate(>>,>>).
translate(done,done).
translate(putStr,putStr).
translate(putStrLn,putStrLn).
translate(getLine,getLine).
translate(print,print).
translate(doSolve,doSolve).
translate(sequenceIO,sequenceIO).
translate(sequenceIO_,sequenceIO_).
translate(mapIO,mapIO).
translate(mapIO_,mapIO_).
translate(?,?).
translate(inject,inject).
translate(solveAll,solveAll).
translate(solveAll2,solveAll2).
translate(once,once).
translate(tryone,tryone).
translate(one,one).
translate(condSearch,condSearch).
translate(best,best).
translate(findall,findall).
translate(findfirst,findfirst).
translate(browse,browse).
translate(browseList,browseList).
translate(unpack,unpack).
translate('PEVAL','PEVAL').

functiontype(seq,'FuncType'(A,'FuncType'(B,B))).
functiontype(error,'FuncType'('TCons'([],['TCons'('Char',[])]),A)).
functiontype(==,'FuncType'(A,'FuncType'(A,'TCons'('Bool',[])))).
functiontype(ord,'FuncType'('TCons'('Char',[]),'TCons'('Int',[]))).
functiontype(chr,'FuncType'('TCons'('Int',[]),'TCons'('Char',[]))).
functiontype(+,'FuncType'('TCons'('Int',[]),'FuncType'('TCons'('Int',[]),'TCons'('Int',[])))).
functiontype(-,'FuncType'('TCons'('Int',[]),'FuncType'('TCons'('Int',[]),'TCons'('Int',[])))).
functiontype(*,'FuncType'('TCons'('Int',[]),'FuncType'('TCons'('Int',[]),'TCons'('Int',[])))).
functiontype(div,'FuncType'('TCons'('Int',[]),'FuncType'('TCons'('Int',[]),'TCons'('Int',[])))).
functiontype(mod,'FuncType'('TCons'('Int',[]),'FuncType'('TCons'('Int',[]),'TCons'('Int',[])))).
functiontype(<,'FuncType'('TCons'('Int',[]),'FuncType'('TCons'('Int',[]),'TCons'('Bool',[])))).
functiontype(>,'FuncType'('TCons'('Int',[]),'FuncType'('TCons'('Int',[]),'TCons'('Bool',[])))).
functiontype(<=,'FuncType'('TCons'('Int',[]),'FuncType'('TCons'('Int',[]),'TCons'('Bool',[])))).
functiontype(>=,'FuncType'('TCons'('Int',[]),'FuncType'('TCons'('Int',[]),'TCons'('Bool',[])))).
functiontype(=:=,'FuncType'(A,'FuncType'(A,'TCons'('Success',[])))).
functiontype(success,'TCons'('Success',[])).
functiontype(&,'FuncType'('TCons'('Success',[]),'FuncType'('TCons'('Success',[]),'TCons'('Success',[])))).
functiontype(>>=,'FuncType'('TCons'('IO',[A]),'FuncType'('FuncType'(A,'TCons'('IO',[B])),'TCons'('IO',[B])))).
functiontype(return,'FuncType'(A,'TCons'('IO',[A]))).
functiontype(putChar,'FuncType'('TCons'('Char',[]),'TCons'('IO',['TCons'('()',[])]))).
functiontype(getChar,'TCons'('IO',['TCons'('Char',[])])).
functiontype(readFile,'FuncType'('TCons'([],['TCons'('Char',[])]),'TCons'('IO',['TCons'([],['TCons'('Char',[])])]))).
functiontype(writeFile,'FuncType'('TCons'([],['TCons'('Char',[])]),'FuncType'('TCons'([],['TCons'('Char',[])]),'TCons'('IO',['TCons'('()',[])])))).
functiontype(appendFile,'FuncType'('TCons'([],['TCons'('Char',[])]),'FuncType'('TCons'([],['TCons'('Char',[])]),'TCons'('IO',['TCons'('()',[])])))).
functiontype(catchFail,'FuncType'('TCons'('IO',[A]),'FuncType'('TCons'('IO',[A]),'TCons'('IO',[A])))).
functiontype(show,'FuncType'(A,'TCons'([],['TCons'('Char',[])]))).
functiontype(try,'FuncType'('FuncType'(A,'TCons'('Success',[])),'TCons'([],['FuncType'(A,'TCons'('Success',[]))]))).
functiontype(apply,'FuncType'('FuncType'(A,B),'FuncType'(A,B))).
functiontype(cond,'FuncType'('TCons'('Success',[]),'FuncType'(A,A))).
functiontype(commit,'FuncType'(A,A)).
functiontype('prelude..','FuncType'('FuncType'(A,B),'FuncType'('FuncType'(C,A),'FuncType'(C,B)))).
functiontype(id,'FuncType'(A,A)).
functiontype(const,'FuncType'(A,'FuncType'(B,A))).
functiontype(curry,'FuncType'('FuncType'('TCons'('(,)',[A,B]),C),'FuncType'(A,'FuncType'(B,C)))).
functiontype(uncurry,'FuncType'('FuncType'(A,'FuncType'(B,C)),'FuncType'('TCons'('(,)',[A,B]),C))).
functiontype(flip,'FuncType'('FuncType'(A,'FuncType'(B,C)),'FuncType'(B,'FuncType'(A,C)))).
functiontype(until,'FuncType'('FuncType'(A,'TCons'('Bool',[])),'FuncType'('FuncType'(A,A),'FuncType'(A,A)))).
functiontype($,'FuncType'('FuncType'(A,B),'FuncType'(A,B))).
functiontype('$!','FuncType'('FuncType'(A,B),'FuncType'(A,B))).
functiontype(failed,A).
functiontype(&&,'FuncType'('TCons'('Bool',[]),'FuncType'('TCons'('Bool',[]),'TCons'('Bool',[])))).
functiontype('||','FuncType'('TCons'('Bool',[]),'FuncType'('TCons'('Bool',[]),'TCons'('Bool',[])))).
functiontype(not,'FuncType'('TCons'('Bool',[]),'TCons'('Bool',[]))).
functiontype(otherwise,'TCons'('Bool',[])).
functiontype(if_then_else,'FuncType'('TCons'('Bool',[]),'FuncType'(A,'FuncType'(A,A)))).
functiontype(/=,'FuncType'(A,'FuncType'(A,'TCons'('Bool',[])))).
functiontype(fst,'FuncType'('TCons'('(,)',[A,B]),A)).
functiontype(snd,'FuncType'('TCons'('(,)',[A,B]),B)).
functiontype(head,'FuncType'('TCons'([],[A]),A)).
functiontype(tail,'FuncType'('TCons'([],[A]),'TCons'([],[A]))).
functiontype(null,'FuncType'('TCons'([],[A]),'TCons'('Bool',[]))).
functiontype(++,'FuncType'('TCons'([],[A]),'FuncType'('TCons'([],[A]),'TCons'([],[A])))).
functiontype(length,'FuncType'('TCons'([],[A]),'TCons'('Int',[]))).
functiontype('!!','FuncType'('TCons'([],[A]),'FuncType'('TCons'('Int',[]),A))).
functiontype(map,'FuncType'('FuncType'(A,B),'FuncType'('TCons'([],[A]),'TCons'([],[B])))).
functiontype(foldl,'FuncType'('FuncType'(A,'FuncType'(B,A)),'FuncType'(A,'FuncType'('TCons'([],[B]),A)))).
functiontype(foldl1,'FuncType'('FuncType'(A,'FuncType'(A,A)),'FuncType'('TCons'([],[A]),A))).
functiontype(foldr,'FuncType'('FuncType'(A,'FuncType'(B,B)),'FuncType'(B,'FuncType'('TCons'([],[A]),B)))).
functiontype(foldr1,'FuncType'('FuncType'(A,'FuncType'(A,A)),'FuncType'('TCons'([],[A]),A))).
functiontype(filter,'FuncType'('FuncType'(A,'TCons'('Bool',[])),'FuncType'('TCons'([],[A]),'TCons'([],[A])))).
functiontype(zip,'FuncType'('TCons'([],[A]),'FuncType'('TCons'([],[B]),'TCons'([],['TCons'('(,)',[A,B])])))).
functiontype(zip3,'FuncType'('TCons'([],[A]),'FuncType'('TCons'([],[B]),'FuncType'('TCons'([],[C]),'TCons'([],['TCons'('(,,)',[A,B,C])]))))).
functiontype(zipWith,'FuncType'('FuncType'(A,'FuncType'(B,C)),'FuncType'('TCons'([],[A]),'FuncType'('TCons'([],[B]),'TCons'([],[C]))))).
functiontype(zipWith3,'FuncType'('FuncType'(A,'FuncType'(B,'FuncType'(C,D))),'FuncType'('TCons'([],[A]),'FuncType'('TCons'([],[B]),'FuncType'('TCons'([],[C]),'TCons'([],[D])))))).
functiontype(unzip,'FuncType'('TCons'([],['TCons'('(,)',[A,B])]),'TCons'('(,)',['TCons'([],[A]),'TCons'([],[B])]))).
functiontype(unzip3,'FuncType'('TCons'([],['TCons'('(,,)',[A,B,C])]),'TCons'('(,,)',['TCons'([],[A]),'TCons'([],[B]),'TCons'([],[C])]))).
functiontype(concat,'FuncType'('TCons'([],['TCons'([],[A])]),'TCons'([],[A]))).
functiontype(concatMap,'FuncType'('FuncType'(A,'TCons'([],[B])),'FuncType'('TCons'([],[A]),'TCons'([],[B])))).
functiontype(iterate,'FuncType'('FuncType'(A,A),'FuncType'(A,'TCons'([],[A])))).
functiontype(repeat,'FuncType'(A,'TCons'([],[A]))).
functiontype(replicate,'FuncType'('TCons'('Int',[]),'FuncType'(A,'TCons'([],[A])))).
functiontype(take,'FuncType'('TCons'('Int',[]),'FuncType'('TCons'([],[A]),'TCons'([],[A])))).
functiontype(drop,'FuncType'('TCons'('Int',[]),'FuncType'('TCons'([],[A]),'TCons'([],[A])))).
functiontype(splitAt,'FuncType'('TCons'('Int',[]),'FuncType'('TCons'([],[A]),'TCons'('(,)',['TCons'([],[A]),'TCons'([],[A])])))).
functiontype(takeWhile,'FuncType'('FuncType'(A,'TCons'('Bool',[])),'FuncType'('TCons'([],[A]),'TCons'([],[A])))).
functiontype(dropWhile,'FuncType'('FuncType'(A,'TCons'('Bool',[])),'FuncType'('TCons'([],[A]),'TCons'([],[A])))).
functiontype(span,'FuncType'('FuncType'(A,'TCons'('Bool',[])),'FuncType'('TCons'([],[A]),'TCons'('(,)',['TCons'([],[A]),'TCons'([],[A])])))).
functiontype(break,'FuncType'('FuncType'(A,'TCons'('Bool',[])),'FuncType'('TCons'([],[A]),'TCons'('(,)',['TCons'([],[A]),'TCons'([],[A])])))).
functiontype(lines,'FuncType'('TCons'([],['TCons'('Char',[])]),'TCons'([],['TCons'([],['TCons'('Char',[])])]))).
functiontype(unlines,'FuncType'('TCons'([],['TCons'([],['TCons'('Char',[])])]),'TCons'([],['TCons'('Char',[])]))).
functiontype(words,'FuncType'('TCons'([],['TCons'('Char',[])]),'TCons'([],['TCons'([],['TCons'('Char',[])])]))).
functiontype(unwords,'FuncType'('TCons'([],['TCons'([],['TCons'('Char',[])])]),'TCons'([],['TCons'('Char',[])]))).
functiontype(reverse,'FuncType'('TCons'([],[A]),'TCons'([],[A]))).
functiontype(and,'FuncType'('TCons'([],['TCons'('Bool',[])]),'TCons'('Bool',[]))).
functiontype(or,'FuncType'('TCons'([],['TCons'('Bool',[])]),'TCons'('Bool',[]))).
functiontype(any,'FuncType'('FuncType'(A,'TCons'('Bool',[])),'FuncType'('TCons'([],[A]),'TCons'('Bool',[])))).
functiontype(all,'FuncType'('FuncType'(A,'TCons'('Bool',[])),'FuncType'('TCons'([],[A]),'TCons'('Bool',[])))).
functiontype(elem,'FuncType'(A,'FuncType'('TCons'([],[A]),'TCons'('Bool',[])))).
functiontype(notElem,'FuncType'(A,'FuncType'('TCons'([],[A]),'TCons'('Bool',[])))).
functiontype(lookup,'FuncType'(A,'FuncType'('TCons'([],['TCons'('(,)',[A,B])]),'TCons'('Maybe',[B])))).
functiontype(enumFrom,'FuncType'('TCons'('Int',[]),'TCons'([],['TCons'('Int',[])]))).
functiontype(enumFromThen,'FuncType'('TCons'('Int',[]),'FuncType'('TCons'('Int',[]),'TCons'([],['TCons'('Int',[])])))).
functiontype(enumFromTo,'FuncType'('TCons'('Int',[]),'FuncType'('TCons'('Int',[]),'TCons'([],['TCons'('Int',[])])))).
functiontype(enumFromThenTo,'FuncType'('TCons'('Int',[]),'FuncType'('TCons'('Int',[]),'FuncType'('TCons'('Int',[]),'TCons'([],['TCons'('Int',[])]))))).
functiontype(negate,'FuncType'('TCons'('Int',[]),'TCons'('Int',[]))).
functiontype(&>,'FuncType'('TCons'('Success',[]),'FuncType'(A,A))).
functiontype(maybe,'FuncType'(A,'FuncType'('FuncType'(B,A),'FuncType'('TCons'('Maybe',[B]),A)))).
functiontype(either,'FuncType'('FuncType'(A,B),'FuncType'('FuncType'(C,B),'FuncType'('TCons'('Either',[A,C]),B)))).
functiontype(>>,'FuncType'('TCons'('IO',[A]),'FuncType'('TCons'('IO',[B]),'TCons'('IO',[B])))).
functiontype(done,'TCons'('IO',['TCons'('()',[])])).
functiontype(putStr,'FuncType'('TCons'([],['TCons'('Char',[])]),'TCons'('IO',['TCons'('()',[])]))).
functiontype(putStrLn,'FuncType'('TCons'([],['TCons'('Char',[])]),'TCons'('IO',['TCons'('()',[])]))).
functiontype(getLine,'TCons'('IO',['TCons'([],['TCons'('Char',[])])])).
functiontype(print,'FuncType'(A,'TCons'('IO',['TCons'('()',[])]))).
functiontype(doSolve,'FuncType'('TCons'('Success',[]),'TCons'('IO',['TCons'('()',[])]))).
functiontype(sequenceIO,'FuncType'('TCons'([],['TCons'('IO',[A])]),'TCons'('IO',['TCons'([],[A])]))).
functiontype(sequenceIO_,'FuncType'('TCons'([],['TCons'('IO',[A])]),'TCons'('IO',['TCons'('()',[])]))).
functiontype(mapIO,'FuncType'('FuncType'(A,'TCons'('IO',[B])),'FuncType'('TCons'([],[A]),'TCons'('IO',['TCons'([],[B])])))).
functiontype(mapIO_,'FuncType'('FuncType'(A,'TCons'('IO',[B])),'FuncType'('TCons'([],[A]),'TCons'('IO',['TCons'('()',[])])))).
functiontype(?,'FuncType'(A,'FuncType'(A,A))).
functiontype(inject,'FuncType'('FuncType'(A,'TCons'('Success',[])),'FuncType'('FuncType'(A,'TCons'('Success',[])),'FuncType'(A,'TCons'('Success',[]))))).
functiontype(solveAll,'FuncType'('FuncType'(A,'TCons'('Success',[])),'TCons'([],['FuncType'(A,'TCons'('Success',[]))]))).
functiontype(solveAll2,'FuncType'('FuncType'(A,'TCons'('Success',[])),'TCons'([],['FuncType'(A,'TCons'('Success',[]))]))).
functiontype(once,'FuncType'('FuncType'(A,'TCons'('Success',[])),'FuncType'(A,'TCons'('Success',[])))).
functiontype(tryone,'FuncType'('FuncType'(A,'TCons'('Success',[])),A)).
functiontype(one,'FuncType'('FuncType'(A,'TCons'('Success',[])),'TCons'([],['FuncType'(A,'TCons'('Success',[]))]))).
functiontype(condSearch,'FuncType'('FuncType'(A,'TCons'('Success',[])),'FuncType'('FuncType'(A,'TCons'('Bool',[])),'TCons'([],['FuncType'(A,'TCons'('Success',[]))])))).
functiontype(best,'FuncType'('FuncType'(A,'TCons'('Success',[])),'FuncType'('FuncType'(A,'FuncType'(A,'TCons'('Bool',[]))),'TCons'([],['FuncType'(A,'TCons'('Success',[]))])))).
functiontype(findall,'FuncType'('FuncType'(A,'TCons'('Success',[])),'TCons'([],[A]))).
functiontype(findfirst,'FuncType'('FuncType'(A,'TCons'('Success',[])),A)).
functiontype(browse,'FuncType'('FuncType'(A,'TCons'('Success',[])),'TCons'('IO',['TCons'('()',[])]))).
functiontype(browseList,'FuncType'('TCons'([],['FuncType'(A,'TCons'('Success',[]))]),'TCons'('IO',['TCons'('()',[])]))).
functiontype(unpack,'FuncType'('FuncType'(A,'TCons'('Success',[])),A)).
functiontype('PEVAL','FuncType'(A,A)).
functiontype(prelude_lambda0_0,'FuncType'('FuncType'(A,B),'FuncType'('FuncType'(C,A),'FuncType'(C,B)))).
functiontype(prelude_lambda1_0,'FuncType'(A,'FuncType'(B,'FuncType'('TCons'('(,)',['TCons'([],[A]),'TCons'([],[B])]),'TCons'('(,)',['TCons'([],[A]),'TCons'([],[B])]))))).
functiontype(prelude_lambda2_0,'FuncType'(A,'FuncType'(B,'FuncType'(C,'FuncType'('TCons'('(,,)',['TCons'([],[A]),'TCons'([],[B]),'TCons'([],[C])]),'TCons'('(,,)',['TCons'([],[A]),'TCons'([],[B]),'TCons'([],[C])])))))).
functiontype(takep_0,'FuncType'('TCons'('Int',[]),'FuncType'('TCons'([],[A]),'TCons'([],[A])))).
functiontype(dropp_0,'FuncType'('TCons'('Int',[]),'FuncType'('TCons'([],[A]),'TCons'([],[A])))).
functiontype(splitAtp_0,'FuncType'('TCons'('Int',[]),'FuncType'('TCons'([],[A]),'TCons'('(,)',['TCons'([],[A]),'TCons'([],[A])])))).
functiontype(prelude_lambda3_0,'FuncType'(A,'FuncType'('TCons'('(,)',['TCons'([],[A]),'TCons'([],[A])]),'TCons'('(,)',['TCons'([],[A]),'TCons'([],[A])])))).
functiontype(prelude_lambda4_0,'FuncType'(A,'FuncType'('TCons'('(,)',['TCons'([],[A]),'TCons'([],[A])]),'TCons'('(,)',['TCons'([],[A]),'TCons'([],[A])])))).
functiontype(splitline_0,'FuncType'('TCons'([],['TCons'('Char',[])]),'TCons'('(,)',['TCons'([],['TCons'('Char',[])]),'TCons'([],['TCons'('Char',[])])]))).
functiontype(prelude_lambda5_0,'FuncType'('TCons'('Char',[]),'FuncType'('TCons'('(,)',['TCons'([],['TCons'('Char',[])]),'TCons'([],['TCons'('Char',[])])]),'TCons'('(,)',['TCons'([],['TCons'('Char',[])]),'TCons'([],['TCons'('Char',[])])])))).
functiontype(prelude_lambda6_0,'FuncType'('TCons'('(,)',['TCons'([],['TCons'('Char',[])]),'TCons'([],['TCons'('Char',[])])]),'TCons'([],['TCons'([],['TCons'('Char',[])])]))).
functiontype(prelude_lambda7_0,'FuncType'('TCons'([],['TCons'('Char',[])]),'TCons'([],['TCons'('Char',[])]))).
functiontype(isSpace_0,'FuncType'('TCons'('Char',[]),'TCons'('Bool',[]))).
functiontype(prelude_lambda9_0,'FuncType'('TCons'('(,)',['TCons'([],['TCons'('Char',[])]),'TCons'([],['TCons'('Char',[])])]),'TCons'([],['TCons'([],['TCons'('Char',[])])]))).
functiontype(prelude_lambda8_0,'FuncType'('TCons'([],['TCons'('Char',[])]),'TCons'([],['TCons'([],['TCons'('Char',[])])]))).
functiontype(prelude_lambda10_0,'FuncType'('TCons'([],['TCons'('Char',[])]),'FuncType'('TCons'([],['TCons'('Char',[])]),'TCons'([],['TCons'('Char',[])])))).
functiontype(prelude_lambda11_0,'FuncType'(A,'FuncType'(A,'TCons'('Bool',[])))).
functiontype(prelude_lambda12_0,'FuncType'(A,'FuncType'(A,'TCons'('Bool',[])))).
functiontype(prelude_lambda13_0,'FuncType'('TCons'('Int',[]),'FuncType'('TCons'('Int',[]),'FuncType'('TCons'('Int',[]),'TCons'('Int',[]))))).
functiontype(p_0,'FuncType'('TCons'('Int',[]),'FuncType'('TCons'('Int',[]),'FuncType'('TCons'('Int',[]),'FuncType'('TCons'('Int',[]),'TCons'('Bool',[])))))).
functiontype(prelude_lambda14_0,'FuncType'('TCons'('IO',[A]),'FuncType'(B,'TCons'('IO',[A])))).
functiontype(lambdafun_0,'FuncType'('TCons'('Char',[]),'TCons'('IO',['TCons'([],['TCons'('Char',[])])]))).
functiontype(lambdafun_2,'FuncType'('TCons'('Char',[]),'FuncType'('TCons'([],['TCons'('Char',[])]),'TCons'('IO',['TCons'([],['TCons'('Char',[])])])))).
functiontype(lambdafun_3,'FuncType'(A,'FuncType'('TCons'([],[A]),'TCons'('IO',['TCons'([],[A])])))).
functiontype(lambdafun_1,'FuncType'('TCons'([],['TCons'('IO',[A])]),'FuncType'(A,'TCons'('IO',['TCons'([],[A])])))).
functiontype(prelude_lambda15_0,'FuncType'('FuncType'(A,'TCons'('Success',[])),'FuncType'('FuncType'(A,'TCons'('Success',[])),'FuncType'(A,'TCons'('Success',[]))))).
functiontype(evalall_0,'FuncType'('TCons'([],['FuncType'(A,'TCons'('Success',[]))]),'TCons'([],['FuncType'(A,'TCons'('Success',[]))]))).
functiontype(evalall2_0,'FuncType'('TCons'([],['FuncType'(A,'TCons'('Success',[]))]),'TCons'([],['FuncType'(A,'TCons'('Success',[]))]))).
functiontype(evalall3_0,'FuncType'('TCons'([],['FuncType'(A,'TCons'('Success',[]))]),'FuncType'('TCons'([],['FuncType'(A,'TCons'('Success',[]))]),'TCons'([],['FuncType'(A,'TCons'('Success',[]))])))).
functiontype(evalResult_0,'FuncType'('TCons'([],['FuncType'(A,'TCons'('Success',[]))]),'TCons'([],['FuncType'(A,'TCons'('Success',[]))]))).
functiontype(prelude_lambda16_0,'FuncType'('FuncType'(A,'TCons'('Success',[])),'FuncType'(A,'TCons'('Success',[])))).
functiontype(prelude_lambda17_0,'FuncType'('FuncType'(A,'TCons'('Success',[])),'FuncType'('FuncType'(A,'TCons'('Bool',[])),'FuncType'(A,'TCons'('Success',[]))))).
functiontype(bestHelp_0,'FuncType'('FuncType'(A,'FuncType'(A,'TCons'('Bool',[]))),'FuncType'('TCons'([],['FuncType'(A,'TCons'('Success',[]))]),'FuncType'('TCons'([],['FuncType'(A,'TCons'('Success',[]))]),'FuncType'('TCons'([],['FuncType'(A,'TCons'('Success',[]))]),'TCons'([],['FuncType'(A,'TCons'('Success',[]))])))))).
functiontype(evalY_0,'FuncType'('FuncType'(A,'FuncType'(A,'TCons'('Bool',[]))),'FuncType'('TCons'([],['FuncType'(A,'TCons'('Success',[]))]),'FuncType'('TCons'([],['FuncType'(A,'TCons'('Success',[]))]),'FuncType'('TCons'([],['FuncType'(A,'TCons'('Success',[]))]),'TCons'([],['FuncType'(A,'TCons'('Success',[]))])))))).
functiontype(evalX_0,'FuncType'('FuncType'(A,'FuncType'(A,'TCons'('Bool',[]))),'FuncType'('TCons'([],['FuncType'(A,'TCons'('Success',[]))]),'FuncType'('TCons'([],['FuncType'(A,'TCons'('Success',[]))]),'FuncType'('TCons'([],['FuncType'(A,'TCons'('Success',[]))]),'FuncType'('TCons'([],['FuncType'(A,'TCons'('Success',[]))]),'TCons'([],['FuncType'(A,'TCons'('Success',[]))]))))))).
functiontype(constrain_0,'FuncType'('FuncType'(A,'FuncType'(A,'TCons'('Bool',[]))),'FuncType'('FuncType'(A,'TCons'('Success',[])),'FuncType'('TCons'([],['FuncType'(A,'TCons'('Success',[]))]),'FuncType'(A,'TCons'('Success',[])))))).
functiontype(prelude_lambda18_0,'FuncType'('FuncType'(A,'TCons'('Success',[])),'FuncType'('FuncType'(A,'FuncType'(A,'TCons'('Bool',[]))),'FuncType'(A,'TCons'('Success',[]))))).

functiontype('()','TCons'('()',[])).
functiontype([],'TCons'([],[A])).
functiontype('.','FuncType'(A,'FuncType'('TCons'([],[A]),'TCons'([],[A])))).
functiontype('True','TCons'('Bool',[])).
functiontype('False','TCons'('Bool',[])).
functiontype('(,)','FuncType'(A,'FuncType'(B,'TCons'('(,)',[A,B])))).
functiontype('(,,)','FuncType'(A,'FuncType'(B,'FuncType'(C,'TCons'('(,,)',[A,B,C]))))).
functiontype('(,,,)','FuncType'(A,'FuncType'(B,'FuncType'(C,'FuncType'(D,'TCons'('(,,,)',[A,B,C,D])))))).
functiontype('(,,,,)','FuncType'(A,'FuncType'(B,'FuncType'(C,'FuncType'(D,'FuncType'(E,'TCons'('(,,,,)',[A,B,C,D,E]))))))).
functiontype('(,,,,,)','FuncType'(A,'FuncType'(B,'FuncType'(C,'FuncType'(D,'FuncType'(E,'FuncType'(F,'TCons'('(,,,,,)',[A,B,C,D,E,F])))))))).
functiontype('(,,,,,,)','FuncType'(A,'FuncType'(B,'FuncType'(C,'FuncType'(D,'FuncType'(E,'FuncType'(F,'FuncType'(G,'TCons'('(,,,,,,)',[A,B,C,D,E,F,G]))))))))).
functiontype('(,,,,,,,)','FuncType'(A,'FuncType'(B,'FuncType'(C,'FuncType'(D,'FuncType'(E,'FuncType'(F,'FuncType'(G,'FuncType'(H,'TCons'('(,,,,,,,)',[A,B,C,D,E,F,G,H])))))))))).
functiontype('(,,,,,,,,)','FuncType'(A,'FuncType'(B,'FuncType'(C,'FuncType'(D,'FuncType'(E,'FuncType'(F,'FuncType'(G,'FuncType'(H,'FuncType'(I,'TCons'('(,,,,,,,,)',[A,B,C,D,E,F,G,H,I]))))))))))).
functiontype('(,,,,,,,,,)','FuncType'(A,'FuncType'(B,'FuncType'(C,'FuncType'(D,'FuncType'(E,'FuncType'(F,'FuncType'(G,'FuncType'(H,'FuncType'(I,'FuncType'(J,'TCons'('(,,,,,,,,,)',[A,B,C,D,E,F,G,H,I,J])))))))))))).
functiontype('(,,,,,,,,,,)','FuncType'(A,'FuncType'(B,'FuncType'(C,'FuncType'(D,'FuncType'(E,'FuncType'(F,'FuncType'(G,'FuncType'(H,'FuncType'(I,'FuncType'(J,'FuncType'(K,'TCons'('(,,,,,,,,,,)',[A,B,C,D,E,F,G,H,I,J,K]))))))))))))).
functiontype('(,,,,,,,,,,,)','FuncType'(A,'FuncType'(B,'FuncType'(C,'FuncType'(D,'FuncType'(E,'FuncType'(F,'FuncType'(G,'FuncType'(H,'FuncType'(I,'FuncType'(J,'FuncType'(K,'FuncType'(L,'TCons'('(,,,,,,,,,,,)',[A,B,C,D,E,F,G,H,I,J,K,L])))))))))))))).
functiontype('(,,,,,,,,,,,,)','FuncType'(A,'FuncType'(B,'FuncType'(C,'FuncType'(D,'FuncType'(E,'FuncType'(F,'FuncType'(G,'FuncType'(H,'FuncType'(I,'FuncType'(J,'FuncType'(K,'FuncType'(L,'FuncType'(M,'TCons'('(,,,,,,,,,,,,)',[A,B,C,D,E,F,G,H,I,J,K,L,M]))))))))))))))).
functiontype('(,,,,,,,,,,,,,)','FuncType'(A,'FuncType'(B,'FuncType'(C,'FuncType'(D,'FuncType'(E,'FuncType'(F,'FuncType'(G,'FuncType'(H,'FuncType'(I,'FuncType'(J,'FuncType'(K,'FuncType'(L,'FuncType'(M,'FuncType'(N,'TCons'('(,,,,,,,,,,,,,)',[A,B,C,D,E,F,G,H,I,J,K,L,M,N])))))))))))))))).
functiontype('(,,,,,,,,,,,,,,)','FuncType'(A,'FuncType'(B,'FuncType'(C,'FuncType'(D,'FuncType'(E,'FuncType'(F,'FuncType'(G,'FuncType'(H,'FuncType'(I,'FuncType'(J,'FuncType'(K,'FuncType'(L,'FuncType'(M,'FuncType'(N,'FuncType'(O,'TCons'('(,,,,,,,,,,,,,,)',[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O]))))))))))))))))).
functiontype('LT','TCons'('Ordering',[])).
functiontype('EQ','TCons'('Ordering',[])).
functiontype('GT','TCons'('Ordering',[])).
functiontype('Nothing','TCons'('Maybe',[A])).
functiontype('Just','FuncType'(A,'TCons'('Maybe',[A]))).
functiontype('Left','FuncType'(A,'TCons'('Either',[A,B]))).
functiontype('Right','FuncType'(A,'TCons'('Either',[B,A]))).

evaluation('prelude..','Rule'([0,1,2],'Apply'('Comb'('PartCall',prelude_lambda0_0,['Var'(0),'Var'(1)]),'Var'(2)))).
evaluation(id,'Rule'([0],'Var'(0))).
evaluation(const,'Rule'([0,1],'Var'(0))).
evaluation(curry,'Rule'([0,1,2],'Apply'('Var'(0),'Comb'('ConsCall','(,)',['Var'(1),'Var'(2)])))).
evaluation(uncurry,'Rule'([0,1],'Case'('Flex','Var'(1),['Branch'('Pattern'('(,)',[2,3]),'Apply'('Apply'('Var'(0),'Var'(2)),'Var'(3)))]))).
evaluation(flip,'Rule'([0,1,2],'Apply'('Apply'('Var'(0),'Var'(2)),'Var'(1)))).
evaluation(until,'Rule'([0,1,2],'Comb'('FuncCall',if_then_else,['Apply'('Var'(0),'Var'(2)),'Var'(2),'Comb'('FuncCall',until,['Var'(0),'Var'(1),'Apply'('Var'(1),'Var'(2))])]))).
evaluation($,'Rule'([0,1],'Apply'('Var'(0),'Var'(1)))).
evaluation('$!','Rule'([0,1],'Comb'('FuncCall',seq,['Var'(1),'Apply'('Var'(0),'Var'(1))]))).
evaluation(failed,'Rule'([],'GuardedExpr'([0],'Constr'([],'Comb'('FuncCall',=:=,['Lit'('Intc'(1)),'Lit'('Intc'(2))])),'Var'(0)))).
evaluation(&&,'Rule'([0,1],'Case'('Flex','Var'(0),['Branch'('Pattern'('True',[]),'Var'(1)),'Branch'('Pattern'('False',[]),'Comb'('ConsCall','False',[]))]))).
evaluation('||','Rule'([0,1],'Case'('Flex','Var'(0),['Branch'('Pattern'('True',[]),'Comb'('ConsCall','True',[])),'Branch'('Pattern'('False',[]),'Var'(1))]))).
evaluation(not,'Rule'([0],'Case'('Flex','Var'(0),['Branch'('Pattern'('True',[]),'Comb'('ConsCall','False',[])),'Branch'('Pattern'('False',[]),'Comb'('ConsCall','True',[]))]))).
evaluation(otherwise,'Rule'([],'Comb'('ConsCall','True',[]))).
evaluation(if_then_else,'Rule'([0,1,2],'Case'('Rigid','Var'(0),['Branch'('Pattern'('True',[]),'Var'(1)),'Branch'('Pattern'('False',[]),'Var'(2))]))).
evaluation(/=,'Rule'([0,1],'Comb'('FuncCall',not,['Comb'('FuncCall',==,['Var'(0),'Var'(1)])]))).
evaluation(fst,'Rule'([0],'Case'('Flex','Var'(0),['Branch'('Pattern'('(,)',[1,2]),'Var'(1))]))).
evaluation(snd,'Rule'([0],'Case'('Flex','Var'(0),['Branch'('Pattern'('(,)',[1,2]),'Var'(2))]))).
evaluation(head,'Rule'([0],'Case'('Flex','Var'(0),['Branch'('Pattern'('.',[1,2]),'Var'(1))]))).
evaluation(tail,'Rule'([0],'Case'('Flex','Var'(0),['Branch'('Pattern'('.',[1,2]),'Var'(2))]))).
evaluation(null,'Rule'([0],'Case'('Flex','Var'(0),['Branch'('Pattern'([],[]),'Comb'('ConsCall','True',[])),'Branch'('Pattern'('.',[1,2]),'Comb'('ConsCall','False',[]))]))).
evaluation(++,'Rule'([0,1],'Case'('Flex','Var'(0),['Branch'('Pattern'([],[]),'Var'(1)),'Branch'('Pattern'('.',[2,3]),'Comb'('ConsCall','.',['Var'(2),'Comb'('FuncCall',++,['Var'(3),'Var'(1)])]))]))).
evaluation(length,'Rule'([0],'Case'('Flex','Var'(0),['Branch'('Pattern'([],[]),'Lit'('Intc'(0))),'Branch'('Pattern'('.',[1,2]),'Comb'('FuncCall',+,['Lit'('Intc'(1)),'Comb'('FuncCall',length,['Var'(2)])]))]))).
evaluation('!!','Rule'([0,1],'Case'('Flex','Var'(0),['Branch'('Pattern'('.',[2,3]),'Comb'('FuncCall',if_then_else,['Comb'('FuncCall',==,['Var'(1),'Lit'('Intc'(0))]),'Var'(2),'Comb'('FuncCall',if_then_else,['Comb'('FuncCall',>,['Var'(1),'Lit'('Intc'(0))]),'Comb'('FuncCall','!!',['Var'(3),'Comb'('FuncCall',-,['Var'(1),'Lit'('Intc'(1))])]),'Comb'('FuncCall',failed,[])])]))]))).
evaluation(map,'Rule'([0,1],'Case'('Flex','Var'(1),['Branch'('Pattern'([],[]),'Comb'('ConsCall',[],[])),'Branch'('Pattern'('.',[2,3]),'Comb'('ConsCall','.',['Apply'('Var'(0),'Var'(2)),'Comb'('FuncCall',map,['Var'(0),'Var'(3)])]))]))).
evaluation(foldl,'Rule'([0,1,2],'Case'('Flex','Var'(2),['Branch'('Pattern'([],[]),'Var'(1)),'Branch'('Pattern'('.',[3,4]),'Comb'('FuncCall',foldl,['Var'(0),'Apply'('Apply'('Var'(0),'Var'(1)),'Var'(3)),'Var'(4)]))]))).
evaluation(foldl1,'Rule'([0,1],'Case'('Flex','Var'(1),['Branch'('Pattern'('.',[2,3]),'Comb'('FuncCall',foldl,['Var'(0),'Var'(2),'Var'(3)]))]))).
evaluation(foldr,'Rule'([0,1,2],'Case'('Flex','Var'(2),['Branch'('Pattern'([],[]),'Var'(1)),'Branch'('Pattern'('.',[3,4]),'Apply'('Apply'('Var'(0),'Var'(3)),'Comb'('FuncCall',foldr,['Var'(0),'Var'(1),'Var'(4)])))]))).
evaluation(foldr1,'Rule'([0,1],'Case'('Flex','Var'(1),['Branch'('Pattern'('.',[2,3]),'Case'('Flex','Var'(3),['Branch'('Pattern'([],[]),'Var'(2)),'Branch'('Pattern'('.',[4,5]),'Apply'('Apply'('Var'(0),'Var'(2)),'Comb'('FuncCall',foldr1,['Var'(0),'Comb'('ConsCall','.',['Var'(4),'Var'(5)])])))]))]))).
evaluation(filter,'Rule'([0,1],'Case'('Flex','Var'(1),['Branch'('Pattern'([],[]),'Comb'('ConsCall',[],[])),'Branch'('Pattern'('.',[2,3]),'Comb'('FuncCall',if_then_else,['Apply'('Var'(0),'Var'(2)),'Comb'('ConsCall','.',['Var'(2),'Comb'('FuncCall',filter,['Var'(0),'Var'(3)])]),'Comb'('FuncCall',filter,['Var'(0),'Var'(3)])]))]))).
evaluation(zip,'Rule'([0,1],'Case'('Flex','Var'(0),['Branch'('Pattern'([],[]),'Comb'('ConsCall',[],[])),'Branch'('Pattern'('.',[2,3]),'Case'('Flex','Var'(1),['Branch'('Pattern'([],[]),'Comb'('ConsCall',[],[])),'Branch'('Pattern'('.',[4,5]),'Comb'('ConsCall','.',['Comb'('ConsCall','(,)',['Var'(2),'Var'(4)]),'Comb'('FuncCall',zip,['Var'(3),'Var'(5)])]))]))]))).
evaluation(zip3,'Rule'([0,1,2],'Case'('Flex','Var'(0),['Branch'('Pattern'([],[]),'Comb'('ConsCall',[],[])),'Branch'('Pattern'('.',[3,4]),'Case'('Flex','Var'(1),['Branch'('Pattern'([],[]),'Comb'('ConsCall',[],[])),'Branch'('Pattern'('.',[5,6]),'Case'('Flex','Var'(2),['Branch'('Pattern'([],[]),'Comb'('ConsCall',[],[])),'Branch'('Pattern'('.',[7,8]),'Comb'('ConsCall','.',['Comb'('ConsCall','(,,)',['Var'(3),'Var'(5),'Var'(7)]),'Comb'('FuncCall',zip3,['Var'(4),'Var'(6),'Var'(8)])]))]))]))]))).
evaluation(zipWith,'Rule'([0,1,2],'Case'('Flex','Var'(1),['Branch'('Pattern'([],[]),'Comb'('ConsCall',[],[])),'Branch'('Pattern'('.',[3,4]),'Case'('Flex','Var'(2),['Branch'('Pattern'([],[]),'Comb'('ConsCall',[],[])),'Branch'('Pattern'('.',[5,6]),'Comb'('ConsCall','.',['Apply'('Apply'('Var'(0),'Var'(3)),'Var'(5)),'Comb'('FuncCall',zipWith,['Var'(0),'Var'(4),'Var'(6)])]))]))]))).
evaluation(zipWith3,'Rule'([0,1,2,3],'Case'('Flex','Var'(1),['Branch'('Pattern'([],[]),'Comb'('ConsCall',[],[])),'Branch'('Pattern'('.',[4,5]),'Case'('Flex','Var'(2),['Branch'('Pattern'([],[]),'Comb'('ConsCall',[],[])),'Branch'('Pattern'('.',[6,7]),'Case'('Flex','Var'(3),['Branch'('Pattern'([],[]),'Comb'('ConsCall',[],[])),'Branch'('Pattern'('.',[8,9]),'Comb'('ConsCall','.',['Apply'('Apply'('Apply'('Var'(0),'Var'(4)),'Var'(6)),'Var'(8)),'Comb'('FuncCall',zipWith3,['Var'(0),'Var'(5),'Var'(7),'Var'(9)])]))]))]))]))).
evaluation(unzip,'Rule'([0],'Case'('Flex','Var'(0),['Branch'('Pattern'([],[]),'Comb'('ConsCall','(,)',['Comb'('ConsCall',[],[]),'Comb'('ConsCall',[],[])])),'Branch'('Pattern'('.',[1,2]),'Case'('Flex','Var'(1),['Branch'('Pattern'('(,)',[3,4]),'Comb'('FuncCall',prelude_lambda1_0,['Var'(3),'Var'(4),'Comb'('FuncCall',unzip,['Var'(2)])]))]))]))).
evaluation(unzip3,'Rule'([0],'Case'('Flex','Var'(0),['Branch'('Pattern'([],[]),'Comb'('ConsCall','(,,)',['Comb'('ConsCall',[],[]),'Comb'('ConsCall',[],[]),'Comb'('ConsCall',[],[])])),'Branch'('Pattern'('.',[1,2]),'Case'('Flex','Var'(1),['Branch'('Pattern'('(,,)',[3,4,5]),'Comb'('FuncCall',prelude_lambda2_0,['Var'(3),'Var'(4),'Var'(5),'Comb'('FuncCall',unzip3,['Var'(2)])]))]))]))).
evaluation(concat,'Rule'([0],'Comb'('FuncCall',foldr,['Comb'('PartCall',++,[]),'Comb'('ConsCall',[],[]),'Var'(0)]))).
evaluation(concatMap,'Rule'([0,1],'Apply'('Comb'('FuncCall','prelude..',['Comb'('PartCall',concat,[]),'Comb'('PartCall',map,['Var'(0)])]),'Var'(1)))).
evaluation(iterate,'Rule'([0,1],'Comb'('ConsCall','.',['Var'(1),'Comb'('FuncCall',iterate,['Var'(0),'Apply'('Var'(0),'Var'(1))])]))).
evaluation(repeat,'Rule'([0],'Comb'('ConsCall','.',['Var'(0),'Comb'('FuncCall',repeat,['Var'(0)])]))).
evaluation(replicate,'Rule'([0,1],'Comb'('FuncCall',take,['Var'(0),'Comb'('FuncCall',repeat,['Var'(1)])]))).
evaluation(take,'Rule'([0,1],'Comb'('FuncCall',if_then_else,['Comb'('FuncCall',<=,['Var'(0),'Lit'('Intc'(0))]),'Comb'('ConsCall',[],[]),'Comb'('FuncCall',takep_0,['Var'(0),'Var'(1)])]))).
evaluation(drop,'Rule'([0,1],'Comb'('FuncCall',if_then_else,['Comb'('FuncCall',<=,['Var'(0),'Lit'('Intc'(0))]),'Var'(1),'Comb'('FuncCall',dropp_0,['Var'(0),'Var'(1)])]))).
evaluation(splitAt,'Rule'([0,1],'Comb'('FuncCall',if_then_else,['Comb'('FuncCall',<=,['Var'(0),'Lit'('Intc'(0))]),'Comb'('ConsCall','(,)',['Comb'('ConsCall',[],[]),'Var'(1)]),'Comb'('FuncCall',splitAtp_0,['Var'(0),'Var'(1)])]))).
evaluation(takeWhile,'Rule'([0,1],'Case'('Flex','Var'(1),['Branch'('Pattern'([],[]),'Comb'('ConsCall',[],[])),'Branch'('Pattern'('.',[2,3]),'Comb'('FuncCall',if_then_else,['Apply'('Var'(0),'Var'(2)),'Comb'('ConsCall','.',['Var'(2),'Comb'('FuncCall',takeWhile,['Var'(0),'Var'(3)])]),'Comb'('ConsCall',[],[])]))]))).
evaluation(dropWhile,'Rule'([0,1],'Case'('Flex','Var'(1),['Branch'('Pattern'([],[]),'Comb'('ConsCall',[],[])),'Branch'('Pattern'('.',[2,3]),'Comb'('FuncCall',if_then_else,['Apply'('Var'(0),'Var'(2)),'Comb'('FuncCall',dropWhile,['Var'(0),'Var'(3)]),'Comb'('ConsCall','.',['Var'(2),'Var'(3)])]))]))).
evaluation(span,'Rule'([0,1],'Case'('Flex','Var'(1),['Branch'('Pattern'([],[]),'Comb'('ConsCall','(,)',['Comb'('ConsCall',[],[]),'Comb'('ConsCall',[],[])])),'Branch'('Pattern'('.',[2,3]),'Comb'('FuncCall',if_then_else,['Apply'('Var'(0),'Var'(2)),'Comb'('FuncCall',prelude_lambda4_0,['Var'(2),'Comb'('FuncCall',span,['Var'(0),'Var'(3)])]),'Comb'('ConsCall','(,)',['Comb'('ConsCall',[],[]),'Comb'('ConsCall','.',['Var'(2),'Var'(3)])])]))]))).
evaluation(break,'Rule'([0,1],'Apply'('Comb'('PartCall',span,['Comb'('FuncCall','prelude..',['Comb'('PartCall',not,[]),'Var'(0)])]),'Var'(1)))).
evaluation(lines,'Rule'([0],'Case'('Flex','Var'(0),['Branch'('Pattern'([],[]),'Comb'('ConsCall',[],[])),'Branch'('Pattern'('.',[1,2]),'Comb'('FuncCall',prelude_lambda6_0,['Comb'('FuncCall',splitline_0,['Comb'('ConsCall','.',['Var'(1),'Var'(2)])])]))]))).
evaluation(unlines,'Rule'([0],'Apply'('Comb'('FuncCall',concatMap,['Comb'('PartCall',prelude_lambda7_0,[])]),'Var'(0)))).
evaluation(words,'Rule'([0],'Comb'('FuncCall',prelude_lambda8_0,['Comb'('FuncCall',dropWhile,['Comb'('PartCall',isSpace_0,[]),'Var'(0)])]))).
evaluation(unwords,'Rule'([0],'Comb'('FuncCall',if_then_else,['Comb'('FuncCall',==,['Var'(0),'Comb'('ConsCall',[],[])]),'Comb'('ConsCall',[],[]),'Comb'('FuncCall',foldr1,['Comb'('PartCall',prelude_lambda10_0,[]),'Var'(0)])]))).
evaluation(reverse,'Rule'([0],'Apply'('Comb'('PartCall',foldl,['Comb'('PartCall',flip,['Comb'('ConsCall','.',[])]),'Comb'('ConsCall',[],[])]),'Var'(0)))).
evaluation(and,'Rule'([0],'Apply'('Comb'('PartCall',foldr,['Comb'('PartCall',&&,[]),'Comb'('ConsCall','True',[])]),'Var'(0)))).
evaluation(or,'Rule'([0],'Apply'('Comb'('PartCall',foldr,['Comb'('PartCall','||',[]),'Comb'('ConsCall','False',[])]),'Var'(0)))).
evaluation(any,'Rule'([0,1],'Apply'('Comb'('FuncCall','prelude..',['Comb'('FuncCall',or,[]),'Comb'('PartCall',map,['Var'(0)])]),'Var'(1)))).
evaluation(all,'Rule'([0,1],'Apply'('Comb'('FuncCall','prelude..',['Comb'('FuncCall',and,[]),'Comb'('PartCall',map,['Var'(0)])]),'Var'(1)))).
evaluation(elem,'Rule'([0,1],'Apply'('Comb'('FuncCall',any,['Comb'('PartCall',prelude_lambda11_0,['Var'(0)])]),'Var'(1)))).
evaluation(notElem,'Rule'([0,1],'Apply'('Comb'('FuncCall',all,['Comb'('PartCall',prelude_lambda12_0,['Var'(0)])]),'Var'(1)))).
evaluation(lookup,'Rule'([0,1],'Case'('Flex','Var'(1),['Branch'('Pattern'([],[]),'Comb'('ConsCall','Nothing',[])),'Branch'('Pattern'('.',[2,3]),'Case'('Flex','Var'(2),['Branch'('Pattern'('(,)',[4,5]),'Comb'('FuncCall',if_then_else,['Comb'('FuncCall',==,['Var'(0),'Var'(4)]),'Comb'('ConsCall','Just',['Var'(5)]),'Comb'('FuncCall',lookup,['Var'(0),'Var'(3)])]))]))]))).
evaluation(enumFrom,'Rule'([0],'Comb'('ConsCall','.',['Var'(0),'Comb'('FuncCall',enumFrom,['Comb'('FuncCall',+,['Var'(0),'Lit'('Intc'(1))])])]))).
evaluation(enumFromThen,'Rule'([0,1],'Comb'('FuncCall',iterate,['Comb'('PartCall',prelude_lambda13_0,['Var'(1),'Var'(0)]),'Var'(0)]))).
evaluation(enumFromTo,'Rule'([0,1],'Comb'('FuncCall',if_then_else,['Comb'('FuncCall',>,['Var'(0),'Var'(1)]),'Comb'('ConsCall',[],[]),'Comb'('ConsCall','.',['Var'(0),'Comb'('FuncCall',enumFromTo,['Comb'('FuncCall',+,['Var'(0),'Lit'('Intc'(1))]),'Var'(1)])])]))).
evaluation(enumFromThenTo,'Rule'([0,1,2],'Comb'('FuncCall',takeWhile,['Comb'('PartCall',p_0,['Var'(1),'Var'(0),'Var'(2)]),'Comb'('FuncCall',enumFromThen,['Var'(0),'Var'(1)])]))).
evaluation(negate,'Rule'([0],'Comb'('FuncCall',-,['Lit'('Intc'(0)),'Var'(0)]))).
evaluation(&>,'Rule'([0,1],'GuardedExpr'([],'Constr'([],'Var'(0)),'Var'(1)))).
evaluation(maybe,'Rule'([0,1,2],'Case'('Flex','Var'(2),['Branch'('Pattern'('Nothing',[]),'Var'(0)),'Branch'('Pattern'('Just',[3]),'Apply'('Var'(1),'Var'(3)))]))).
evaluation(either,'Rule'([0,1,2],'Case'('Flex','Var'(2),['Branch'('Pattern'('Left',[3]),'Apply'('Var'(0),'Var'(3))),'Branch'('Pattern'('Right',[4]),'Apply'('Var'(1),'Var'(4)))]))).
evaluation(done,'Rule'([0],'Apply'('Comb'('FuncCall',return,['Comb'('ConsCall','()',[])]),'Var'(0)))).
evaluation(putStr,'Rule'([0,3],'Case'('Rigid','Var'(0),['Branch'('Pattern'([],[]),'Apply'('Comb'('FuncCall',done,[]),'Var'(3))),'Branch'('Pattern'('.',[1,2]),'Apply'('Comb'('FuncCall',>>,['Comb'('FuncCall',putChar,['Var'(1)]),'Comb'('FuncCall',putStr,['Var'(2)])]),'Var'(3)))]))).
evaluation(putStrLn,'Rule'([0,1],'Apply'('Comb'('FuncCall',>>,['Comb'('FuncCall',putStr,['Var'(0)]),'Comb'('FuncCall',putChar,['Lit'('Charc'(10))])]),'Var'(1)))).
evaluation(getLine,'Rule'([0],'Apply'('Comb'('FuncCall',>>=,['Comb'('FuncCall',getChar,[]),'Comb'('PartCall',lambdafun_0,[])]),'Var'(0)))).
evaluation(print,'Rule'([0,1],'Apply'('Comb'('FuncCall',putStrLn,['Comb'('FuncCall',show,['Var'(0)])]),'Var'(1)))).
evaluation(doSolve,'Rule'([0,1],'GuardedExpr'([],'Constr'([],'Var'(0)),'Apply'('Comb'('FuncCall',done,[]),'Var'(1))))).
evaluation(sequenceIO,'Rule'([0,3],'Case'('Rigid','Var'(0),['Branch'('Pattern'([],[]),'Apply'('Comb'('FuncCall',return,['Comb'('ConsCall',[],[])]),'Var'(3))),'Branch'('Pattern'('.',[1,2]),'Apply'('Comb'('FuncCall',>>=,['Var'(1),'Comb'('PartCall',lambdafun_1,['Var'(2)])]),'Var'(3)))]))).
evaluation(sequenceIO_,'Rule'([0,1],'Apply'('Apply'('Comb'('PartCall',foldr,['Comb'('PartCall',>>,[]),'Comb'('FuncCall',done,[])]),'Var'(0)),'Var'(1)))).
evaluation(mapIO,'Rule'([0,1,2],'Apply'('Apply'('Comb'('FuncCall','prelude..',['Comb'('PartCall',sequenceIO,[]),'Comb'('PartCall',map,['Var'(0)])]),'Var'(1)),'Var'(2)))).
evaluation(mapIO_,'Rule'([0,1,2],'Apply'('Apply'('Comb'('FuncCall','prelude..',['Comb'('FuncCall',sequenceIO_,[]),'Comb'('PartCall',map,['Var'(0)])]),'Var'(1)),'Var'(2)))).
evaluation(?,'Rule'([0,1],'Or'('Var'(0),'Var'(1)))).
evaluation(inject,'Rule'([0,1,2],'Apply'('Comb'('PartCall',prelude_lambda15_0,['Var'(1),'Var'(0)]),'Var'(2)))).
evaluation(solveAll,'Rule'([0],'Comb'('FuncCall',evalall_0,['Comb'('FuncCall',try,['Var'(0)])]))).
evaluation(solveAll2,'Rule'([0],'Comb'('FuncCall',evalResult_0,['Comb'('FuncCall',try,['Var'(0)])]))).
evaluation(once,'Rule'([0,1],'Apply'('Comb'('FuncCall',head,['Comb'('FuncCall',solveAll,['Var'(0)])]),'Var'(1)))).
evaluation(one,'Rule'([0],'Comb'('FuncCall',solveAll,['Comb'('PartCall',prelude_lambda16_0,['Var'(0)])]))).
evaluation(condSearch,'Rule'([0,1],'Comb'('FuncCall',one,['Comb'('PartCall',prelude_lambda17_0,['Var'(0),'Var'(1)])]))).
evaluation(best,'Rule'([0,1],'Comb'('FuncCall',bestHelp_0,['Var'(1),'Comb'('ConsCall',[],[]),'Comb'('FuncCall',try,['Var'(0)]),'Comb'('ConsCall',[],[])]))).
evaluation(browse,'Rule'([0,1],'Apply'('Comb'('FuncCall',putStr,['Comb'('FuncCall',show,['Comb'('FuncCall',unpack,['Var'(0)])])]),'Var'(1)))).
evaluation(browseList,'Rule'([0,3],'Case'('Rigid','Var'(0),['Branch'('Pattern'([],[]),'Apply'('Comb'('FuncCall',done,[]),'Var'(3))),'Branch'('Pattern'('.',[1,2]),'Apply'('Comb'('FuncCall',>>,['Comb'('FuncCall',>>,['Comb'('FuncCall',browse,['Var'(1)]),'Comb'('FuncCall',putChar,['Lit'('Charc'(10))])]),'Comb'('FuncCall',browseList,['Var'(2)])]),'Var'(3)))]))).
evaluation(unpack,'Rule'([0],'GuardedExpr'([1],'Constr'([],'Apply'('Var'(0),'Var'(1))),'Var'(1)))).
evaluation('PEVAL','Rule'([0],'Var'(0))).
evaluation(prelude_lambda0_0,'Rule'([0,1,2],'Apply'('Var'(0),'Apply'('Var'(1),'Var'(2))))).
evaluation(prelude_lambda1_0,'Rule'([0,1,2],'Case'('Flex','Var'(2),['Branch'('Pattern'('(,)',[3,4]),'Comb'('ConsCall','(,)',['Comb'('ConsCall','.',['Var'(0),'Var'(3)]),'Comb'('ConsCall','.',['Var'(1),'Var'(4)])]))]))).
evaluation(prelude_lambda2_0,'Rule'([0,1,2,3],'Case'('Flex','Var'(3),['Branch'('Pattern'('(,,)',[4,5,6]),'Comb'('ConsCall','(,,)',['Comb'('ConsCall','.',['Var'(0),'Var'(4)]),'Comb'('ConsCall','.',['Var'(1),'Var'(5)]),'Comb'('ConsCall','.',['Var'(2),'Var'(6)])]))]))).
evaluation(takep_0,'Rule'([0,1],'Case'('Flex','Var'(1),['Branch'('Pattern'([],[]),'Comb'('ConsCall',[],[])),'Branch'('Pattern'('.',[2,3]),'Comb'('ConsCall','.',['Var'(2),'Comb'('FuncCall',take,['Comb'('FuncCall',-,['Var'(0),'Lit'('Intc'(1))]),'Var'(3)])]))]))).
evaluation(dropp_0,'Rule'([0,1],'Case'('Flex','Var'(1),['Branch'('Pattern'([],[]),'Comb'('ConsCall',[],[])),'Branch'('Pattern'('.',[2,3]),'Comb'('FuncCall',drop,['Comb'('FuncCall',-,['Var'(0),'Lit'('Intc'(1))]),'Var'(3)]))]))).
evaluation(splitAtp_0,'Rule'([0,1],'Case'('Flex','Var'(1),['Branch'('Pattern'([],[]),'Comb'('ConsCall','(,)',['Comb'('ConsCall',[],[]),'Comb'('ConsCall',[],[])])),'Branch'('Pattern'('.',[2,3]),'Comb'('FuncCall',prelude_lambda3_0,['Var'(2),'Comb'('FuncCall',splitAt,['Comb'('FuncCall',-,['Var'(0),'Lit'('Intc'(1))]),'Var'(3)])]))]))).
evaluation(prelude_lambda3_0,'Rule'([0,1],'Case'('Flex','Var'(1),['Branch'('Pattern'('(,)',[2,3]),'Comb'('ConsCall','(,)',['Comb'('ConsCall','.',['Var'(0),'Var'(2)]),'Var'(3)]))]))).
evaluation(prelude_lambda4_0,'Rule'([0,1],'Case'('Flex','Var'(1),['Branch'('Pattern'('(,)',[2,3]),'Comb'('ConsCall','(,)',['Comb'('ConsCall','.',['Var'(0),'Var'(2)]),'Var'(3)]))]))).
evaluation(splitline_0,'Rule'([0],'Case'('Flex','Var'(0),['Branch'('Pattern'([],[]),'Comb'('ConsCall','(,)',['Comb'('ConsCall',[],[]),'Comb'('ConsCall',[],[])])),'Branch'('Pattern'('.',[1,2]),'Comb'('FuncCall',if_then_else,['Comb'('FuncCall',==,['Var'(1),'Lit'('Charc'(10))]),'Comb'('ConsCall','(,)',['Comb'('ConsCall',[],[]),'Var'(2)]),'Comb'('FuncCall',prelude_lambda5_0,['Var'(1),'Comb'('FuncCall',splitline_0,['Var'(2)])])]))]))).
evaluation(prelude_lambda5_0,'Rule'([0,1],'Case'('Flex','Var'(1),['Branch'('Pattern'('(,)',[2,3]),'Comb'('ConsCall','(,)',['Comb'('ConsCall','.',['Var'(0),'Var'(2)]),'Var'(3)]))]))).
evaluation(prelude_lambda6_0,'Rule'([0],'Case'('Flex','Var'(0),['Branch'('Pattern'('(,)',[1,2]),'Comb'('ConsCall','.',['Var'(1),'Comb'('FuncCall',lines,['Var'(2)])]))]))).
evaluation(prelude_lambda7_0,'Rule'([0],'Comb'('FuncCall',++,['Var'(0),'Comb'('ConsCall','.',['Lit'('Charc'(10)),'Comb'('ConsCall',[],[])])]))).
evaluation(isSpace_0,'Rule'([0],'Comb'('FuncCall','||',['Comb'('FuncCall',==,['Var'(0),'Lit'('Charc'(32))]),'Comb'('FuncCall','||',['Comb'('FuncCall',==,['Var'(0),'Lit'('Charc'(9))]),'Comb'('FuncCall','||',['Comb'('FuncCall',==,['Var'(0),'Lit'('Charc'(10))]),'Comb'('FuncCall',==,['Var'(0),'Lit'('Charc'(13))])])])]))).
evaluation(prelude_lambda9_0,'Rule'([0],'Case'('Flex','Var'(0),['Branch'('Pattern'('(,)',[1,2]),'Comb'('ConsCall','.',['Var'(1),'Comb'('FuncCall',words,['Var'(2)])]))]))).
evaluation(prelude_lambda8_0,'Rule'([0],'Comb'('FuncCall',if_then_else,['Comb'('FuncCall',==,['Var'(0),'Comb'('ConsCall',[],[])]),'Comb'('ConsCall',[],[]),'Comb'('FuncCall',prelude_lambda9_0,['Apply'('Comb'('FuncCall',break,['Comb'('PartCall',isSpace_0,[])]),'Var'(0))])]))).
evaluation(prelude_lambda10_0,'Rule'([0,1],'Comb'('FuncCall',++,['Var'(0),'Comb'('ConsCall','.',['Lit'('Charc'(32)),'Var'(1)])]))).
evaluation(prelude_lambda11_0,'Rule'([0,1],'Comb'('FuncCall',==,['Var'(0),'Var'(1)]))).
evaluation(prelude_lambda12_0,'Rule'([0,1],'Comb'('FuncCall',/=,['Var'(0),'Var'(1)]))).
evaluation(prelude_lambda13_0,'Rule'([0,1,2],'Comb'('FuncCall',+,['Comb'('FuncCall',-,['Var'(0),'Var'(1)]),'Var'(2)]))).
evaluation(p_0,'Rule'([0,1,2,3],'Comb'('FuncCall',if_then_else,['Comb'('FuncCall',>=,['Var'(0),'Var'(1)]),'Comb'('FuncCall',<=,['Var'(3),'Var'(2)]),'Comb'('FuncCall',>=,['Var'(3),'Var'(2)])]))).
evaluation(prelude_lambda14_0,'Rule'([0,1,2],'Apply'('Var'(0),'Var'(2)))).
evaluation(lambdafun_0,'Rule'([0,1],'Apply'('Comb'('FuncCall',if_then_else,['Comb'('FuncCall',==,['Var'(0),'Lit'('Charc'(10))]),'Comb'('FuncCall',return,['Comb'('ConsCall',[],[])]),'Comb'('FuncCall',>>=,['Comb'('FuncCall',getLine,[]),'Comb'('PartCall',lambdafun_2,['Var'(0)])])]),'Var'(1)))).
evaluation(lambdafun_2,'Rule'([0,1,2],'Apply'('Comb'('FuncCall',return,['Comb'('ConsCall','.',['Var'(0),'Var'(1)])]),'Var'(2)))).
evaluation(lambdafun_3,'Rule'([0,1,2],'Apply'('Comb'('FuncCall',return,['Comb'('ConsCall','.',['Var'(0),'Var'(1)])]),'Var'(2)))).
evaluation(lambdafun_1,'Rule'([0,1,2],'Apply'('Comb'('FuncCall',>>=,['Comb'('FuncCall',sequenceIO,['Var'(0)]),'Comb'('PartCall',lambdafun_3,['Var'(1)])]),'Var'(2)))).
evaluation(prelude_lambda15_0,'Rule'([0,1,2],'Comb'('FuncCall',&,['Apply'('Var'(0),'Var'(2)),'Apply'('Var'(1),'Var'(2))]))).
evaluation(evalall_0,'Rule'([0],'Case'('Flex','Var'(0),['Branch'('Pattern'([],[]),'Comb'('ConsCall',[],[])),'Branch'('Pattern'('.',[1,2]),'Case'('Flex','Var'(2),['Branch'('Pattern'([],[]),'Comb'('ConsCall','.',['Var'(1),'Comb'('ConsCall',[],[])])),'Branch'('Pattern'('.',[3,4]),'Comb'('FuncCall',evalall3_0,['Comb'('FuncCall',try,['Var'(1)]),'Comb'('ConsCall','.',['Var'(3),'Var'(4)])]))]))]))).
evaluation(evalall2_0,'Rule'([0],'Case'('Flex','Var'(0),['Branch'('Pattern'([],[]),'Comb'('ConsCall',[],[])),'Branch'('Pattern'('.',[1,2]),'Comb'('FuncCall',evalall3_0,['Comb'('FuncCall',try,['Var'(1)]),'Var'(2)]))]))).
evaluation(evalall3_0,'Rule'([0,1],'Case'('Flex','Var'(0),['Branch'('Pattern'([],[]),'Comb'('FuncCall',evalall2_0,['Var'(1)])),'Branch'('Pattern'('.',[2,3]),'Case'('Flex','Var'(3),['Branch'('Pattern'([],[]),'Comb'('ConsCall','.',['Var'(2),'Comb'('FuncCall',evalall2_0,['Var'(1)])])),'Branch'('Pattern'('.',[4,5]),'Comb'('FuncCall',evalall3_0,['Comb'('FuncCall',try,['Var'(2)]),'Comb'('ConsCall','.',['Var'(4),'Comb'('FuncCall',++,['Var'(5),'Var'(1)])])]))]))]))).
evaluation(evalResult_0,'Rule'([0],'Case'('Flex','Var'(0),['Branch'('Pattern'([],[]),'Comb'('ConsCall',[],[])),'Branch'('Pattern'('.',[1,2]),'Case'('Flex','Var'(2),['Branch'('Pattern'([],[]),'Comb'('ConsCall','.',['Var'(1),'Comb'('ConsCall',[],[])])),'Branch'('Pattern'('.',[3,4]),'Apply'('Comb'('FuncCall',concatMap,['Comb'('PartCall',solveAll2,[])]),'Comb'('ConsCall','.',['Var'(1),'Comb'('ConsCall','.',['Var'(3),'Var'(4)])])))]))]))).
evaluation(prelude_lambda16_0,'Rule'([0,1],'Comb'('FuncCall',=:=,['Var'(1),'Comb'('FuncCall',tryone,['Var'(0)])]))).
evaluation(prelude_lambda17_0,'Rule'([0,1,2],'Comb'('FuncCall',&,['Apply'('Var'(0),'Var'(2)),'Comb'('FuncCall',=:=,['Apply'('Var'(1),'Var'(2)),'Comb'('ConsCall','True',[])])]))).
evaluation(bestHelp_0,'Rule'([0,1,2,3],'Case'('Flex','Var'(1),['Branch'('Pattern'([],[]),'Case'('Flex','Var'(2),['Branch'('Pattern'([],[]),'Var'(3)),'Branch'('Pattern'('.',[4,5]),'Comb'('FuncCall',evalY_0,['Var'(0),'Comb'('FuncCall',try,['Comb'('FuncCall',constrain_0,['Var'(0),'Var'(4),'Var'(3)])]),'Var'(5),'Var'(3)]))])),'Branch'('Pattern'('.',[6,7]),'Comb'('FuncCall',evalX_0,['Var'(0),'Comb'('FuncCall',try,['Var'(6)]),'Var'(7),'Var'(2),'Var'(3)]))]))).
evaluation(evalY_0,'Rule'([0,1,2,3],'Case'('Flex','Var'(1),['Branch'('Pattern'([],[]),'Comb'('FuncCall',bestHelp_0,['Var'(0),'Comb'('ConsCall',[],[]),'Var'(2),'Var'(3)])),'Branch'('Pattern'('.',[4,5]),'Case'('Flex','Var'(5),['Branch'('Pattern'([],[]),'Comb'('FuncCall',bestHelp_0,['Var'(0),'Comb'('ConsCall',[],[]),'Var'(2),'Comb'('ConsCall','.',['Var'(4),'Comb'('ConsCall',[],[])])])),'Branch'('Pattern'('.',[6,7]),'Comb'('FuncCall',bestHelp_0,['Var'(0),'Comb'('ConsCall','.',['Var'(4),'Comb'('ConsCall','.',['Var'(6),'Var'(7)])]),'Var'(2),'Var'(3)]))]))]))).
evaluation(evalX_0,'Rule'([0,1,2,3,4],'Case'('Flex','Var'(1),['Branch'('Pattern'([],[]),'Comb'('FuncCall',bestHelp_0,['Var'(0),'Var'(2),'Var'(3),'Var'(4)])),'Branch'('Pattern'('.',[5,6]),'Case'('Flex','Var'(6),['Branch'('Pattern'([],[]),'Comb'('FuncCall',bestHelp_0,['Var'(0),'Comb'('ConsCall',[],[]),'Comb'('FuncCall',++,['Var'(2),'Var'(3)]),'Comb'('ConsCall','.',['Var'(5),'Comb'('ConsCall',[],[])])])),'Branch'('Pattern'('.',[7,8]),'Comb'('FuncCall',bestHelp_0,['Var'(0),'Comb'('FuncCall',++,['Comb'('ConsCall','.',['Var'(5),'Comb'('ConsCall','.',['Var'(7),'Var'(8)])]),'Var'(2)]),'Var'(3),'Var'(4)]))]))]))).
evaluation(constrain_0,'Rule'([0,1,2,5],'Case'('Flex','Var'(2),['Branch'('Pattern'([],[]),'Apply'('Var'(1),'Var'(5))),'Branch'('Pattern'('.',[3,4]),'Case'('Flex','Var'(4),['Branch'('Pattern'([],[]),'Apply'('Comb'('FuncCall',inject,['Var'(1),'Comb'('PartCall',prelude_lambda18_0,['Var'(3),'Var'(0)])]),'Var'(5)))]))]))).
evaluation(prelude_lambda18_0,'Rule'([0,1,2],'Constr'([3],'Comb'('FuncCall',&,['Apply'('Var'(0),'Var'(3)),'Comb'('FuncCall',=:=,['Apply'('Apply'('Var'(1),'Var'(2)),'Var'(3)),'Comb'('ConsCall','True',[])])])))).



:-ensure_loaded('/home/mh/pakcs_dist/curry2prolog/prim_standard').

initializationsInProg(true).

:-dynamic dynamicPredInfo/2.

%%%%% Number of shared variables: 51
