%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Interface to the CLP(R) solver library of Sicstus-Prolog
%
% The clauses in this file are added to each compiled Curry program
% if real arithmetic constraints are used.

:- use_module(library(clpr)).

?- block 'prim_CLPR.+.'(?,?,?,-,?).
'prim_CLPR.+.'(A,B,R,E0,E) :- {R = A+B}, E0=E.

?- block 'prim_CLPR.-.'(?,?,?,-,?).
'prim_CLPR.-.'(A,B,R,E0,E) :- {R = A-B}, E0=E.

?- block 'prim_CLPR.*.'(?,?,?,-,?).
'prim_CLPR.*.'(A,B,R,E0,E) :- {R = A*B}, E0=E.

?- block 'prim_CLPR./.'(?,?,?,-,?).
'prim_CLPR./.'(A,B,R,E0,E) :- {R = A/B}, E0=E.

?- block 'prim_CLPR.<.'(?,?,?,-,?).
'prim_CLPR.<.'(A,B,success,E0,E) :- {A<B}, E0=E.

?- block 'prim_CLPR.<=.'(?,?,?,-,?).
'prim_CLPR.<=.'(A,B,success,E0,E) :- {A=<B}, E0=E.

?- block 'prim_CLPR.>.'(?,?,?,-,?).
'prim_CLPR.>.'(A,B,success,E0,E) :- {A>B}, E0=E.

?- block 'prim_CLPR.>=.'(?,?,?,-,?).
'prim_CLPR.>=.'(A,B,success,E0,E) :- {A>=B}, E0=E.

% transform an integer into a float:
?- block 'prim_CLPR.i2f'(-,?,?,?), 'prim_CLPR.i2f'(?,?,-,?).
'prim_CLPR.i2f'(A,R,E0,E) :- R is A*1.0, E0=E.

?- block 'prim_CLPR.minimumFor'(?,?,?,-,?).
'prim_CLPR.minimumFor'(Guard,Fun,X,E0,E) :-
        waitGround('(,)'(Guard,Fun),E0,E1),
	exec_minimum(Guard,Fun,X,E1,E).

?- block exec_minimum(?,?,?,-,?).
exec_minimum(Guard,Fun,X,E,E3) :-
        apply(Guard,X,success,E,E1),
        apply(Fun,X,Z,E1,E2),
	minimize(Z), E2=E3.

?- block 'prim_CLPR.maximumFor'(?,?,?,-,?).
'prim_CLPR.maximumFor'(Guard,Fun,X,E0,E) :-
        waitGround('(,)'(Guard,Fun),E0,E1),
	exec_maximum(Guard,Fun,X,E1,E).

?- block exec_maximum(?,?,?,-,?).
exec_maximum(Guard,Fun,X,E,E3) :-
        apply(Guard,X,success,E,E1),
        apply(Fun,X,Z,E1,E2),
	maximize(Z), E2=E3.
