%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Definition of external functions of module IO:
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- use_module(library(system)).

?- block 'prim_IO.stdin'(?,-,?).
'prim_IO.stdin'(Stream,E0,E) :- prolog_flag(user_input,Stream), E0=E.

?- block 'prim_IO.stdout'(?,-,?).
'prim_IO.stdout'(Stream,E0,E) :- prolog_flag(user_output,Stream), E0=E.

?- block 'prim_IO.stderr'(?,-,?).
'prim_IO.stderr'(Stream,E0,E) :- prolog_flag(user_error,Stream), E0=E.

?- block 'prim_IO.openFile'(?,?,?,?,-,?).
'prim_IO.openFile'(A,Mode,_,'$io'(Stream),E0,E) :-
	string2Atom(A,FName),
	curryFileMode2plmode(Mode,PMode),
	open(FName,PMode,Stream), E0=E.

curryFileMode2plmode('IO.ReadMode',read).
curryFileMode2plmode('IO.WriteMode',write).
curryFileMode2plmode('IO.AppendMode',append).

?- block 'prim_IO.hClose'(?,?,?,?,-,?).
'prim_IO.hClose'(Stream,_,'$io'('()'),E0,E) :-
        (stream_property(Stream,output) -> flush_output(Stream) ; true),
        close(Stream), E0=E.

?- block 'prim_IO.hFlush'(?,?,?,?,-,?).
'prim_IO.hFlush'(Stream,_,'$io'('()'),E0,E) :-
        (stream_property(Stream,output) -> flush_output(Stream) ; true), E0=E.

?- block 'prim_IO.hIsEOF'(-,?,?,?,?), 'prim_IO.hIsEOF'(?,?,?,-,?).
'prim_IO.hIsEOF'(Str,_,'$io'(B),E0,E) :-
        (at_end_of_stream(Str) -> B='True' ; B='False'), E0=E.

?- block 'prim_IO.hGetChar'(-,?,?,?,?), 'prim_IO.hGetChar'(?,?,?,-,?).
'prim_IO.hGetChar'(Str,_,'$io'(C),E0,E) :-
        get0(Str,N), char_int(C,N), E0=E.

?- block 'prim_IO.hPutChar'(-,?,?,?,?,?), 'prim_IO.hPutChar'(?,-,?,?,?,?),
         'prim_IO.hPutChar'(?,?,?,?,-,?).
'prim_IO.hPutChar'(Stream,C,_,'$io'('()'),E0,E) :-
	char_int(C,N), put(Stream,N), E0=E.


?- block 'prim_IO.hIsReadable'(-,?,?,?,?), 'prim_IO.hIsReadable'(?,?,?,-,?).
'prim_IO.hIsReadable'(Stream,_,'$io'(B),E0,E) :-
        (stream_property(Stream,input) -> B='True' ; B='False'), E0=E.

?- block 'prim_IO.hIsWritable'(-,?,?,?,?), 'prim_IO.hIsWritable'(?,?,?,-,?).
'prim_IO.hIsWritable'(Stream,_,'$io'(B),E0,E) :-
        (stream_property(Stream,output) -> B='True' ; B='False'), E0=E.

