%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Definitions of builtins of module System:
%

?- block 'prim_PlProfileData.profileReset'(?,?,-,?).
'prim_PlProfileData.profileReset'(_,'$io'('()'),E0,E) :-
	checkPlProfiling,
	profile_reset([user:_]), E0=E.

?- block 'prim_PlProfileData.profilePredicates'(-,?,?,?,?),
         'prim_PlProfileData.profilePredicates'(?,?,?,-,?).
'prim_PlProfileData.profilePredicates'(SelS,_,'$io'(TData),E0,E) :-
	checkPlProfiling,
        transSelection(SelS,Selection),
	profile_data([user:_],Selection,predicate,Data),
	transPredData(Data,TData), E0=E.

transPredData([],[]).
transPredData([-(_,0)|Ds],TDs) :- !, % don't show zero values
	transPredData(Ds,TDs).
transPredData([-(:(user,Pred/_),C)|Ds],['(,)'(PredString,C)|TDs]) :-
	atom2String(Pred,PredString),
	transPredData(Ds,TDs).

?- block 'prim_PlProfileData.profileClauses'(-,?,?,?,?),
         'prim_PlProfileData.profileClauses'(?,?,?,-,?).
'prim_PlProfileData.profileClauses'(SelS,PredS,_,'$io'(TData),E0,E) :-
	checkPlProfiling,
        transSelection(SelS,Selection),
        string2Atom(PredS,Pred),
	profile_data([user:Pred],Selection,clause,Data),
	transClauseData(Data,TData), E0=E.

transClauseData([],[]).
transClauseData([-(:(user,_/_/CNr),C)|Ds],['(,)'(CNr,C)|TDs]) :-
	transClauseData(Ds,TDs).


?- block 'prim_PlProfileData.profileView'(?,?,?).
'prim_PlProfileData.profileView'(_,'$io'('()'),E0,E) :-
	checkPlProfiling,
	view([user:_]), E0=E.


transSelection('PlProfileData.Calls',calls).
transSelection('PlProfileData.Backtracks',backtracks).
transSelection('PlProfileData.ChoicePoints',choice_points).
transSelection('PlProfileData.ExecTime',execution_time).


% check for profiling mode and terminate with error:
checkPlProfiling :-
	prolog_flag(compiling,Mode,Mode),
	(Mode = profiledcode -> true
	  ; raise_exception('No in profiling mode, use ":set +plprofile"!')).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% get the list of hnf definitions in a translated Curry program:

?- block 'prim_PlProfileData.getHnfDefinitions'(?,?,-,?).
'prim_PlProfileData.getHnfDefinitions'(_,'$io'(HnfInfos),E0,E) :-
	checkPlProfiling,
	lastload(Prog),
	(Prog="" -> write('ERROR: no program loaded for profiling'), nl,
	            !, fail
                  ; true),
	append(Prog,".pl",PlFileS), atom_chars(PlFile,PlFileS),
	see(PlFile),
	repeat,
	read(Clause),
	isHnfClause(Clause),
	!,
	hnfClause2Info(Clause,I1),
	hnfClauses2InfoList(2,Is),
	seen,
	map2(hnfInfo2Curry,[1/I1|Is],HnfInfos), E0=E.

hnfInfo2Curry(N/A,'(,)'(N,AS)) :- atom2String(A,AS).
	
hnfClauses2InfoList(N,[N/I|Is]) :-
	read(Clause),
	isHnfClause(Clause), !,
	hnfClause2Info(Clause,I),
	N1 is N+1,
	hnfClauses2InfoList(N1,Is).
hnfClauses2InfoList(_,[]).

isHnfClause((:- hnf(_,_,_,_))) :- !.
isHnfClause((hnf(_,_,_,_):-_)) :- !.
isHnfClause(hnf(_,_,_,_)) :- !.

hnfClause2Info((hnf(Arg,_,_,_) :- _),'LOGVAR') :- var(Arg), !.
hnfClause2Info((hnf(FCall,_,_,_) :- _),Func) :- !,
        functor(FCall,Func,_).
hnfClause2Info(hnf(_,_,_,_),'CONS').

