%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Definition of standard external functions:
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- use_module(library(system)).
:- use_module(prim_readshowterm). % for prim_show

% definition of concurrent conjunction:
?- block 'prim_&'(?,?,?,-,?).
'prim_&'(C1,C2,success,E0,E) :-
      hnf(C1,S1,E0,E1),
      hnf(C2,S2,E0,E2),
      waitconj(S1,S2,E1,E2,E).

?- block waitconj(?,?,-,?,?), waitconj(?,?,?,-,?).
waitconj(success,success,_,E,E).


% primitive for conditional rules:
?- block 'prim_cond'(-,?,?,?,?), 'prim_cond'(?,?,?,-,?).
'prim_cond'(success,RHS,R,E0,E) :- hnf(RHS,R,E0,E).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Definition of arithmetic functions:
%
?- block 'prim_+'(-,?,?,?,?), 'prim_+'(?,-,?,?,?), 'prim_+'(?,?,?,-,?).
'prim_+'(A,B,R,E0,E) :- R is A+B, E0=E.

?- block 'prim_-'(-,?,?,?,?), 'prim_-'(?,-,?,?,?), 'prim_-'(?,?,?,-,?).
'prim_-'(A,B,R,E0,E) :- R is A-B, E0=E.

?- block 'prim_*'(-,?,?,?,?), 'prim_*'(?,-,?,?,?), 'prim_*'(?,?,?,-,?).
'prim_*'(A,B,R,E0,E) :- R is A*B, E0=E.

?- block 'prim_div'(-,?,?,?,?), 'prim_div'(?,-,?,?,?), 'prim_div'(?,?,?,-,?).
'prim_div'(A,B,R,E0,E) :- R is A // B, E0=E.

?- block 'prim_mod'(-,?,?,?,?), 'prim_mod'(?,-,?,?,?), 'prim_mod'(?,?,?,-,?).
'prim_mod'(A,B,R,E0,E) :- R is A mod B, E0=E.

?- block 'prim_<'(-,?,?,?,?), 'prim_<'(?,-,?,?,?), 'prim_<'(?,?,?,-,?).
'prim_<'(A,B,R,E0,E) :- (A<B -> R='True' ; R='False'), E0=E.

?- block 'prim_<='(-,?,?,?,?), 'prim_<='(?,-,?,?,?), 'prim_<='(?,?,?,-,?).
'prim_<='(A,B,R,E0,E) :- (A=<B -> R='True' ; R='False'), E0=E.

?- block 'prim_>'(-,?,?,?,?), 'prim_>'(?,-,?,?,?), 'prim_>'(?,?,?,-,?).
'prim_>'(A,B,R,E0,E) :- (A>B -> R='True' ; R='False'), E0=E.

?- block 'prim_>='(-,?,?,?,?), 'prim_>='(?,-,?,?,?), 'prim_>='(?,?,?,-,?).
'prim_>='(A,B,R,E0,E) :- (A>=B -> R='True' ; R='False'), E0=E.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Definition of conversion functions for characters:
%
?- block prim_ord(-,?,?,?), prim_ord(?,?,-,?).
prim_ord(C,N,E0,E) :- char_int(C,N), E0=E.

?- block prim_chr(-,?,?,?), prim_chr(?,?,-,?).
prim_chr(N,C,E0,E) :- N>=0, N<256, !, char_int(C,N), E0=E.
prim_chr(_,_,_,_) :-
	raise_exception('chr: argument out of range').


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Definition of I/O actions:
%
?- block 'prim_>>'(-,?,?,?,?,?), 'prim_>>'(?,?,?,?,-,?).
'prim_>>'(Action1,Action2,W,R,E0,E) :-
        apply(Action1,W,_,E0,E1),
	hnf(Action2,HAction2,E1,E2),
	apply(HAction2,W,R,E2,E).

?- block 'prim_>>='(-,?,?,?,?,?), 'prim_>>='(?,?,?,?,-,?).
'prim_>>='(Action1,FunAction2,W,R,E0,E) :-
        apply(Action1,W,'$io'(R1),E0,E1),
	hnf(FunAction2,HFunAction2,E1,E2),
	apply(HFunAction2,R1,HAction2,E2,E3),
	apply(HAction2,W,R,E3,E).

?- block prim_putChar(-,?,?,?,?), prim_putChar(?,?,?,-,?).
prim_putChar(C,_,'$io'('()'),E0,E) :-
	char_int(C,N), put(N), flush_output, E0=E.

?- block prim_getChar(?,?,-,?).
prim_getChar(_,'$io'(C),E0,E) :- get0(N), char_int(C,N), E0=E.

?- block prim_return(?,?,?,-,?).
prim_return(A,_,'$io'(A),E,E).

?- block prim_readFile(-,?,?,?,?), prim_readFile(?,?,?,-,?).
prim_readFile(A,_,_,_,_) :-
	map2(char_int,A,As),
	isURL(As), !,
	append("readFile """,As,E1),
	append(E1,""": URLs no longer supported in readFile!",E2),
	atom_chars(EMsg,E2),
	raise_exception(EMsg).
prim_readFile(A,W,'$io'('readFileContents'(Stream,W)),E0,E) :-
	string2Atom(A,FName),
	open(FName,read,Stream), E0=E.

isURL(S) :- append("http://",_,S), !.
isURL(S) :- append("ftp://",_,S), !.

?- block 'prim_readFileContents'(?,?,?,-,?).
'prim_readFileContents'(Stream,_,Result,E0,E) :-
	at_end_of_stream(Stream), !,
	Result=[],
	close(Stream), E0=E.
'prim_readFileContents'(Stream,W,Result,E0,E) :-
	get0(Stream,NChar),
	char_int(Char,NChar),
	Result=[Char|'readFileContents'(Stream,W)], E0=E.

?- block prim_writeFile(-,?,?,?,?,?), prim_writeFile(?,?,?,?,-,?).
prim_writeFile(A,S,W,H,E0,E) :-
	string2Atom(A,FName),
	open(FName,write,Stream),
	hnf('writeFileContents'(Stream,S,W),H,E0,E).	

?- block prim_appendFile(-,?,?,?,?,?), prim_appendFile(?,?,?,?,-,?).
prim_appendFile(A,S,W,H,E0,E) :-
	string2Atom(A,FName),
	open(FName,append,Stream),
	hnf('writeFileContents'(Stream,S,W),H,E0,E).	

?- block 'prim_writeFileContents'(?,-,?,?,-,?),
         'prim_writeFileContents'(?,?,?,?,-,?).
'prim_writeFileContents'(Stream,[],_,'$io'('()'),E0,E) :-
	flush_output(Stream), close(Stream), E0=E.
'prim_writeFileContents'(Stream,[C|Cs],W,R,E0,E) :-
	hnf(C,HC,E0,E1),
	'eval_writeFileContents'(Stream,HC,Cs,W,R,E1,E).
?- block 'eval_writeFileContents'(?,-,?,?,?,-,?),
         'eval_writeFileContents'(?,?,?,?,?,-,?).
'eval_writeFileContents'(Stream,C,Cs,W,H,E0,E) :-
	char_int(C,N), put(Stream,N),
	hnf('writeFileContents'(Stream,Cs,W),H,E0,E).


?- block prim_catchFail(?,?,?,?,-,?).
prim_catchFail(Action,_,W,R,E0,E) :-
	on_exception(ErrorMsg,
		     hnf(apply(Action,W),R,E0,E),
		     (print_error(ErrorMsg), fail)),
	!.
prim_catchFail(_,ErrAction,W,R,E0,E) :-
	hnf(apply(ErrAction,W),R,E0,E).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% definition of show primitive:
?- block prim_show(-,?,?,?), prim_show(?,?,-,?).
prim_show(T,String,E0,E) :-
	show_term(T,unqualified,String,[]), % new ReadShowTerm.showTerm
	E0=E.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% definition of seq primitive (rigid on first argument):
?- block prim_seq(-,?,?,?,?), prim_seq(?,?,?,-,?).
prim_seq(_,Val,Result,E0,E) :-
	hnf(Val,Result,E0,E).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% definition of runtime error:
?- block prim_error(-,?,?,?), prim_error(?,?,-,?).
prim_error(Msg,_,E,E) :-
	string2Atom(Msg,AMsg),
	raise_exception(AMsg).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% committed choice not yet implemented in Curry2Prolog:
?- block 'prim_commit'(?,?,-,?).
'prim_commit'(_,_,E,E) :-
        raise_exception('prelude.commit not yet implemented!').


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% encapsulated search not yet implemented in Curry2Prolog:
?- block 'prim_try'(?,?,-,?).
'prim_try'(_,_,E,E) :-
        raise_exception('prelude.try not yet implemented!').


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% hack for implementing Curry's findall:
%
% Warning: in contrast to Curry's definition, this evaluation
% of findall is suspended until it contains no global variables
% (whereas according to Curry, findall is suspended only if
% it tries to bind a global variable).
% Moreover, it is strict, i.e., it evaluates always all solutions!

:- block prim_findall(?,?,-,?).
prim_findall(SG,Sols,E0,E) :- 
	waitGround(SG,E0,E1), prim_findall_exec(SG,Sols,E1,E).

:- block prim_findall_exec(?,?,-,?).
prim_findall_exec(SG,Sols,E0,E) :- 
	hasPrintedFailure
	 -> findall(X,apply(SG,X,success,E0,_),Sols), E0=E
	  ; asserta(hasPrintedFailure),
	    findall(X,apply(SG,X,success,E0,_),Sols),
	    retract(hasPrintedFailure), E0=E.

:- block waitGround(-,?,?), waitGround(?,-,?).
waitGround(share(M),E0,E) :-
	!,
	get_mutable(V,M),
	(V='$eval'(Exp) -> true ; Exp=V),
	waitGround(Exp,E0,E).
waitGround(T,E0,E) :- functor(T,_,N), waitGroundArgs(1,N,T,E0,E).

:- block waitGroundArgs(?,?,?,-,?).
waitGroundArgs(A,N,_,E0,E) :- A>N, !, E0=E.
waitGroundArgs(A,N,T,E0,E) :-
	arg(A,T,Arg), waitGround(Arg,E0,E1),
	A1 is A+1, waitGroundArgs(A1,N,T,E1,E).


% since the above implementation of findall is strict,
% we offer also findfirst which only evaluates the first solution:

:- block prim_findfirst(?,?,-,?).
prim_findfirst(SG,Sol,E0,E) :-
	waitGround(SG,E0,E1), prim_findfirst_exec(SG,Sol,E1,E).

:- block prim_findfirst_exec(?,?,-,?).
prim_findfirst_exec(SG,Sol,E0,E) :-
	hasPrintedFailure
	 -> prim_findfirstWithPF(SG,Sol,E0), E0=E
	  ; asserta(hasPrintedFailure),
	    prim_findfirstWithoutPF(SG,Sol,E0), E0=E.

prim_findfirstWithPF(SG,Sol,E) :-
	apply(SG,X,success,E,_), !, Sol=X.

prim_findfirstWithoutPF(SG,Sol,E) :-
	apply(SG,X,success,E,_), retract(hasPrintedFailure), !, Sol=X.
prim_findfirstWithoutPF(_,_,_) :-
	retract(hasPrintedFailure), fail.


% We provide a similar implementation of AllSolutions.getFirstSolution:

:- block 'prim_AllSolutions.getFirstSolution'(?,?,-,?).
'prim_AllSolutions.getFirstSolution'(SG,_,Sol,E0,E) :-
	waitGround(SG,E0,E1),
	prim_getFirstSol_exec(SG,Sol,E1,E).

:- block prim_getFirstSol_exec(?,?,-,?).
prim_getFirstSol_exec(SG,Sol,E0,E) :-
	hasPrintedFailure
	 -> prim_getFirstSolWithPF(SG,Sol,E0,E)
	  ; asserta(hasPrintedFailure),
	    prim_getFirstSolWithoutPF(SG,Sol,E0,E).

prim_getFirstSolWithPF(SG,Sol,E0,E) :-
	apply(SG,X,success,E0,E1), !, Sol='$io'('Just'(X)), E1=E.
prim_getFirstSolWithPF(_,'$io'('Nothing'),E,E).

prim_getFirstSolWithoutPF(SG,Sol,E0,E) :-
	apply(SG,X,success,E0,E1), retract(hasPrintedFailure), !,
	Sol='$io'('Just'(X)), E1=E.
prim_getFirstSolWithoutPF(_,'$io'('Nothing'),E0,E) :-
	retract(hasPrintedFailure), E0=E.
