%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Definitions of builtins of module System:
%

:- use_module(library(system)).

?- block 'prim_System.getCPUTime'(?,?,-,?).
'prim_System.getCPUTime'(_,'$io'(MS),E0,E) :-
	statistics(runtime,[MS,_]), E0=E.

?- block 'prim_System.getElapsedTime'(?,?,-,?).
'prim_System.getElapsedTime'(_,'$io'(MS),E0,E) :-
	statistics(walltime,[MS,_]), E0=E.

?- block 'prim_System.getDate'(?,?,-,?).
'prim_System.getDate'(_,'$io'('System.Date'(Y,M,D,H,Min,S)),E0,E) :-
        datime(datime(Y,M,D,H,Min,S)), E0=E.

?- block 'prim_System.getArgs'(?,?,-,?).
'prim_System.getArgs'(_,'$io'(StringArgs),E0,E) :-
        (rtargs(Args) -> true ; prolog_flag(argv,Args,Args)),
        map2(atom2String,Args,StringArgs), E0=E.

?- block 'prim_System.getEnviron'(?,?,?,-,?).
'prim_System.getEnviron'(Var,_,'$io'(Value),E0,E) :-
	string2Atom(Var,AtomVar),
	(environ(AtomVar,AtomValue) -> atom2String(AtomValue,Value)
	                             ; Value = []), % empty string if undefined
	E0=E.

?- block 'prim_System.getHostname'(?,?,-,?).
'prim_System.getHostname'(_,'$io'(String),E0,E) :-
        host_name(Name),
        atom2String(Name,String), E0=E.

?- block 'prim_System.getPID'(?,?,-,?).
'prim_System.getPID'(_,'$io'(Pid),E0,E) :- pid(Pid), E0=E.

?- block 'prim_System.getProgName'(?,?,-,?).
'prim_System.getProgName'(_,'$io'(String),E0,E) :-
        modulename(Name),
        atom2String(Name,String), E0=E.

?- block 'prim_System.system'(?,?,?,-,?).
'prim_System.system'(S,_,'$io'(Status),E0,E) :-
	string2Atom(S,Cmd),
	system(Cmd,Status), E0=E.

?- block 'prim_System.sleep'(?,?,?,-,?).
'prim_System.sleep'(S,_,'$io'('()'),E0,E) :- sleep(S), E0=E.

% evaluate the argument to normal form and print the evaluation time:
?- block 'prim_System.evalTime'(?,?,-,?).
'prim_System.evalTime'(T,NF,E0,E) :-
	statistics(runtime,[RTime1,_]),
	statistics(walltime,[ETime1,_]),
	statistics(garbage_collection,[GC1,_,_]),
	nf(T,NF,E0,E1),
	statistics(runtime,[RTime2,_]),
	statistics(walltime,[ETime2,_]),
	statistics(garbage_collection,[GC2,_,_]),
	write(user_error,'EVALTIME: Runtime: '),
	RTime is RTime2-RTime1, write(user_error,RTime),
	write(user_error,' msec.'), nl(user_error),
	write(user_error,'EVALTIME: Elapsed time: '),
	ETime is ETime2-ETime1, write(user_error,ETime),
	write(user_error,' msec.'), nl(user_error),
	write(user_error,'EVALTIME: Number of garbage collections: '),
	GC is GC2-GC1, write(user_error,GC), nl(user_error),
	prolog_flag(gc,_,on), E1=E.

% evaluate the argument to normal form (without garbage collection)
% and print the evaluation time and memory usage:
?- block 'prim_System.evalSpace'(?,?,-,?).
'prim_System.evalSpace'(T,NF,E0,E) :-
	garbage_collect,
	prolog_flag(gc,_,off),
	statistics(runtime,[RTime1,_]),
	statistics(walltime,[ETime1,_]),
	statistics(garbage_collection,[GC1,_,_]),
	statistics(global_stack,[GSU1,_]),
	statistics(local_stack,[LSU1,_]),
	nf(T,NF,E0,E1),
	statistics(runtime,[RTime2,_]),
	statistics(walltime,[ETime2,_]),
	statistics(garbage_collection,[GC2,_,_]),
	statistics(global_stack,[GSU2,_]),
	statistics(local_stack,[LSU2,_]),
	write(user_error,'EVALTIME: Runtime: '),
	RTime is RTime2-RTime1, write(user_error,RTime),
	write(user_error,' msec.'), nl(user_error),
	write(user_error,'EVALTIME: Elapsed time: '),
	ETime is ETime2-ETime1, write(user_error,ETime),
	write(user_error,' msec.'), nl(user_error),
	write(user_error,'EVALTIME: Number of garbage collections: '),
	GC is GC2-GC1, write(user_error,GC), nl(user_error),
	write(user_error,'EVALTIME: Global stack usage: '),
	GS is GSU2-GSU1, write(user_error,GS), nl(user_error),
	write(user_error,'EVALTIME: Local stack usage: '),
	LS is LSU2-LSU1, write(user_error,LS), nl(user_error),
	prolog_flag(gc,_,on), E1=E.
