%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Reading .fcy file and translate it into internal old FLC format

:- use_module(basics).
:- use_module(prim_readshowterm). % for term en/decoding

readFlcFromFcy(FileName,FlatProg) :-
	(pakcsrc(showfcyload,yes)
	 -> write(user_error,'>>> Reading '),
	    write(user_error,FileName), write(user_error,' ... '),
	    statistics(runtime,[RT1,_]) ; true),
	open(FileName,read,Stream),
	readFcyStream(Stream,FcyPrologString), !,
	readTerm(FcyPrologString,unchecked,Tail,Term),
	skipWhiteSpace(Tail,[]),
	(pakcsrc(showfcyload,yes)
	 -> statistics(runtime,[RT2,_]),
	    RT is RT2-RT1,
	    write(user_error,RT), write(user_error,' ms.'), nl(user_error)
	  ; true),
	fcy2flcProg(Term,FlatProg).

readFcyStream(Stream,[]) :-
	at_end_of_stream(Stream), !,
	close(Stream).
readFcyStream(Stream,[NChar|NChars]) :-
	get0(Stream,NChar),
	readFcyStream(Stream,NChars).


fcy2flcProg('Prog'(ModName,Imps,Types,Funcs,Ops),
	    'Prog'(FModName,FImps,FTypes,FFuncs,FOps,Trans)) :-
        cp_string(ModName,FModName),
        map2(cp_string,Imps,FImps),
	fcy2flcTypes(Types,FTypes),
	map2(fcy2flcFunc,Funcs,FFuncs),
	map2(fcy2flcOp,Ops,FOps),
	fcyGetPublicTypes(Types,PTNames),
	fcyGetPublicFuncs(Funcs,PFNames),
	append(PTNames,PFNames,PNames),
	map2(fcy2flcTrans,PNames,Trans).

fcy2flcTypes([],[]).
fcy2flcTypes(['Type'(TName,_,TParams,Cons)|Types],
	     ['Type'(FTName,TParams,FCons)|FTypes]) :-
	fcy2flcQName(TName,FTName),
	map2(fcy2flcCons,Cons,FCons),
	fcy2flcTypes(Types,FTypes).
fcy2flcTypes(['TypeSyn'(_,_,_,_)|Types],FTypes) :- % ignore type synonyms
	fcy2flcTypes(Types,FTypes).

fcy2flcCons('Cons'(CName,Arity,_,Types),'Cons'(FCName,Arity,FTypes)) :-
	fcy2flcQName(CName,FCName),
	map2(fcy2flcTypeExpr,Types,FTypes).

fcy2flcTypeExpr('TVar'(I),'TVar'(I)).
fcy2flcTypeExpr('FuncType'(T1,T2),'FuncType'(FT1,FT2)) :-
	fcy2flcTypeExpr(T1,FT1),
	fcy2flcTypeExpr(T2,FT2).
fcy2flcTypeExpr('TCons'(TName,TEs),'TCons'(FTName,FTEs)) :-
	fcy2flcQName(TName,FTName),
	map2(fcy2flcTypeExpr,TEs,FTEs).

fcyGetPublicTypes([],[]).
fcyGetPublicTypes(['Type'(TName,'Public',_,Cons)|Types],PNames) :- !,
	fcyGetPublicConstrs(Cons,CNames),
	fcyGetPublicTypes(Types,TNames),
	append([TName|CNames],TNames,PNames).
fcyGetPublicTypes(['Type'(_,_,_,Cons)|Types],PNames) :- !,
	fcyGetPublicConstrs(Cons,CNames),
	fcyGetPublicTypes(Types,TNames),
	append(CNames,TNames,PNames).
fcyGetPublicTypes(['TypeSyn'(TName,'Public',_,_)|Types],[TName|TNames]) :- !,
	fcyGetPublicTypes(Types,TNames).
fcyGetPublicTypes([_|Types],TNames) :-
	fcyGetPublicTypes(Types,TNames).

fcyGetPublicConstrs([],[]).
fcyGetPublicConstrs(['Cons'(CName,_,'Public',_)|Cs],[CName|CNames]) :- !,
	fcyGetPublicConstrs(Cs,CNames).
fcyGetPublicConstrs([_|Cs],CNames) :-
	fcyGetPublicConstrs(Cs,CNames).

fcyGetPublicFuncs([],[]).
fcyGetPublicFuncs(['Func'(FName,_,'Public',_,_)|Funcs],[FName|FNames]) :-
	!, fcyGetPublicFuncs(Funcs,FNames).
fcyGetPublicFuncs([_|Funcs],FNames) :-
	fcyGetPublicFuncs(Funcs,FNames).

fcy2flcFunc('Func'(FName,Arity,_,Type,Rule),
	    'Func'(FFName,Arity,FType,FRule)) :-
	fcy2flcQName(FName,FFName),
	fcy2flcTypeExpr(Type,FType),
	fcy2flcTypeRule(Rule,FRule).

fcy2flcTypeRule('External'(N),'External'(FN)) :-
	cp_string(N,FN).
fcy2flcTypeRule('Rule'(Args,Exp),'Rule'(Args,FExp)) :-
	fcy2flcExpr(Exp,FExp).

fcy2flcLit('Intc'(I),'Intc'(I)).
fcy2flcLit('Floatc'(F),'Floatc'(F)).
fcy2flcLit('Charc'(C),'Charc'(I)) :- char_int(C,I).

fcy2flcExpr('Var'(I),'Var'(I)).
fcy2flcExpr('Lit'(L),'Lit'(FL)) :- fcy2flcLit(L,FL).
fcy2flcExpr('Comb'('FuncCall',QName,[E]),'Choice'(FE)) :-
	fcy2flcQName(QName,"commit"), !,
	fcy2flcExpr(E,FE).
fcy2flcExpr('Comb'('FuncCall',QName,[E1,E2]),'Apply'(FE1,FE2)) :-
	fcy2flcQName(QName,"apply"), !,
	fcy2flcExpr(E1,FE1),
	fcy2flcExpr(E2,FE2).
fcy2flcExpr('Comb'('FuncCall',QName,[E1,E2]),
	    'GuardedExpr'([],'Constr'([],FE1),FE2)) :-
	fcy2flcQName(QName,"cond"), !,
	fcy2flcExpr(E1,FE1),
	fcy2flcExpr(E2,FE2).
fcy2flcExpr('Free'(Vars,'Comb'('FuncCall',QName,[E1,E2])),
	    'GuardedExpr'(Vars,'Constr'([],FE1),FE2)) :-
	fcy2flcQName(QName,"cond"), !,
	fcy2flcExpr(E1,FE1),
	fcy2flcExpr(E2,FE2).
fcy2flcExpr('Comb'(CT,QName,Es),'Comb'(FCT,FQName,FEs)) :-
	fcy2flcQName(QName,FQName),
	fcy2flcCombType(CT,FCT),
	map2(fcy2flcExpr,Es,FEs).
fcy2flcExpr('Free'(Vars,Exp),'Constr'(Vars,FExp)) :-
	fcy2flcExpr(Exp,FExp).
fcy2flcExpr('Or'(E1,E2),'Or'(FE1,FE2)) :-
	fcy2flcExpr(E1,FE1),
	fcy2flcExpr(E2,FE2).
fcy2flcExpr('Case'(CT,Exp,Branches),'Case'(CT,FExp,FBranches)) :-
	fcy2flcExpr(Exp,FExp),
	map2(fcy2flcBranch,Branches,FBranches).

fcy2flcCombType('FuncCall','FuncCall').
fcy2flcCombType('ConsCall','ConsCall').
fcy2flcCombType('PartCall'(_),'PartCall').

fcy2flcBranch('Branch'('LPattern'(L),Exp),'Branch'('LPattern'(FL),FExp)) :-
	fcy2flcLit(L,FL), fcy2flcExpr(Exp,FExp).
fcy2flcBranch('Branch'('Pattern'(QName,Vars),Exp),
	      'Branch'('Pattern'(FQName,Vars),FExp)) :-
	fcy2flcQName(QName,FQName),
	fcy2flcExpr(Exp,FExp).


fcy2flcOp('Op'(OName,Fix,Int),'Op'(FOName,Fix,Int)) :-
	fcy2flcQName(OName,FOName).

fcy2flcTrans('(,)'(Mod,Name),'Trans'(FName,FName)) :-
	cp_string(Mod,"prelude"), !,
	cp_string(Name,FName).
fcy2flcTrans('(,)'(Mod,Name),'Trans'(FName,FMName)) :-
	cp_string(Mod,FMod),
	cp_string(Name,FName),
	append(FMod,[46|FName],FMName).

fcy2flcQName('(,)'(Mod,Name),FName) :-
	cp_string(Mod,"prelude"), !,
	cp_string(Name,FName).
fcy2flcQName('(,)'(Mod,Name),FMName) :-
	cp_string(Mod,FMod),
	cp_string(Name,FName),
	append(FMod,[46|FName],FMName).






