% Definition of a predicate to store a saved state
% together with an entry goal for starting the state

% Due to incompatibilities between the various versions of
% Sicstus-Prolog, we have to use different methods to save states etc.

:- use_module(library(system)).

% get Sicstus version (e.g., '3.9'):
% Currently, we support only 3.5, 3.7, 3.8, 3.9, 3.10, 3.11:
getSicstusVersion(SV) :-
	prolog_flag(version,V,V),
	atom_chars(V,Vs),
	(append("SICStus 3.5",_,Vs) -> SV='3.5' ;
	 append("SICStus 3.7",_,Vs) -> SV='3.7' ;
	 append("SICStus 3.8",_,Vs) -> SV='3.8' ;
	 append("SICStus 3.9",_,Vs) -> SV='3.9' ;
	 append("SICStus 3.10",_,Vs) -> SV='3.10' ;
	 append("SICStus 3.11",_,Vs) -> SV='3.11' ;
	 write(user_error,'ERROR: UNKNOWN SICSTUS PROLOG VERSION:'),
	 nl(user_error),
	 write(user_error,'Please modify pakcs/curry2prolog/saveprog.pl'),
	 nl(user_error), halt(1)).

sicstus37orHigher :-
	getSicstusVersion(SV),
	(SV = '3.7' ; sicstus38orHigher).
sicstus38orHigher :-
	getSicstusVersion(SV),
	(SV = '3.8' ; SV = '3.9' ; sicstus310orHigher).
sicstus310orHigher :-
	getSicstusVersion(SV),
	(SV = '3.10' ; SV = '3.11').

% turn off discontiguous clauses warnings:
:- sicstus38orHigher ->	prolog_flag(discontiguous_warnings ,_,off) ; true.

saveprog_entry(State,Entry) :-
	pakcsrc(standalone,yes), !,
	save_standalone_executable(State,Entry).
saveprog_entry(State,Entry) :-
	getSicstusVersion(SV), SV = '3.5',
	!, % if we use Sicstus 3.5
	save(State), Entry.
saveprog_entry(State,Entry) :-
	sicstus37orHigher, !,
	save_program(State,Entry).   % if we use Sicstus 3.7 or higher
% Note for Sicstus 3.9 and higher:
% It works only if the name of the state contains a suffix with a dot,
% like 'xxx.state', otherwise Sicstus adds automatically
% the suffix '.sav'!

% try to save a predicate in a .po file if it is supported by this
% Sicstus version:
try_save_predicates(P/N,POFile) :-
	sicstus38orHigher, !,
	save_predicates(P/N,POFile).
try_save_predicates(_,_).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% for stand-alone executables:
:- dynamic rt_entry/1.

% use this instead of saveprog_entry:
save_standalone_executable(State,Entry) :-
	(sicstus310orHigher -> true
	 ; write(user_error,'ERROR: stand-alone executables require Sictus 3.10 or higher!'),
	   nl(user_error), fail),
	(retract(rt_entry(_)) -> true ; true),
	asserta(rt_entry(Entry)),
	save_program(State).

runtime_entry(start) :-
	rt_entry(Entry),
	call(Entry).
