------------------------------------------------------------------------------
--- A collection of operations on floating point numbers.
------------------------------------------------------------------------------

module Float where

-- The operator declarations are similar to the standard arithmetic operators.

infixl 7 *., /.
infixl 6 +., -.
infix  4 <., >., <=., >=.


--- Addition on floats.

(+.)   :: Float -> Float -> Float
(+.) external

--- Subtraction on floats.

(-.)   :: Float -> Float -> Float
(-.) external

--- Multiplication on floats.

(*.)   :: Float -> Float -> Float
(*.) external

--- Division on floats.

(/.)   :: Float -> Float -> Float
(/.) external

--- Predicate "less than" on floats.

(<.)   :: Float -> Float -> Bool
(<.) external

--- Predicate "greater than" on floats.

(>.)   :: Float -> Float -> Bool
(>.) external

--- Predicate "less than or equal" on floats.

(<=.)  :: Float -> Float -> Bool
(<=.) external

--- Predicate "greater than or equal" on floats.

(>=.)  :: Float -> Float -> Bool
(>=.) external

--- Conversion function from integers to floats.

i2f    :: Int -> Float
i2f external

--- Conversion function from floats to integers.
--- The result is the closest integer between the argument and 0.

truncate :: Float -> Int
truncate external

--- Conversion function from floats to integers.
--- The result is the nearest integer to the argument.
--- If the argument is equidistant between two integers,
--- it is rounded to the closest even integer value.

round :: Float -> Int
round external

--- Square root.

sqrt :: Float -> Float
sqrt external

--- Natural logarithm.

log :: Float -> Float
log external

--- Natural exponent.

exp :: Float -> Float
exp external

--- Sine.

sin :: Float -> Float
sin external

--- Cosine.

cos :: Float -> Float
cos external

--- Tangent.

tan :: Float -> Float
tan external

