-----------------------------------------------------------------------------
--- Library for IO operations like reading and writing files
--- that are not already contained in the prelude.
---
--- @author Michael Hanus
--- @version August 2004
-----------------------------------------------------------------------------

module IO(Handle,IOMode(..),stdin,stdout,stderr,
          openFile,hClose,hFlush,hIsEOF,isEOF,
          hGetChar,hGetLine,hPutChar,hPutStr,hPutStrLn,hPrint,
          hIsReadable,hIsWritable) where

data Handle = Int -- internally defined


data IOMode = ReadMode | WriteMode | AppendMode

--- Standard input stream.
stdin :: Handle
stdin external

--- Standard output stream.
stdout :: Handle
stdout external

--- Standard error stream.
stderr :: Handle
stderr external

--- Opens a file in specified mode and returns a handle to it.
openFile :: String -> IOMode -> IO Handle
openFile external

--- Closes a file handle and flushes the buffer in case of output file.
hClose :: Handle -> IO ()
hClose external

--- Flushes the buffer associated to handle in case of output file.
hFlush :: Handle -> IO ()
hFlush external

--- Is handle at end of file?
hIsEOF :: Handle -> IO Bool
hIsEOF external

--- Is standard input at end of file?
isEOF :: IO Bool
isEOF = hIsEOF stdin

--- Reads a character from an input handle and returns it.
hGetChar    :: Handle -> IO Char
hGetChar external

--- Reads a line from an input handle and returns it.
hGetLine  :: Handle -> IO String
hGetLine h = do c <- hGetChar h
                if c=='\n' then return ""
                           else do cs <- hGetLine h
                                   return (c:cs)

--- Puts a character to an output handle.
hPutChar    :: Handle -> Char -> IO ()
hPutChar external

--- Puts a string to an output handle.
hPutStr :: Handle -> String -> IO ()
hPutStr _ []     = done
hPutStr h (c:cs) = hPutChar h c >> hPutStr h cs

--- Puts a string with a newline to an output handle.
hPutStrLn :: Handle -> String -> IO ()
hPutStrLn h s = hPutStr h s >> hPutChar h '\n'

--- Converts a term into a string and puts it to an output handle.
hPrint :: Handle -> _ -> IO ()
hPrint h = hPutStrLn h . show


--- Is the handle readable?
hIsReadable :: Handle -> IO Bool
hIsReadable external

--- Is the handle writable?
hIsWritable :: Handle -> IO Bool
hIsWritable external
